#===========================================================================
#  FileName: VerifyFunctions.ksh
#    Author: Craig Schneider
# Orig Date: 06/14/00
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Some standard functions for performing verifications.
#===========================================================================

if [ -f VerifyEnv.ksh ] ; then
   . VerifyEnv.ksh
fi

# put these into the environment so that VerifyEverything.ksh
# knows how to look for passed and failed verifications.

if [ "$TESTCASE_FORMAT" = psv ] ; then
   export VERIFY_PASSED="pass"
   export VERIFY_FAILED="@@ FAIL @@"
else
   export VERIFY_PASSED="Successfully passed"
   export VERIFY_FAILED="ERROR: Failed executing"
fi
export VERIFY_PASSING_LEVEL=PASS
export VERIFY_FAILING_LEVEL=FAIL

VERIFICATION_LOG=Verification.log
if [ -f $VERIFICATION_LOG ] ; then
   TESTCASE_NUMBER=`egrep "$VERIFY_PASSED|$VERIFY_FAILED" $VERIFICATION_LOG | \
                               wc -l | awk '{print $1}'`
else
   TESTCASE_NUMBER=0
fi

#===========================================================================
# Generate a test name based upon the filename being executed.
#===========================================================================
function TestNameFromFileName
{
   TESTCASE_FILENAME=${1%.ksh}
   TESTCASE_FILENAME="${TESTCASE_FILENAME##*/}"
   TESTCASE_FILENAME="${TESTCASE_FILENAME#v_}"
   TESTCASE_FILENAME="${TESTCASE_FILENAME#Verify*_}"   # eliminate this guy
   export TESTCASE_FILENAME="${TESTCASE_FILENAME#SubV*_}"
   echo "$TESTCASE_FILENAME"
}

if [ $# -ne 1 ] ; then
   echo "Warning: VerifyFunctions.ksh can not properly set TESTCASE_FILENAME."
   echo "Warning: Pass in \$0 as an argument when dotting VerifyFunctions.ksh."
   export TESTCASE_FILENAME='Unknown'
else
   export TESTCASE_FILENAME=$1
fi
export TEST_NAME=`TestNameFromFileName $TESTCASE_FILENAME`
export ORIG_TEST_NAME="$TEST_NAME"     # this is very helpful when looping

#===========================================================================
# Output the details of a passed test.  Call this function wherever
# possible so the output format can be easily modified in the future.
#===========================================================================
function VerifyPassed
{
   TESTCASE_NAME=$1
   TESTCASE_DESC=$2
   TESTCASE_NOTE=$3

   let TESTCASE_NUMBER=TESTCASE_NUMBER+1
   if [ "$TESTCASE_FORMAT" = psv ] ; then
      echo "$TESTCASE_NUMBER)|$TESTCASE_NAME|$TESTCASE_NOTE|$VERIFY_PASSED"
   else
      echo "$VERIFY_PASSED $TESTCASE_NAME $TESTCASE_NOTE"
   fi
}

#===========================================================================
# Output the details of a failed test.  Call this function wherever
# possible so the output format can be easily modified in the future.
#===========================================================================
function VerifyFailed
{
   TESTCASE_NAME=$1
   TESTCASE_DESC=$2
   TESTCASE_NOTE=$3

   let TESTCASE_NUMBER=TESTCASE_NUMBER+1
   if [ "$TESTCASE_FORMAT" = psv ] ; then
      echo "$TESTCASE_NUMBER)|$TESTCASE_NAME|$TESTCASE_NOTE|$VERIFY_FAILED"
   else
      echo "ERROR: Verification failed in file $TESTCASE_FILENAME" 
      echo "$VERIFY_FAILED $TESTCASE_NAME" 
      if [ "$TESTCASE_DESC" != "" ] ; then
         echo " DESC: $TESTCASE_DESC" 
      fi
      if [ "$TESTCASE_NOTE" != "" ] ; then
         echo " NOTE: $TESTCASE_NOTE" 
      fi
   fi
}

#===========================================================================
# Output the details of a failed test and prompt before continuing.
#===========================================================================
function VerifyFailedPrompt
{
   VerifyFailed "$1" "$2" "$3" "$4"
   if [ "$TESTCASE_ANSWER" = "" ] ; then
      echo -n "Please press enter to continue... "
      read ANSWER
   fi
}

#===========================================================================
# Compare the data within two strings.
#===========================================================================
function VerifyStrings
{
   ACTUAL_RESULTS_STRING=$1
   EXPECTED_RESULTS_STRING=$2
   TESTCASE_NAME=$3
   TESTCASE_DESC=$4
   TESTCASE_NOTE=$5
   TESTCASE_FAIL_EXPECTED=$6

   if [ "$ACTUAL_RESULTS_STRING" = "$EXPECTED_RESULTS_STRING" ] ; then
      VerifyPassed "$TESTCASE_NAME" "$TESTCASE_DESC" "$TESTCASE_NOTE"
      return 0
   else
      if [ "$TESTCASE_FAIL_EXPECTED" != "" ] ; then
         VerifyFailed "$TESTCASE_NAME" "$TESTCASE_DESC" "$TESTCASE_NOTE"
         return 1
      fi
      echo
      echo "ERROR: Actual   results ($ACTUAL_RESULTS_STRING)"
      echo "ERROR: Expected results ($EXPECTED_RESULTS_STRING)"
         
      VerifyFailed "$TESTCASE_NAME" "$TESTCASE_DESC" "$TESTCASE_NOTE"
      if [ "$TESTCASE_ANSWER" = "" ] ; then
         echo -n "Please press enter to continue... "
         read ANSWER
      fi
      return 1
   fi
}

#===========================================================================
# Compare the contents of two files.
#===========================================================================
function VerifyFiles
{
   RM_OPTS=
   if [ "$1" = "-r" ] ; then
      RM_OPTS="-r"
      shift
   fi

   ACTUAL_RESULTS_FILE=$1
   EXPECTED_RESULTS_FILE=$2
   TESTCASE_NAME=$3
   TESTCASE_DESC=$4
   TESTCASE_NOTE=$5

   if [ "$TESTCASE_NAME" == "" ] ; then
      TESTCASE_NAME=${EXPECTED_RESULTS_FILE#Output_}
   fi

   STABALIZE_ANSWER=
   if [ "$TESTCASE_ANSWER" = y -o "$TESTCASE_ANSWER" = Y ] ; then
      STABALIZE_ANSWER="-yes"
   elif [ "$TESTCASE_ANSWER" = n -o "$TESTCASE_ANSWER" = N ] ; then
      STABALIZE_ANSWER="-no_prompt"
   fi
   STABALIZE_TEXT=`VerifyFailed "$TESTCASE_NAME" "$TESTCASE_DESC" \
                                "$TESTCASE_NOTE"`
   src.ksh stabalize -c deduce $RM_OPTS -diff -diffs $STABALIZE_ANSWER \
                     -t Difference "$STABALIZE_TEXT" -t Unchanged "" \
                     -m "Updated by $LOGNAME on $(date '+%b %e, %Y')" \
                     "$ACTUAL_RESULTS_FILE" "$EXPECTED_RESULTS_FILE"
   RET_CODE=$?
   if [ "$RET_CODE" = 0 ] ; then
      VerifyPassed "$TESTCASE_NAME" "$TESTCASE_DESC" "$TESTCASE_NOTE"
   fi
   return $RET_CODE
}

#===========================================================================
# Compare the contents of a string with those in a file.
#===========================================================================
function VerifyStringFile
{
   ACTUAL_RESULTS_STRING=$1
   EXPECTED_RESULTS_FILE=$2
   TESTCASE_NAME=$3
   TESTCASE_DESC=$4
   TESTCASE_NOTE=$5

   echo "$ACTUAL_RESULTS_STRING" > VerifyStringFile.dat
   VerifyFiles VerifyStringFile.dat "$EXPECTED_RESULTS_FILE" \
               "$TESTCASE_NAME" "$TESTCASE_DESC" "$TESTCASE_NOTE"
   return $?
}

#===========================================================================
# Perform sed processing where the resulting file replaces the original.
#===========================================================================
function SameFileSed
{
   SED_FILE=$1
   SED_STRING=$2
   sed "$SED_STRING" $SED_FILE > $SED_FILE.fix
   mv $SED_FILE.fix $SED_FILE
}

#===========================================================================
# Determine our processing throughput in records/second.
# To generate the TIME_OUTPUT_FILE that this routine needs, use the time(1)
# function to run the program/script being timed and direct stderr to the
# filename that you pass into this routine.
#===========================================================================
function ComputeRecordsPerSecond
{
   RECORDS_PROCESSED=$1
   TIME_OUTPUT_FILE=$2
   CALCULATION_METHOD=$3

   case $CALCULATION_METHOD in
      REAL) TIME_TAG="real"
            ;;
      USER) TIME_TAG="user"
            ;;
       SYS) TIME_TAG="sys"
            ;;
         *) TIME_TAG="real"
            ;;
   esac

   CALC_TIME=`grep $TIME_TAG $TIME_OUTPUT_FILE | awk '{ print $2 }'`
   CALC_MINUTES=${CALC_TIME%m*}
   SECS_PLUS=${CALC_TIME#*m}
   CALC_SECONDS=${SECS_PLUS%.*}
   FRAC_SECONDS=${SECS_PLUS#*.}
   HUNDREDS_SECONDS=${FRAC_SECONDS%s}
   let TOTAL_HUNDREDS=CALC_MINUTES*6000+CALC_SECONDS*100+HUNDREDS_SECONDS
   let RECORDS_PER_SEC=RECORDS_PROCESSED*100/TOTAL_HUNDREDS
   rm -f $TIME_OUTPUT_FILE
   echo $RECORDS_PER_SEC
}
