#===========================================================================
#  Filename: 4db_extract.ksh
#    Author: Craig Schneider
# Orig Date: 02/14/02
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Use 4db to extract data from the specified table.
#===========================================================================

FOUR_DB_OPTIONS=
DATA_FILE_LIST=
TOTALS_FILE=
ZERO_FILES_OK=FALSE
MAX_LINE_LEN_OPTS=
STABALIZE_ARGS=
VERBOSE_MODE=FALSE
EXCLUDE_FILE_NAME="$EZ_ARCHIVE_ROOT/public/4db_extract_excludes"
ALLOW_DOS2UNIX=TRUE

OVERALL_FILES=0
OVERALL_RECS=0

function Usage
{
cat <<!

Usage: $0 [-s ServerName] [-d Database] [DataFileName ...] [-diffs]
          [-insert | -insertnl | -replace |
           -upsert | upsertnl | -upsertnlrs | -upsertrs]
          [-i SingleDataFileName] [-no] [-yes] [-u UserName] [-p Password]
          [-quiet] [-t TotalsFileName] [-verbose] [-0] [-x] [-z MaxLineLength]
          [-no_dos2unix]

      -diffs: Display differences of files before the user is prompted.
         -no: All user questions will be implicitly answered with a no.
      -quiet: Suppress non critical messages about operations being performed.
    -verbose: Display verbose messages about operations being performed.
        -yes: All user questions will be implicitly answered with a yes.
!
   exit 1
}

SPECIFIED_FORMAT=-deduce    # Default if file does not exist is -upsertnlrs
while [ "$#" -ne 0 ]
do
  case $1 in
    -0) ZERO_FILES_OK=TRUE
        shift;;
    -d) FOUR_DB_OPTIONS="$FOUR_DB_OPTIONS -d $2"
        shift
        shift;;
-diffs) STABALIZE_ARGS="$STABALIZE_ARGS -diffs"
        shift;;
-insert) SPECIFIED_FORMAT="-insert"
        shift ;;
-insertnl) SPECIFIED_FORMAT="-insertnl"
        shift ;;
    -i) DATA_FILE_LIST="$DATA_FILE_LIST $2"
        shift
        shift ;;
    -j) FOUR_DB_OPTIONS="$FOUR_DB_OPTIONS -j $2"
        shift
        shift;;
   -no) STABALIZE_ARGS="$STABALIZE_ARGS -no"
        shift;;
-no_dos2unix) ALLOW_DOS2UNIX=FALSE
        shift;;
    -p) FOUR_DB_OPTIONS="$FOUR_DB_OPTIONS -p $2"
        shift
        shift;;
-quiet) STABALIZE_ARGS="$STABALIZE_ARGS -quiet"
        shift;;
-replace) SPECIFIED_FORMAT="-replace"
        shift ;;
-string) FOUR_DB_OPTIONS="$FOUR_DB_OPTIONS -string $2 $3"
        shift
        shift
        shift;;
    -s) FOUR_DB_OPTIONS="$FOUR_DB_OPTIONS -s $2"
        shift
        shift;;
    -t) TOTALS_FILE="$2"
        shift
        shift;;
-upsert) SPECIFIED_FORMAT="-upsert"
        shift ;;
-upsertnl) SPECIFIED_FORMAT="-upsertnl"
        shift ;;
-upsertnlrs) SPECIFIED_FORMAT="-upsertnlrs"
        shift ;;
-upsertrs) SPECIFIED_FORMAT="-upsertrs"
        shift ;;
    -u) FOUR_DB_OPTIONS="$FOUR_DB_OPTIONS -u $2"
        shift
        shift;;
-verbose) FOUR_DB_OPTIONS="$FOUR_DB_OPTIONS -verbose"
        VERBOSE_MODE=TRUE
        shift;;
    -x) STABALIZE_ARGS="$STABALIZE_ARGS -e $EXCLUDE_FILE_NAME"
        shift;;
  -yes) STABALIZE_ARGS="$STABALIZE_ARGS -yes"
        shift;;
    -z) MAX_LINE_LEN_OPTS="-z $2"
        shift
        shift;;
    -?) Usage ;;
     *) DATA_FILE_LIST="$DATA_FILE_LIST $1"
        shift ;;
  esac
done

if [ "$DATA_FILE_LIST" = "" ] ; then
   DATA_FILE_LIST=`find . \( -name \*.4db -o -name \*.exp \) -print`
fi
if [ "$DATA_FILE_LIST" = "" ] ; then
   if [ "$ZERO_FILES_OK" = TRUE ] ; then
      exit 0
   fi
   Usage
fi

function EZGetFileRecordCount
{
   FILE_NAME="$1"
   UNFIX_NAME="${FILE_NAME%.fix}"
   EXTENSION="${UNFIX_NAME##*.}"

   if [ "$EXTENSION" = "exp" ] ; then
      LINE_COUNT=`wc -l "$FILE_NAME" | awk '{print $1}'`
      let LINE_COUNT=LINE_COUNT-1
   else
      # need to subtract 2 alters and 1 truncate
      LINE_COUNT=`cut -c 1-50 "$FILE_NAME" | \
                  egrep "^UPSERT|^INSERT|^UPDATE" | \
                  wc -l | awk '{print $1}'`
   fi
   echo "$LINE_COUNT"
}

ORIG_DIR=$PWD

for DATA_FILE_PATH in $DATA_FILE_LIST
do
   cd $ORIG_DIR
   DATA_DIR=${DATA_FILE_PATH%/*}
   DATA_FILE=${DATA_FILE_PATH##*/}
   if [ "${DATA_FILE#my_}" != "$DATA_FILE" ] ; then
      continue
   fi
   IGNORE_DATA_FILE="${DATA_FILE%.4db}.ignore"
   if [ -f "$IGNORE_DATA_FILE" ] ; then
      echo "Successfully ignoring $DATA_FILE."
      continue
   fi
   if [ "$DATA_DIR" != "$DATA_FILE" ] ; then
      cd $DATA_DIR
   fi

   EXTRACT_EXCLUDE_COLUMNS="UPDATE_COUNT,UPDATE_TIMESTAMP,UPDATE_SOURCE,UPDATE_DETAILS,UPDATE_DATA_CHANGE_STATE"
   EXCLUDE_DATA_FILE="${DATA_FILE%.4db}.exclude"
   if [ -f "$EXCLUDE_DATA_FILE" ] ; then
      EXTRA_EXCLUDE=`cat $EXCLUDE_DATA_FILE`
      EXTRACT_EXCLUDE_COLUMNS="$EXTRACT_EXCLUDE_COLUMNS,$EXTRA_EXCLUDE"
   fi
   ORDER_BY_EXCLUDE_COLUMNS="$EXTRACT_EXCLUDE_COLUMNS"
   ORDER_BY_EXCLUDE_DATA_FILE="${DATA_FILE%.4db}.order_by_exclude"
   if [ -f "$ORDER_BY_EXCLUDE_DATA_FILE" ] ; then
      EXTRA_EXCLUDE=`cat $ORDER_BY_EXCLUDE_DATA_FILE`
      ORDER_BY_EXCLUDE_COLUMNS="$ORDER_BY_EXCLUDE_COLUMNS,$EXTRA_EXCLUDE"
   fi

   OPTIONS_DATA_FILE="${DATA_FILE%.4db}.options"
   if [ -f "$OPTIONS_DATA_FILE" ] ; then
      FOUR_DB_OPTIONS="$FOUR_DB_OPTIONS $(cat $OPTIONS_DATA_FILE)"
   fi

   EXTRACT_RECORD_SRC=""
   EXTRACT_WHERE_CLAUSE=""
   DATA_FILE_MINUS_4DB="${DATA_FILE%.4db}"
   EXTRACT_TYPE="${DATA_FILE_MINUS_4DB##*.}"
   EXTRACT_OBJECT_NAME="${DATA_FILE_MINUS_4DB%.$EXTRACT_TYPE}"
   # TODO: Get list of record sources from ExtractSrc/record_sources.dat
   if [ "$EXTRACT_TYPE" = c4 -o "$EXTRACT_TYPE" = np ] ; then
      EXTRACT_RECORD_SRC="$EXTRACT_TYPE"
      EXTRACT_WHERE_CLAUSE="where RECORD_SRC = '$EXTRACT_RECORD_SRC'"
      EXTRACT_TYPE="not .data.4db"  # can not be data
   else
      if [ "$EXTRACT_OBJECT_NAME" != "${EXTRACT_OBJECT_NAME%.*}" ] ; then
         EXTRACT_RECORD_SRC="${EXTRACT_OBJECT_NAME##*.}"
         EXTRACT_WHERE_CLAUSE="where RECORD_SRC = '$EXTRACT_RECORD_SRC'"
         EXTRACT_OBJECT_NAME="${EXTRACT_OBJECT_NAME%.*}"
      fi
   fi

   EXTRACT_FORMAT="$SPECIFIED_FORMAT"
   if [ "$EXTRACT_FORMAT" = "-deduce" ] ; then
      if [ "$EXTRACT_TYPE" = "exp" ] ; then
         EXTRACT_FORMAT="-CaretRaw"
      elif [ ! -f "$DATA_FILE" ] ; then
         if [ "$EXTRACT_RECORD_SRC" != "" ] ; then
            #EXTRACT_FORMAT="-upsertnlrs"
            EXTRACT_FORMAT="-upsertnl"
         else
            EXTRACT_FORMAT="-upsertnl"
         fi
      else
         egrep $EZ_EGREP_QUIET "^UPSERT" "$DATA_FILE"
         if [ $? -eq 0 ] ; then
            if [ "$EXTRACT_RECORD_SRC" != "" ] ; then
               if [ "$EXTRACT_OBJECT_NAME" = gfld_index      -o \
                    "$EXTRACT_OBJECT_NAME" = gfld_definition -o \
                    "$EXTRACT_OBJECT_NAME" = gfld_value_list  ] ; then
                  EXTRACT_FORMAT="-upsertnlrs"
               else
                  EXTRACT_FORMAT="-upsertnl"
               fi
            else
               EXTRACT_FORMAT="-upsertnl"
            fi
         else
            egrep $EZ_EGREP_QUIET "^ DELETE FROM " "$DATA_FILE"
            if [ $? -eq 0 ] ; then
               EXTRACT_FORMAT="-replace"
            else
               EXTRACT_FORMAT="-insertnl"
            fi
         fi
      fi
   fi

   EXTRACT_NEEDS_SORTING=FALSE
   EXTRACT_DATA_FILE="$DATA_FILE.fix"
   rm -f "$EXTRACT_DATA_FILE"

   RM_EXTRACT_SCRIPT_FILE=FALSE
   EXTRACT_SRC_DIRS=". $EZ_FAMILYTREE_ROOT/ExtractSrc $EZ_ENV_ROOT/ExtractSrc"
   EXTRACT_SCRIPT_FILE="Missing.Extract.File"
#echo "EXTRACT_OBJECT_NAME=$EXTRACT_OBJECT_NAME"
#echo "EXTRACT_TYPE=$EXTRACT_TYPE"
#echo "EXTRACT_RECORD_SRC=$EXTRACT_RECORD_SRC"
   MAP_EXTRACT_TYPE="$EXTRACT_TYPE"
   if [ "$EXTRACT_TYPE" = exp -a "$EXTRACT_RECORD_SRC" != "" ] ; then
      MAP_EXTRACT_TYPE="data"
   fi
   for DIR_PASS in ONE TWO
   do
      for EXTRACT_SRC_DIR in $EXTRACT_SRC_DIRS
      do
         if [ -f "$EXTRACT_SCRIPT_FILE" ] ; then
            break;
         fi
         if [ "$DIR_PASS" = TWO ] ; then
            EXTRACT_SCRIPT_FILE="$EXTRACT_SRC_DIR/$EXTRACT_OBJECT_NAME.extract.4db"
         else
            EXTRACT_SCRIPT_FILE="$EXTRACT_SRC_DIR/$MAP_EXTRACT_TYPE.extract.4db"
         fi
         #echo "Checking EXTRACT_SCRIPT_FILE=$EXTRACT_SCRIPT_FILE"
      done
   done
   #echo "Done.... EXTRACT_SCRIPT_FILE=$EXTRACT_SCRIPT_FILE"
   if [ ! -f "$EXTRACT_SCRIPT_FILE" ] ; then
      echo "ERROR: Failed to find extract.4db file $EXTRACT_SCRIPT_FILE."
      exit 1
#      EXTRACT_SCRIPT_FILE="$TMPDIR/4db.extract.$$"
#      cat > "$EXTRACT_SCRIPT_FILE" <<!
#select [[GetColumnListExcludingNL(ExtractObjectNameUpper,
#                                  ExtractExcludeColumns)]]
#from   [[ExtractObjectNameUpper]]
#[[ExtractWhereClause]]
#order by [[GetColumnListExcludingNL(ExtractObjectNameUpper,
#           OrderByExcludeColumns)]];
#!
#      RM_EXTRACT_SCRIPT_FILE=TRUE
   else
      FOUR_DB_OPTS_NO_URS="${FOUR_DB_OPTIONS#*string OverrideFormat UpsertRS}"
      if [ "$FOUR_DB_OPTS_NO_URS" = "$FOUR_DB_OPTIONS" -a \
           "$ALLOW_DOS2UNIX" = TRUE ] ; then
         STABALIZE_ARGS="$STABALIZE_ARGS -dos2unixcmp"
      fi
   fi

   typeset -u EXTRACT_OBJECT_NAME_UPPER
   EXTRACT_OBJECT_NAME_UPPER="$EXTRACT_OBJECT_NAME"
#echo "EXTRACT_SCRIPT_FILE=$EXTRACT_SCRIPT_FILE"
#echo "EXTRACT_DATA_FILE=$EXTRACT_DATA_FILE"
#echo "EXTRACT_EXCLUDE_COLUMNS=$EXTRACT_EXCLUDE_COLUMNS"
#echo "ORDER_BY_EXCLUDE_COLUMNS=$ORDER_BY_EXCLUDE_COLUMNS"
#echo "EXTRACT_FORMAT=$EXTRACT_FORMAT"
#echo "EXTRACT_OBJECT_NAME=$EXTRACT_OBJECT_NAME"
#echo "EXTRACT_OBJECT_NAME_UPPER=$EXTRACT_OBJECT_NAME_UPPER"
#echo "EXTRACT_RECORD_SRC=$EXTRACT_RECORD_SRC"
#echo "EXTRACT_TYPE=$EXTRACT_TYPE"
#echo "EXTRACT_WHERE_CLAUSE=$EXTRACT_WHERE_CLAUSE"
#echo "EXTRACT_NEEDS_SORTING=$EXTRACT_NEEDS_SORTING"
#echo "FOUR_DB_OPTIONS=$FOUR_DB_OPTIONS"
   if [ "$VERBOSE_MODE" = TRUE ] ; then
      echo "C4DB $FOUR_DB_OPTIONS $EXTRACT_FORMAT -set Feedback False \
        -string ExtractDataFile        '$EXTRACT_DATA_FILE'         \
        -string ExtractObjectName      '$EXTRACT_OBJECT_NAME'       \
        -string ExtractObjectNameUpper '$EXTRACT_OBJECT_NAME_UPPER' \
        -string ExtractRecordSrc       '$EXTRACT_RECORD_SRC'        \
        -string ExtractType            '$EXTRACT_TYPE'              \
        -string ExtractWhereClause     '$EXTRACT_WHERE_CLAUSE'      \
        -string ExtractExcludeColumns  '$EXTRACT_EXCLUDE_COLUMNS'   \
        -string OrderByExcludeColumns  '$ORDER_BY_EXCLUDE_COLUMNS'  \
        -r '$EXTRACT_DATA_FILE'    -i  '$EXTRACT_SCRIPT_FILE'"
   fi

   EXTRACT_SCRIPT_FILE="$(Path2Win.ksh $EXTRACT_SCRIPT_FILE)"
   C4DB $FOUR_DB_OPTIONS $EXTRACT_FORMAT -set Feedback False        \
        -string ExtractDataFile        "$EXTRACT_DATA_FILE"         \
        -string ExtractObjectName      "$EXTRACT_OBJECT_NAME"       \
        -string ExtractObjectNameUpper "$EXTRACT_OBJECT_NAME_UPPER" \
        -string ExtractRecordSrc       "$EXTRACT_RECORD_SRC"        \
        -string ExtractType            "$EXTRACT_TYPE"              \
        -string ExtractWhereClause     "$EXTRACT_WHERE_CLAUSE"      \
        -string ExtractExcludeColumns  "$EXTRACT_EXCLUDE_COLUMNS"   \
        -string OrderByExcludeColumns  "$ORDER_BY_EXCLUDE_COLUMNS"  \
        -r "$EXTRACT_DATA_FILE"    -i  "$EXTRACT_SCRIPT_FILE"
   C4DB_EXIT_CODE=$?
   if [ $C4DB_EXIT_CODE -ne 0 ] ; then
      rm -f $EXTRACT_DATA_FILE
      echo
      echo    "ERROR: Failed extracting data for $DATA_FILE."
      echo -n "ERROR: Press enter to continue... "
      read TO_CONTINUE
      continue
   fi

   if [ "$EXTRACT_NEEDS_SORTING" = TRUE ] ; then
      sort $MAX_LINE_LEN_OPTS $EXTRACT_DATA_FILE > $EXTRACT_DATA_FILE.1
      mv $EXTRACT_DATA_FILE.1 $EXTRACT_DATA_FILE
      if [ "$VERBOSE_MODE" = TRUE ] ; then
         echo "Successfully sorted file $EXTRACT_DATA_FILE."
      fi
   fi

   if [ "$RM_EXTRACT_SCRIPT_FILE" = TRUE ] ; then
      rm -f "$EXTRACT_SCRIPT_FILE"
   fi

   TOTAL_RECORDS=`EZGetFileRecordCount "$EXTRACT_DATA_FILE"`
   TOTAL_RECORDS_OLD=$TOTAL_RECORDS
   if [ -f "$DATA_FILE" ] ; then
      TOTAL_RECORDS_OLD=`EZGetFileRecordCount "$DATA_FILE"`
   fi
   TOTAL_RECS="$TOTAL_RECORDS records"
   if [ $TOTAL_RECORDS -eq 1 ] ; then
      TOTAL_RECS="$TOTAL_RECORDS record"
   fi
   src.ksh stabalize $STABALIZE_ARGS -c deduce -r \
-t Created    "Successfully created $TOTAL_RECS in $DATA_FILE." \
-t Unchanged  "Successfully keeping $TOTAL_RECS in $DATA_FILE." \
-t Stabalized "Successfully updated $TOTAL_RECS in $DATA_FILE." \
-t Excluded   "Successfully IGNORING DIFFS in the $TOTAL_RECORDS_OLD excluded records in $DATA_FILE." \
-t Retained   "Successfully IGNORING DIFFS in the $TOTAL_RECORDS_OLD records in $DATA_FILE." \
       $EXTRACT_DATA_FILE $DATA_FILE
   STABALIZE_EXIT_CODE=$?
   # 0) Matched 1) Created 2) Updated 3) Destroyed 4) Excluded 5) Not Updated
   if [ $STABALIZE_EXIT_CODE -eq 4 -o $STABALIZE_EXIT_CODE -eq 5 ] ; then
      TOTAL_RECORDS=$TOTAL_RECORDS_OLD
   fi
   let OVERALL_RECS=OVERALL_RECS+TOTAL_RECORDS
   let OVERALL_FILES=OVERALL_FILES+1
done

if [ "$TOTALS_FILE" = "" ] ; then
   if [ "$OVERALL_FILES" != 1 ] ; then
      echo "Successfully stabalized $OVERALL_RECS records in $OVERALL_FILES files."
   fi
else
   cat >> $TOTALS_FILE <<!
let EE_OVERALL_FILES=EE_OVERALL_FILES+$OVERALL_FILES
let EE_OVERALL_RECS=EE_OVERALL_RECS+$OVERALL_RECS
!
fi

exit 0
