#===========================================================================
#  Filename: DBSystemCmp.ksh
#    Author: Craig Schneider
# Orig Date: 07/24/12
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Compare database objects between 2 systems.
#===========================================================================

SOURCE_SERVER_NAME=
TARGET_SERVER_NAME=

function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo "$1"
   fi
   cat <<!

Usage: $0 [-s SourceServerName]         [-S TargetServerName]

      -s: Source server instance name to connect to ($SOURCE_SERVER_NAME).
      -S: Target server instance name to connect to ($TARGET_SERVER_NAME).

!
   exit 1
}

while [ "$#" -ne 0 ]
do
  case $1 in
      -s) if [ "$1" = "-s" ] ; then
             SOURCE_SERVER_NAME="$2"
          else
             TARGET_SERVER_NAME="$2"
          fi
          shift
          shift ;;
      -S) if [ "$1" = "-s" ] ; then
             SOURCE_SERVER_NAME="$2"
          else
             TARGET_SERVER_NAME="$2"
          fi
          shift
          shift ;;
       *) if [ "$1" = "-?" ] ; then
             Usage
          else
             Usage "Invalid option $1."
          fi
          shift ;;
  esac
done


#===========================================================================
# Perform usage checks.
#===========================================================================
if [ "$SOURCE_SERVER_NAME" = "" ] ; then
   Usage "Must specify a Source ServerName."
fi
if [ "$TARGET_SERVER_NAME" = "" ] ; then
   Usage "Must specify either a Target ServerName."
fi

#===========================================================================
# Setup source 4db options.
#===========================================================================
SOURCE_DB_OPTS=
if [ "$SOURCE_SERVER_NAME" != "" ] ; then
   SOURCE_DB_OPTS="$SOURCE_DB_OPTS -s $SOURCE_SERVER_NAME"
fi

#===========================================================================
# Setup target 4db options.
#===========================================================================
TARGET_DB_OPTS=
if [ "$TARGET_SERVER_NAME" != "" ] ; then
   TARGET_DB_OPTS="$TARGET_DB_OPTS -s $TARGET_SERVER_NAME"
fi

#===========================================================================
# Compare Database Objects.
#===========================================================================
function CompareDBObjects
{
   DB_OBJECT_TYPE="$1"      # SMART_ACTION, SCHEDULED_TASK, TASK_DIAGRAM
   DB_OBJECT_QUALIFIER="$2" # SmartQuery, JobTemplate, Chart

   OBJECT_TABLE_NAME="$DB_OBJECT_TYPE"
   OBJECT_NAME_COL_NAME="OBJECT_NAME"
   OBJECT_TYPE_COL_NAME=
   OBJECT_WHERE_CLAUSE=

   typeset -L20 OBJECT_STATUS_PAD
   typeset -L15 OBJECT_TYPE_PAD
   typeset -l   FILE_TYPE

   if [ "$DB_OBJECT_TYPE" = "SMART_ACTION" ] ; then
      typeset -L11 OBJECT_TYPE_PAD
      OBJECT_TABLE_NAME="SMART_ACTION_NAME"
      OBJECT_NAME_COL_NAME="SMART_ACTION_NAME"
      OBJECT_TYPE_COL_NAME="SMART_ACTION_TYPE"
      if [ "$DB_OBJECT_QUALIFIER" != "" ] ; then
         OBJECT_WHERE_CLAUSE="where SMART_ACTION_TYPE = '$DB_OBJECT_QUALIFIER'"
      fi
   fi
   if [ "$DB_OBJECT_TYPE" = "SCHEDULED_TASK" ] ; then
      OBJECT_NAME_COL_NAME="SCHEDULE_NAME"
   fi

   if [ "$DB_OBJECT_TYPE" = "TASK_DIAGRAM" ] ; then
      OBJECT_TABLE_NAME="TASK"
      OBJECT_NAME_COL_NAME="TASKNAME"
   fi

   SAVE_PWD="$PWD"
   mkdir -p $TMPDIR/DBCMP
   cd $TMPDIR/DBCMP
   OBJECT_LIST_FILE_NAME="$TMPDIR/DBCMP.$$.ObjList"
   OBJECT_EXISTS_FILE_NAME="$TMPDIR/DBCMP.$$.ObjExists"

   OBJECT_TYPE_SELECT=
   if [ "$OBJECT_TYPE_COL_NAME" != "" ] ; then
      OBJECT_TYPE_SELECT="$OBJECT_TYPE_COL_NAME as OBJECT_TYPE,"
   fi
   C4DB $SOURCE_DB_OPTS -Variables \
        -c "select   distinct top 2000000
                     $OBJECT_TYPE_SELECT
                     $OBJECT_NAME_COL_NAME as OBJECT_NAME,
                     REPLACE(
                     REPLACE(
                     REPLACE(
                     REPLACE($OBJECT_NAME_COL_NAME,
                             ' ', '_'),
                             '-', '_'),
                             '.', '_'),
                             '/', '_') as OBJECT_NAME_SAFE
            from     $OBJECT_TABLE_NAME
            $OBJECT_WHERE_CLAUSE
            order by 1, 2" \
        > "$OBJECT_LIST_FILE_NAME"

   while read SINGLE_LINE
   do
      eval $SINGLE_LINE
      OBJECT_TYPE_PAD="$DB_OBJECT_TYPE"
      OBJECT_TYPE_CRITERIA=
      if [ "$OBJECT_TYPE_COL_NAME" != "" ] ; then
         OBJECT_TYPE_PAD="$OBJECT_TYPE"
         OBJECT_TYPE_CRITERIA="and $OBJECT_TYPE_COL_NAME = '$OBJECT_TYPE'"
      fi
      C4DB $TARGET_DB_OPTS -ResultsOnly \
           -c "select 'Exists'
               from   $OBJECT_TABLE_NAME
               where  $OBJECT_NAME_COL_NAME = '$OBJECT_NAME'
                      $OBJECT_TYPE_CRITERIA" \
           > "$OBJECT_EXISTS_FILE_NAME"

      TARGET_EXISTS=`cat "$OBJECT_EXISTS_FILE_NAME"`
      if [ "$TARGET_EXISTS" = "Exists" ] ; then
         FILE_TYPE="$DB_OBJECT_TYPE"
         if [ "$FILE_TYPE" = "smartquery" ] ; then
            FILE_TYPE="smart_query"
         fi
         4db_extract.ksh -yes -quiet -s $SOURCE_SERVER_NAME \
                         $OBJECT_NAME_SAFE.$FILE_TYPE.4db
         mv $OBJECT_NAME_SAFE.$FILE_TYPE.4db \
            $OBJECT_NAME_SAFE.$FILE_TYPE.4db.source
         4db_extract.ksh -yes -quiet -s $TARGET_SERVER_NAME \
                         $OBJECT_NAME_SAFE.$FILE_TYPE.4db
         mv $OBJECT_NAME_SAFE.$FILE_TYPE.4db \
            $OBJECT_NAME_SAFE.$FILE_TYPE.4db.target
         diff $OBJECT_NAME_SAFE.$FILE_TYPE.4db.source \
              $OBJECT_NAME_SAFE.$FILE_TYPE.4db.target > $EZ_DEV_NULL
         if [ $? -eq 0 ] ; then
            OBJECT_STATUS_PAD="Identical"
         else
            OBJECT_STATUS_PAD="DIFFERENCES"
         fi
      else
         OBJECT_STATUS_PAD="$SOURCE_SERVER_NAME Only"
      fi

      echo "$OBJECT_STATUS_PAD $OBJECT_TYPE_PAD $OBJECT_NAME"
   done < "$OBJECT_LIST_FILE_NAME"

   cd "$SAVE_PWD"
   rm -f $TMPDIR/DBCMP.$$.*
   rm -rf $TMPDIR/DBCMP
}

#CompareDBObjects "SMART_ACTION" "Chart"
#CompareDBObjects "SMART_ACTION" "JobTemplate"
#CompareDBObjects "SMART_ACTION" "SmartQuery"

CompareDBObjects "SCHEDULED_TASK"

#CompareDBObjects "TASK_DIAGRAM"

exit 0
