#===========================================================================
# FileName: <<FileName>>
#   Author: Craig Schneider
#  Created: 2017-01-06
# Synopsis: Provides a better customized docker api utilizing a
#           .dr_config file in the current directory.
# <<Copyright>>
#===========================================================================

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: dr [dangling | images | logs | mk | ps | run | sh | 
           zap | zapall ] [-dryrun]

    dangling: Remove any dangling docker images.
     -dryrun: Display the commands to be executed, without executing them.
      images: Alias for docker images.
        logs: Display the logs for the running docker instance.
          mk: Preform a build of the docker image.
          ps: Alias for docker ps.
         run: Run the docker image.
          sh: Attach to the running docker image and start a sh.
         zap: Stop and remove a running docker instance.
      zapall: Stop and remove all running docker instances.

!
#      -quiet: Suppress non critical messages about operations being performed.
#    -verbose: Display verbose messages about operations being performed.
   exit 1
}

EZ_DRYRUN=FALSE
DR_COMMAND=

# The default shell to use overridable in .dr_config
DOCKER_SHELL="/bin/sh"

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
    -dryrun) EZ_DRYRUN=TRUE
             shift ;;
    dangling|images|logs|mk|ps|run|sh|zap|zapall|mrs|mrsz|rs|rsz)
             DR_COMMAND="$LOWERCASE_ARG"
             shift ;;
          *) if [ "$1" != "-?" ] ; then
                Usage "Invalid argument: $1"
             else
                Usage
             fi ;;
    esac
done

if [ "$DR_COMMAND" = "" ] ; then
   Usage
fi

DR_CONFIG_FILE_NAME="./.dr_config"
if [ -f "$DR_CONFIG_FILE_NAME" ] ; then
   . "$DR_CONFIG_FILE_NAME"
else
   if [ "$DR_COMMAND" != dangling -a \
        "$DR_COMMAND" != images -a   \
        "$DR_COMMAND" != ps ] ; then
      echo "ERROR: Failed to find config file $DR_CONFIG_FILE_NAME."
      exit 1
   fi
fi

function EnsureDockerImageName
{
   if [ "$DOCKER_IMAGE_NAME" = "" ] ; then
      echo "ERROR: DOCKER_IMAGE_NAME has not been specified file $DR_CONFIG_FILE_NAME."
      exit 1
   fi
}

function EnsureDockerInstanceName
{
   if [ "$DOCKER_INSTANCE_NAME" = "" ] ; then
      echo "ERROR: DOCKER_INSTANCE_NAME has not been specified file $DR_CONFIG_FILE_NAME."
      exit 1
   fi
}

if [ "$DR_COMMAND" = dangling ] ; then
   if [ "$EZ_DRYRUN" = TRUE ] ; then
      echo "Would run: docker images -qf dangling=true"
      DANGLING_IDS=`docker images -qf dangling=true`
      echo "Would run: docker rmi $DANGLING_IDS"
   else
      DANGLING_IDS=`docker images -qf dangling=true`
      if [ "$DANGLING_IDS" != "" ] ; then
         docker rmi $DANGLING_IDS
         EXIT_CODE=$?
         if [ $EXIT_CODE -ne 0 ] ; then
            exit $EXIT_CODE
         fi
         echo "Successfully removed all dangling docker images."
      else
         echo "Warning: Failed to find any dangling docker images to remove."
      fi
   fi
   exit 0

elif [ "$DR_COMMAND" = images ] ; then
   if [ "$EZ_DRYRUN" = TRUE ] ; then
      echo "Would run: docker images"
   else
      docker images
   fi
   exit 0

elif [ "$DR_COMMAND" = logs ] ; then
   EnsureDockerInstanceName
   if [ "$EZ_DRYRUN" = TRUE ] ; then
      echo "Would run: docker logs $DOCKER_INSTANCE_NAME"
   else
      docker logs $DOCKER_INSTANCE_NAME
      EXIT_CODE=$?
      if [ $EXIT_CODE -ne 0 ] ; then
         exit $EXIT_CODE
      fi
   fi
   exit 0

elif [ "$DR_COMMAND" = mk ] ; then
   EnsureDockerImageName
   if [ "$EZ_DRYRUN" = TRUE ] ; then
      echo "Would run: docker build -t $DOCKER_IMAGE_NAME ."
   else
      if [ -f xcp/.containerenv ] ; then
         sed -e '/^DOCKER_IMAGE_NAME=/d' xcp/.containerenv > xcp/.containerenv
         echo "DOCKER_IMAGE_NAME=$DOCKER_IMAGE_NAME" >> xcp/.containerenv
      fi
      docker build -t $DOCKER_IMAGE_NAME .
      EXIT_CODE=$?
      if [ $EXIT_CODE -ne 0 ] ; then
         exit $EXIT_CODE
      fi
      echo "Successfully built docker image $DOCKER_IMAGE_NAME."
   fi
   exit 0

elif [ "$DR_COMMAND" = ps ] ; then
   if [ "$EZ_DRYRUN" = TRUE ] ; then
      echo "Would run: docker ps"
   else
      docker ps
   fi
   exit 0

elif [ "$DR_COMMAND" = run ] ; then
   EnsureDockerImageName
   EnsureDockerInstanceName
   if [ "$EZ_DRYRUN" = TRUE ] ; then
      echo "Would run: docker run -d --name $DOCKER_INSTANCE_NAME $DOCKER_RUN_ARGS $DOCKER_IMAGE_NAME"
   else
      docker run -d --name $DOCKER_INSTANCE_NAME $DOCKER_RUN_ARGS $DOCKER_IMAGE_NAME
      EXIT_CODE=$?
      if [ $EXIT_CODE -ne 0 ] ; then
         exit $EXIT_CODE
      fi
      echo "Successfully started docker instance $DOCKER_INSTANCE_NAME."
   fi
   exit 0

elif [ "$DR_COMMAND" = sh ] ; then
   EnsureDockerInstanceName
   if [ "$EZ_DRYRUN" = TRUE ] ; then
      echo "Would run: docker exec -it $DOCKER_INSTANCE_NAME $DOCKER_SHELL -l -o vi"
   else
      docker exec -it $DOCKER_INSTANCE_NAME $DOCKER_SHELL -l -o vi
      exit $?
   fi
   exit 0

elif [ "$DR_COMMAND" = zap ] ; then
   EnsureDockerInstanceName
   if [ "$EZ_DRYRUN" = TRUE ] ; then
      echo "Would run: docker ps -aqf name=$DOCKER_INSTANCE_NAME"
      DOCKER_INSTANCE_ID=`docker ps -aqf name=$DOCKER_INSTANCE_NAME`
      echo "Would run: docker stop $DOCKER_INSTANCE_ID"
      echo "Would run: docker rm   $DOCKER_INSTANCE_ID"
   else
      DOCKER_INSTANCE_ID=`docker ps -aqf name=$DOCKER_INSTANCE_NAME`
      if [ "$DOCKER_INSTANCE_ID" != "" ] ; then
         echo "Successfully stopping docker instance $DOCKER_INSTANCE_NAME."
         docker stop $DOCKER_INSTANCE_ID
         echo "Successfully removing docker instance $DOCKER_INSTANCE_NAME."
         docker rm   $DOCKER_INSTANCE_ID
         echo "Successfully stopped and removed docker instance $DOCKER_INSTANCE_NAME."
      else
         echo "Warning: Failed to find a docker instance named $DOCKER_INSTANCE_NAME."
      fi
   fi
   exit 0

elif [ "$DR_COMMAND" = zapall ] ; then
   if [ "$EZ_DRYRUN" = TRUE ] ; then
      echo "Would run: docker ps -aq"
      DOCKER_INSTANCE_ID=`docker ps -aq`
      echo "Would run: docker stop $DOCKER_INSTANCE_ID"
      echo "Would run: docker rm   $DOCKER_INSTANCE_ID"
   else
      DOCKER_INSTANCE_ID=`docker ps -aq`
      if [ "$DOCKER_INSTANCE_ID" != "" ] ; then
         docker stop $DOCKER_INSTANCE_ID
         docker rm   $DOCKER_INSTANCE_ID
         echo "Successfully stopped and removed all docker instances."
      else
         echo "Warning: Failed to find any docker instances."
      fi
   fi
   exit 0

elif [ "$DR_COMMAND" = mrsz -o \
       "$DR_COMMAND" = mrs  -o \
       "$DR_COMMAND" = rs   -o \
       "$DR_COMMAND" = rsz ] ; then
   if [ "${DR_COMMAND%m*}" != "$DR_COMMAND" ] ; then
      dr.sh mk
   fi
   if [ "${DR_COMMAND%r*}" != "$DR_COMMAND" ] ; then
      dr.sh run
   fi
   if [ "${DR_COMMAND%s*}" != "$DR_COMMAND" ] ; then
      dr.sh sh
   fi
   if [ "${DR_COMMAND%z*}" != "$DR_COMMAND" ] ; then
      dr.sh zap
   fi

else
   Usage "ERROR: Unimplemented dr command $DR_COMMAND."
fi
