#===========================================================================
#  Filename: fam_feature.ksh
#    Author: Craig Schneider
# Orig Date: 07/10/13
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Manage family features per computer.
#===========================================================================

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: fam feature -Name FeatureName [-DryRun] [-Family FamilyName]
           [-Host HostName] [-Delete | -DeleteHost | -Latest | -List |
            -ReqRev Revision | -ReqRevFile FileName | -Revision |
            -Status | -Update] [-Quiet | -Verbose]

      -Delete: Delete the feature.
  -DeleteHost: Delete the specified Host.
      -DryRun: Display the commands to be executed, without executing them.
      -Family: Specifies the name of the source control family of the feature.
        -Host: Specifies the name of the host of the feature.
                  ALL:         Alias for SUBSCRIBED.
                  FEATURE:     The list of hosts with the feature set.
                  FAM_FEATURE: The list of hosts with the family feature set.
                  SUBSCRIBED:  The list of hosts subscribed to the family.
                  UNIVERSE:    The list of hosts with any feature set.
      -Latest: Display the list of features ordered by number.
        -List: Display the list of features.
        -Name: Specifies the name of the feature.
      -ReqRev: Return a status based upon the required minimum revision.
  -ReqRevFile: ReqRev logic, based unpon the last change revison of the file.
    -Revision: Display the revision number of the feature.
      -Status: Display the feature status information.
      -Update: Update the status and revision of the feature.
       -Quiet: Suppress non critical messages about operations being performed.
     -Verbose: Display verbose messages about operations being performed.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi

EXIT_CODE=0
EZ_DRYRUN=FALSE
EZ_VERBOSITY=
FAMILY_NAME=
FEATURE_NAME=
FEATURE_MODE=
HOST_NAME=
HOST_NAME_REQUESTED=FALSE
REQUIRED_REVISION=
REQUIRED_REVISION_FILE_NAME=
FEATURE_ARGS=

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
     -delete) FEATURE_MODE=DELETE
              FEATURE_ARGS="$FEATURE_ARGS $1" 
              shift ;;
 -deletehost) FEATURE_MODE=DELETE_HOST
              FEATURE_ARGS="$FEATURE_ARGS $1" 
              shift ;;
     -dryrun) EZ_DRYRUN=TRUE
              FEATURE_ARGS="$FEATURE_ARGS $1"
              shift ;;
  -f|-family) FAMILY_NAME="$2"
              FEATURE_ARGS="$FEATURE_ARGS $1 $2"
              shift
              shift ;;
    -h|-host) HOST_NAME="$2"
              HOST_NAME_REQUESTED=TRUE
              shift
              shift ;;
     -latest) FEATURE_MODE=LATEST
              FEATURE_ARGS="$FEATURE_ARGS $1" 
              shift ;;
       -list) FEATURE_MODE=LIST
              shift ;;
    -n|-name) FEATURE_NAME="$2"
              FEATURE_ARGS="$FEATURE_ARGS $1 $2"
              shift
              shift ;;
     -reqrev) FEATURE_MODE=REVISION
              REQUIRED_REVISION="$2"
              FEATURE_ARGS="$FEATURE_ARGS $1 $2"
              shift
              shift ;;
 -reqrevfile) FEATURE_MODE=REVISION
              REQUIRED_REVISION_FILE_NAME="$2"
              FEATURE_ARGS="$FEATURE_ARGS $1 $2"
              shift
              shift ;;
   -revision) FEATURE_MODE=REVISION
              FEATURE_ARGS="$FEATURE_ARGS $1"
              shift ;;
     -status) FEATURE_MODE=STATUS
              FEATURE_ARGS="$FEATURE_ARGS $1"
              shift ;;
     -update) FEATURE_MODE=UPDATE
              FEATURE_ARGS="$FEATURE_ARGS $1"
              shift ;;
-quiet|-verbose)
              EZ_VERBOSITY="${LOWERCASE_ARG#-}"
              FEATURE_ARGS="$FEATURE_ARGS $1"
              shift ;;
           *) if [ "$1" != "-?" ] ; then
                 Usage "Invalid argument: $1"
              else
                 Usage
              fi ;;
    esac
done

if [ "$FEATURE_MODE" = "" ] ; then
   FEATURE_MODE="LATEST"
fi

if [ "$FEATURE_MODE" = DELETE_HOST -a \
     "$HOST_NAME"    = "" ] ; then
   Usage "Must specify -Host HostName."
fi

if [ "$FEATURE_NAME"  = ""          -a \
     "$FEATURE_MODE" != DELETE_HOST -a \
     "$FEATURE_MODE" != LATEST      -a \
     "$FEATURE_MODE" != LIST ] ; then
   Usage "Must specify -Name FeatureName."
fi

if [ "$HOST_NAME" = ALL ] ; then
   HOST_NAME="SUBSCRIBED"
fi

if [ "$HOST_NAME" = "" ] ; then
   HOST_NAME="$EZ_COMPUTERNAME"
fi

if [ "$EZ_ACTIVE_RELEASE" != "working" ] ; then
   FEATURES_TEXT="Features"
   if [ "$FEATURE_NAME" = "Subscribed" ] ; then
      FEATURES_TEXT="Subscriptions"
   fi
   if [ "$FEATURE_MODE" = STATUS -o "$FEATURE_MODE" = REVISION ] ; then
      exit 0  # Don't want to see this error, and pretend it passed.
   else
      echo -n "ERROR: $FEATURES_TEXT are only supported in the working stream "
      echo    "not $EZ_ACTIVE_RELEASE."
   fi
   exit 1
fi

#===========================================================================
# Perform the requested operations.
#===========================================================================

if [ "$FAMILY_NAME"   = ""          -a \
     "$FEATURE_MODE" != DELETE_HOST -a \
     "$FEATURE_MODE" != LATEST      -a \
     "$FEATURE_MODE" != LIST ] ; then
   echo
   PS3="Select Family Name: "
   SAVE_IFS="$IFS"
   IFS="
"
   select FAMILY_NAME_SELECT in `fam.sh list`
   do
      if [ "$FAMILY_NAME_SELECT" != "" ] ; then
         FAMILY_NAME="${FAMILY_NAME_SELECT%% *}"
         break
      fi
   done
   IFS="$SAVE_IFS"
fi

if [ "$FAMILY_NAME"   = ""          -a \
     "$FEATURE_MODE" != DELETE_HOST -a \
     "$FEATURE_MODE" != LATEST      -a \
     "$FEATURE_MODE" != LIST ] ; then
   echo
   echo "ERROR: No family has been specified."
   exit 1
fi

# Kludge in case date & egrep are not in PATH
DATE_PATH=`whence date`
if [ "$DATE_PATH" = "" ] ; then
   PATH="$PATH:/usr/bin"
fi

FEATURE_KEY="$EZ_ACTIVE_RELEASE $HOST_NAME $FAMILY_NAME $FEATURE_NAME"
#echo "FEATURE_KEY=$FEATURE_KEY"

FEATURE_FILE_DIR="$EZ_FAMILYTREE_ROOT"
if [ "$EZ_FAMILYTREE_ROOT" = "" ] ; then
   FEATURE_FILE_DIR="${EZ_ENV_DIR%/*}"
fi

if [ ! -d "$FEATURE_FILE_DIR" ] ; then
   echo "ERROR: Failed to find feature file directory $FEATURE_FILE_DIR."
   exit 1
fi

FEATURE_FILE_NAME=Features.dat
FEATURE_FILE_FULL_NAME="$FEATURE_FILE_DIR/$FEATURE_FILE_NAME"
if [ ! -f "$FEATURE_FILE_FULL_NAME" ] ; then
   echo -n "ERROR: Failed to find feature file "
   echo    "$FEATURE_FILE_FULL_NAME."
   exit 1
fi

#===========================================================================
# Special host name processing and looping
#===========================================================================

if [ "$HOST_NAME" = FEATURE    -o \
     "$HOST_NAME" = SUBSCRIBED -o \
     "$HOST_NAME" = UNIVERSE ] ; then
   QUALIFIER_1=" "    # The space here is intentional to match every line
   QUALIFIER_2=" "    # The space here is intentional to match every line
   if [ "$HOST_NAME" = FAM_FEATURE ] ; then
      QUALIFIER_1=" $FEATURE_NAME "
      QUALIFIER_2=" $FAMILY_NAME "
   elif [ "$HOST_NAME" = FEATURE ] ; then
      QUALIFIER_1=" $FEATURE_NAME "
   elif [ "$HOST_NAME" = SUBSCRIBED ] ; then
      QUALIFIER_1=" Subscribed "
      if [ "$FAMILY_NAME" != "" ] ; then
         QUALIFIER_2=" $FAMILY_NAME "
      fi
   #elif [ "$HOST_NAME" = UNIVERSE ] ; then
      # Use the default qualifiers.
   fi
   HOST_LIST=`cat "$FEATURE_FILE_FULL_NAME" | grep "$QUALIFIER_1" | \
                  grep "$QUALIFIER_2" | awk '{ print $2 }' | uniq`

   MULTI_HOST_FILE="$TMPDIR/fam_feat_multi_host.$$"
   for SINGLE_HOST in $HOST_LIST
   do
      #echo "fam.sh feature -host $SINGLE_HOST $FEATURE_ARGS"
      fam.sh feature -host $SINGLE_HOST $FEATURE_ARGS >> "$MULTI_HOST_FILE"
   done
   sort -n "$MULTI_HOST_FILE"
   rm -f "$MULTI_HOST_FILE"
   exit 0
fi

if [ "$FEATURE_MODE" = LATEST ] ; then
   QUALIFIER_1=" "    # The space here is intentional to match every line
   QUALIFIER_2=" "    # The space here is intentional to match every line
   QUALIFIER_3=" "    # The space here is intentional to match every line
   if [ "$FAMILY_NAME" != "" ] ; then
      QUALIFIER_1=" $FAMILY_NAME "
   fi
   if [ "$FEATURE_NAME" != "" ] ; then
      QUALIFIER_2=" $FEATURE_NAME "
   fi
   if [ "$HOST_NAME_REQUESTED" = TRUE ] ; then
      QUALIFIER_3=" $HOST_NAME "
   fi
   src.ksh extract -quiet           "$FEATURE_FILE_FULL_NAME"
   LATEST_OUTPUT_FILE="$TMPDIR/fam_feat_latest.$$"
   cat "$FEATURE_FILE_FULL_NAME" | grep -i "$QUALIFIER_1" | \
       grep -i "$QUALIFIER_2" | grep -i "$QUALIFIER_3" | \
       awk '{ printf("%-7s %-28s %-15s %s\n", $5, $2, $3, $4 ) }' | \
       sort -n > "$LATEST_OUTPUT_FILE"
   cat "$LATEST_OUTPUT_FILE"
   LATEST_LINES=`cat "$LATEST_OUTPUT_FILE" | wc -l`
   if [ "$LATEST_LINES" = "0" ] ; then
      printf "%-7s %-28s %-15s %s\n" -1 $HOST_NAME $FAMILY_NAME $FEATURE_NAME
   fi
   rm -f "$LATEST_OUTPUT_FILE"
elif [ "$FEATURE_MODE" = LIST ] ; then
   QUALIFIER_1=" "    # The space here is intentional to match every line
   QUALIFIER_2=" "    # The space here is intentional to match every line
   QUALIFIER_3=" "    # The space here is intentional to match every line
   if [ "$FAMILY_NAME" != "" ] ; then
      QUALIFIER_1=" $FAMILY_NAME "
   fi
   if [ "$FEATURE_NAME" != "" ] ; then
      QUALIFIER_2=" $FEATURE_NAME "
   fi
   if [ "$HOST_NAME_REQUESTED" = TRUE ] ; then
      QUALIFIER_3=" $HOST_NAME "
   fi
   src.ksh extract -quiet           "$FEATURE_FILE_FULL_NAME"
   cat "$FEATURE_FILE_FULL_NAME" | grep -i "$QUALIFIER_1" | \
       grep -i "$QUALIFIER_2" | \
       awk '{ printf("%-15s %s\n", $4, $3) }' | sort | uniq
elif [ "$FEATURE_MODE" = STATUS   -o \
       "$FEATURE_MODE" = REVISION ] ; then
   FEATURE_INFO_FILE="$TMPDIR/fam_feature.$$"
   egrep "$FEATURE_KEY" "$FEATURE_FILE_FULL_NAME" > "$FEATURE_INFO_FILE"
   EXIT_CODE=$?
   FEATURE_REVISION=`awk '{ print $5 }' "$FEATURE_INFO_FILE"`
   rm -f "$FEATURE_INFO_FILE"

   if [ "$FEATURE_REVISION" = "" ] ; then
      FEATURE_REVISION="-1"
   fi

   if [ "$FEATURE_MODE" = REVISION ] ; then
      if [ "$REQUIRED_REVISION_FILE_NAME" != "" ] ; then
         REQUIRED_REVISION=`src.ksh history -LastChangeRev \
                                $REQUIRED_REVISION_FILE_NAME`
      fi
      if [ "$REQUIRED_REVISION" != "" ] ; then
         if [ $FEATURE_REVISION -ge $REQUIRED_REVISION ] ; then
            EXIT_CODE=0
         else
            EXIT_CODE=1
         fi
      else
         echo "$FEATURE_REVISION"
      fi
   elif [ "$EZ_VERBOSITY" != quiet ] ; then
      if [ "$EXIT_CODE" -eq 0 ] ; then
         echo -n "Successfully detected $FAMILY_NAME $FEATURE_NAME at "
         echo    "rev $FEATURE_REVISION on $HOST_NAME."
      else
         echo -n "Warning detected $FAMILY_NAME $FEATURE_NAME is not "
         echo    "present on $HOST_NAME."
      fi
   fi
elif [ "$FEATURE_MODE" = DELETE      -o \
       "$FEATURE_MODE" = DELETE_HOST -o \
       "$FEATURE_MODE" = UPDATE ] ; then
   TOP_DIR_KSH=`Path2Ksh.ksh "${EZ_FAMILYTREE_ROOT%/*}"`
   cd "$TOP_DIR_KSH"

   FEATURE_FILE_DIR_KSH=`Path2Ksh.ksh     "$FEATURE_FILE_DIR"`
   cd "$FEATURE_FILE_DIR_KSH"
   src.ksh extract -quiet                 "$FEATURE_FILE_NAME"

   if [ "$FEATURE_MODE" = DELETE_HOST ] ; then
      sed "/^$EZ_ACTIVE_RELEASE $HOST_NAME /d" \
                                           $FEATURE_FILE_NAME \
                                         > $FEATURE_FILE_NAME.fix
   else
      sed "/^$FEATURE_KEY[ -~]*\$/d"       $FEATURE_FILE_NAME \
                                         > $FEATURE_FILE_NAME.fix
   fi
   if [ "$FEATURE_MODE" = UPDATE ] ; then
      FEATURE_REVISION=`svn info $EZ_SOURCE_CONTROL_ROOT | tr -d '\r' | \
                           grep 'Revision:' | awk '{print $2}' `
      FEATURE_FULL_LINE="$FEATURE_KEY $FEATURE_REVISION"
      FEATURE_FULL_LINE="$FEATURE_FULL_LINE $(date +%Y-%m-%d)"
      #echo "FEATURE_FULL_LINE=$FEATURE_FULL_LINE"

      echo "$FEATURE_FULL_LINE"         >> $FEATURE_FILE_NAME.fix
      sort $FEATURE_FILE_NAME.fix        > $FEATURE_FILE_NAME.fix1
      mv   $FEATURE_FILE_NAME.fix1         $FEATURE_FILE_NAME.fix
   fi

   if [ "$EZ_DRYRUN" = TRUE ] ; then
      diff                                 $FEATURE_FILE_NAME.fix \
                                           $FEATURE_FILE_NAME
      rm -f                                $FEATURE_FILE_NAME.fix
   else
      src.ksh stabalize -yes -quiet -r     $FEATURE_FILE_NAME.fix \
                                           $FEATURE_FILE_NAME
   fi

   if [ "$EZ_VERBOSITY" != quiet ] ; then
      if [ "$FEATURE_NAME" = Subscribed ] ; then
         echo
         if [ "$FEATURE_MODE" = UPDATE ] ; then
            echo -n "Successfully subscribed to $FAMILY_NAME "
            echo    "rev $FEATURE_REVISION on $HOST_NAME."
         else
            echo -n "Successfully unsubscribed from $FAMILY_NAME "
            echo    "on $HOST_NAME."
         fi
      elif [ "$EZ_DRYRUN" = TRUE ] ; then
         if [ "$FEATURE_MODE" = UPDATE ] ; then
            echo -n "Would have updated $FAMILY_NAME $FEATURE_NAME to "
            echo    "rev $FEATURE_REVISION on $HOST_NAME."
         elif [ "$FEATURE_MODE" = DELETE_HOST ] ; then
            echo    "Would have deleted host $HOST_NAME."
         else
            echo -n "Would have deleted $FAMILY_NAME $FEATURE_NAME "
            echo    "on $HOST_NAME."
         fi
      else
         if [ "$FEATURE_MODE" = UPDATE ] ; then
            echo -n "Successfully updated $FAMILY_NAME $FEATURE_NAME to "
            echo    "rev $FEATURE_REVISION on $HOST_NAME."
         elif [ "$FEATURE_MODE" = DELETE_HOST ] ; then
            echo    "Successfully deleted host $HOST_NAME."
         else
            echo -n "Successfully deleted $FAMILY_NAME $FEATURE_NAME "
            echo    "on $HOST_NAME."
         fi
      fi
   fi
else
   echo "ERROR: Unsupported feature mode $FEATURE_MODE."
   exit 1
fi

exit $EXIT_CODE
