#===========================================================================
#  Filename: last_name.ksh
#    Author: Craig Schneider
# Orig Date: 03/04/00
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Determine the most recent candidate release number.
#===========================================================================

if [ "$EZ_SHARED_ENV" != "" ] ; then
   . $EZ_SHARED_ENV
fi

INCREMENT_DECREMENT_AMOUNT=
VERSION_NUMBER=
FILE_NAME_LIST=
PATTERN="_"
RELEASE_NAME=
SYNC_NAME=
SYNC_ORDER=

typeset -i MAX_VERSION_NUMBERS=5
typeset -i OFFSET
typeset -i LAST_POSITION
typeset -i CURR_POSITION
typeset -i LAST_OFFSET=10
typeset -i CURR_OFFSET=20

function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo "$1"
   fi
   cat <<!

Usage: last_name [+1 | -1 ... -10 ] [-r ReleaseName] [-v VersionNumber]
                 [-p Pattern] [-sync_name SyncName] [-sync_order SyncOrder]
                 ListOfNames

  -1 .. -10) Decrement the build number by one to ten numbers.
         +1) Increment the build number by one.
ListOfNames) A list of names to determine the highest numbered name from.
         -r) Get the last name for the specified release.
         -p) Pattern that preceeds the version number info (defaults to '_').
  sync_name) Specifies that sync name to use for synced release numbers.
 sync_order) Specifies that synced order number of this release.
         -v) The version number prefix used (typically major.minor number).

!
   exit 1
}

while [ "$#" -ne 0 ]
do
  case $1 in
+1|-1|-2|-3|-4|-5|-6|-7|-8|-9|-10)
             INCREMENT_DECREMENT_AMOUNT="$1"
             shift ;;
         -p) PATTERN="$2"
             shift
             shift ;;
         -r) RELEASE_NAME=$2
             shift
             shift ;;
 -sync_name) SYNC_NAME=$2
             shift
             shift ;;
-sync_order) SYNC_ORDER=$2
             shift
             shift ;;
         -v) VERSION_NUMBER=$2
             shift
             shift ;;
         -?) FILE_NAME_LIST=
             shift ;;
          *) if [ "$FILE_NAME_LIST" = "" ] ; then
                FILE_NAME_LIST="$1"
             else
                FILE_NAME_LIST="$FILE_NAME_LIST $1"
             fi
             shift ;;
  esac
done

if [ "$INCREMENT_DECREMENT_AMOUNT" = "" -a "$VERSION_NUMBER" != "" ] ; then
   Usage "The -v argument is not supported unless +1 or -1 ... -5 is specified."
fi

function IsCurrGreaterThanLast
{
   typeset -i CURR_NUMBER
   typeset -i LAST_NUMBER
   OFFSET=$MAX_VERSION_NUMBERS
   while [ $OFFSET -ge 1 ]
   do
       LAST_POSITION=OFFSET+LAST_OFFSET
       CURR_POSITION=OFFSET+CURR_OFFSET
       CURR_NUMBER=VERSION_NUMBERS[$CURR_POSITION]
       LAST_NUMBER=VERSION_NUMBERS[$LAST_POSITION]
       if [ $CURR_NUMBER -gt $LAST_NUMBER ] ; then
          return 0
       elif [ $CURR_NUMBER -lt $LAST_NUMBER ] ; then
          return 1
       fi
       OFFSET=OFFSET-1
   done
   return 1
}

function ListVersionNumbers
{
   OFFSET=$MAX_VERSION_NUMBERS
   while [ $OFFSET -ge 1 ]
   do
       LAST_POSITION=OFFSET+LAST_OFFSET
       CURR_POSITION=OFFSET+CURR_OFFSET
       echo -n "LAST: $LAST_POSITION  ${VERSION_NUMBERS[LAST_POSITION]}   "
       echo    "CURR: $CURR_POSITION  ${VERSION_NUMBERS[CURR_POSITION]}"
       OFFSET=OFFSET-1
   done
}

function SetVersionNumbers
{
   VERSION_NUMBER_STRING="$1"
   typeset -i LAST_OR_CURR_OFFSET=$2
   typeset -i POSITION

   OFFSET=1
   while [ $OFFSET -le $MAX_VERSION_NUMBERS ]
   do
       SINGLE_VERSION_NUMBER="${VERSION_NUMBER_STRING##*.}"
       if [ "$SINGLE_VERSION_NUMBER" = "$VERSION_NUMBER_STRING" ] ; then
          if [ "$SINGLE_VERSION_NUMBER" = "" ] ; then
             SINGLE_VERSION_NUMBER="0"
          fi
          VERSION_NUMBER_STRING=""
       else
          VERSION_NUMBER_STRING="${VERSION_NUMBER_STRING%.*}"
       fi

       # if the single version number is not an integer, skip it (P01). 
       #if [[ "$SINGLE_VERSION_NUMBER" = *([0-9]) ]] ; then
       if [ "${SINGLE_VERSION_NUMBER#[0-9]}" != \
            "$SINGLE_VERSION_NUMBER" ] ; then
          POSITION=OFFSET+LAST_OR_CURR_OFFSET
          VERSION_NUMBERS[$POSITION]="$SINGLE_VERSION_NUMBER"
          OFFSET=OFFSET+1
       else
          LETTER_VERSION=0
          typeset -l LOWER_LETTER="$SINGLE_VERSION_NUMBER"
          if   [ "$LOWER_LETTER" = "a" ] ; then
             LETTER_VERSION=1
          elif [ "$LOWER_LETTER" = "b" ] ; then
             LETTER_VERSION=2
          elif [ "$LOWER_LETTER" = "c" ] ; then
             LETTER_VERSION=3
          elif [ "$LOWER_LETTER" = "d" ] ; then
             LETTER_VERSION=4
          elif [ "$LOWER_LETTER" = "e" ] ; then
             LETTER_VERSION=5
          fi
          if [ "$LETTER_VERSION" != 0 ] ; then
             POSITION=OFFSET+LAST_OR_CURR_OFFSET
             VERSION_NUMBERS[$POSITION]="$LETTER_VERSION"
             OFFSET=OFFSET+1
          fi
       fi
   done
}
   
if [ "$SYNC_NAME" != "" -o "$SYNC_ORDER" != "" ] ; then
   if [ "$RELEASE_NAME" = "" ] ; then
      echo "Syncing is only supported with a release name."
      exit 1
   fi
   if [ "$VERSION_NUMBER" = "" ] ; then
      echo "Syncing is only supported with a version number."
      exit 1
   fi


   SYNC_FILE_DIR="$EZ_FAMILYTREE_ROOT/Releases"
   if [ ! -d "$SYNC_FILE_DIR" ] ; then
      echo "ERROR: Failed to find build number sync dir $SYNC_FILE_DIR."
      exit 1
   fi
   cd "$SYNC_FILE_DIR"

   SYNC_FILE_NAME="build_numbers.sync"
   if [ ! -f "$SYNC_FILE_NAME" ] ; then
      echo -n "ERROR: Failed to find build number sync file "
      echo    "$SYNC_FILE_DIR/$SYNC_FILE_NAME."
      exit 1
   fi

   src.ksh extract -quiet "$SYNC_FILE_NAME"
   
   INCREMENTING_REQUIRED=FALSE
   MAX_BUILD_NUM=0
   while read NAME ORDER BUILD_NUM REL_NAME
   do
      if [ "$NAME" = "$SYNC_NAME" ] ; then
         if [ $MAX_BUILD_NUM -le $BUILD_NUM ] ; then
            MAX_BUILD_NUM="$BUILD_NUM"
            if [ $ORDER -ge $SYNC_ORDER ] ; then
               INCREMENTING_REQUIRED=TRUE
            fi
         fi
      fi
   done < "$SYNC_FILE_NAME"
   
   if [ "$INCREMENT_DECREMENT_AMOUNT" != "" ] ; then
      if [ "$INCREMENTING_REQUIRED" = TRUE ] ; then
         let MAX_BUILD_NUM=MAX_BUILD_NUM$INCREMENT_DECREMENT_AMOUNT
      fi
      INCREMENT_DECREMENT_AMOUNT=""
   fi
   LAST_FILE_NAME="${RELEASE_NAME}_${VERSION_NUMBER}.$MAX_BUILD_NUM"

   sed "/^$SYNC_NAME [ -~]* $RELEASE_NAME\$/d" $SYNC_FILE_NAME \
       > $SYNC_FILE_NAME.fix
   echo "$SYNC_NAME $SYNC_ORDER $MAX_BUILD_NUM $RELEASE_NAME" \
      >> $SYNC_FILE_NAME.fix
   sort $SYNC_FILE_NAME.fix > $SYNC_FILE_NAME.fix1
   mv $SYNC_FILE_NAME.fix1 $SYNC_FILE_NAME.fix
   src.ksh stabalize -yes -quiet -r $SYNC_FILE_NAME.fix $SYNC_FILE_NAME
else

   if [ "$RELEASE_NAME" != "" ] ; then
      REL_INFO_DIR=$EZ_FAMILYTREE_ROOT/Releases/Built
      if [ ! -d $REL_INFO_DIR ] ; then
         echo "ERROR: Failed to find release info directory $REL_INFO_DIR"
         exit 1
      fi

      if [ "${RELEASE_NAME%_}" != "$RELEASE_NAME" ] ; then
         MATCHING_RELEASE_NAME="${RELEASE_NAME}"
      else
         MATCHING_RELEASE_NAME="${RELEASE_NAME}_"
      fi

      cd "$REL_INFO_DIR"
      FILE_NAME_LIST=`echo $MATCHING_RELEASE_NAME*.ksh | sed 's/\.ksh//g'`
      # if we don't have a built release, maybe we have a delivered one.
      if [ "$FILE_NAME_LIST" = "$MATCHING_RELEASE_NAME*" ] ; then
         cd $EZ_DELIVER_ROOT
         FILE_NAME_LIST=`echo $MATCHING_RELEASE_NAME*`
      fi

      # if this is the first build of this release, number it correctly
      if [ "$FILE_NAME_LIST" = "$MATCHING_RELEASE_NAME*" ] ; then
         FILE_NAME_LIST="$RELEASE_NAME"
      fi
   fi

   if [ "$FILE_NAME_LIST" = "" ] ; then
      Usage "Must supply a list of file or directory names."
   fi

   LAST_FILE_NAME=
   for CURR_FILE_NAME in $FILE_NAME_LIST
   do
      # 06/24/2010 : Match to the right-most PATTERN
      VERSION_PORTION="${CURR_FILE_NAME##*$PATTERN}"
      # if there is no version portion to the string
      # last_name yyy_* where no yyy_ files exist or
      # last_name yyy*  where no yyy  files exist or
      # last_name -r NewRel where no NewRel releases were built yet
      if [ "$VERSION_PORTION" == "$CURR_FILE_NAME" -o \
           "$VERSION_PORTION" == "" ] ; then
         CURR_FILE_NAME="${CURR_FILE_NAME%\*}"
         if [ "$INCREMENT_DECREMENT_AMOUNT" != "" ] ; then
            CURR_FILE_NAME="${CURR_FILE_NAME%_*}_1.0.0"
         fi
      fi

      if [ "$LAST_FILE_NAME" = "" ] ; then
         LAST_FILE_NAME="$CURR_FILE_NAME"
      else
         LAST_FILE_BASE_NAME="${LAST_FILE_NAME##*/}"
         CURR_FILE_BASE_NAME="${CURR_FILE_NAME##*/}"
         # 06/24/2010 : Match to the right-most PATTERN
         SetVersionNumbers "${LAST_FILE_BASE_NAME##*$PATTERN}" $LAST_OFFSET
         SetVersionNumbers "${CURR_FILE_BASE_NAME##*$PATTERN}" $CURR_OFFSET

         IsCurrGreaterThanLast
         if [ $? -eq 0 ] ; then
            LAST_FILE_NAME="$CURR_FILE_NAME"
         fi
      fi
   done
fi

if [ "$INCREMENT_DECREMENT_AMOUNT" != "" ] ; then
   MINOR_REV_NAME=${LAST_FILE_NAME%.*}
   BUILD_REV_NUMBER=${LAST_FILE_NAME#$MINOR_REV_NAME.}
   let BUILD_REV_NUMBER=BUILD_REV_NUMBER$INCREMENT_DECREMENT_AMOUNT
   if [ "$VERSION_NUMBER" != "" ] ; then
      VERSIONLESS_NAME="${MINOR_REV_NAME%_*}"
      echo ${VERSIONLESS_NAME}_$VERSION_NUMBER.$BUILD_REV_NUMBER
   else
      echo $MINOR_REV_NAME.$BUILD_REV_NUMBER
   fi
else
   echo $LAST_FILE_NAME
fi
exit 0
