#===========================================================================
#  Filename: rel_3rd_ref.ksh
#    Author: Craig Schneider
# Orig Date: 01/03/13
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Add/remove 3rd party references.
#===========================================================================

. $EZ_SHARED_ENV

function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo "$1"
   fi
   cat <<!

Usage: 3rd_ref [-d DirectoryName] [-r ReleaseName] [-m Message]
               [-Add] [-Delete] [-Status] [-Update]
               [-DryRun] [ -Quiet | -Verbose]

        -Add: Add the specified reference.
          -d: The 3rd party directory being referenced.
     -Delete: Delete the specified reference.
     -DryRun: Display the commands to be executed, without executing them.
          -m: Message to include in the reference.
      -Quiet: Suppress non critical messages about operations being performed.
          -r: Release name of the 3rd party directory reference.
     -Status: Return a status indicating if the directory is referenced.
     -Update: Update the specified reference.
    -Verbose: Display verbose messages about operations being performed.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
EZ_DRYRUN=FALSE
EZ_VERBOSITY=
ONLY_REF_MESSAGES=FALSE
REF_DIR_NAME=
REF_MESSAGE=
RELEASE_NAME=
REFERENCE_MODE=ADD

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
          -add) REFERENCE_MODE="ADD"
                shift ;;
            -d) REF_DIR_NAME="$2"
                shift
                shift ;;
       -delete) REFERENCE_MODE="DELETE"
                shift ;;
       -dryrun) EZ_DRYRUN=TRUE
                shift ;;
            -m) REF_MESSAGE="$2"
                shift
                shift ;;
-onlyrefmessages) ONLY_REF_MESSAGES=TRUE
                shift ;;
       -quiet|-verbose)
                EZ_VERBOSITY="${LOWERCASE_ARG#-}"
                shift ;;
       -status) REFERENCE_MODE="STATUS"
                shift ;;
       -update) REFERENCE_MODE="UPDATE"
                shift ;;
            -r) RELEASE_NAME="$2"
                shift
                shift ;;
            -?) Usage ;;
    esac
done        

if [ "$RELEASE_NAME" = "" ] ; then
   if [ "$REFERENCE_MODE" != "STATUS" ] ; then
      Usage "Must specify -r ReleaseName."
   fi
fi

EZ_3RD_DIR="$EZ_SUPER_DRIVE/3rdParty"
REF_DIR_ROOT="$EZ_3RD_DIR/_References"

if [ ! -d "$EZ_3RD_DIR" ] ; then
   echo "ERROR: Failed to find 3rd Party directory $EZ_3RD_DIR."
   exit 1
fi
if [ ! -d "$REF_DIR_ROOT" ] ; then
   echo "ERROR: Failed to find 3rd Party _References \c"
   echo "directory $REF_DIR_ROOT."
   exit 1
fi

if [ "$REF_DIR_NAME" = "" -a "$REFERENCE_MODE" = DELETE ] ; then
   POSSIBLE_DIR=`echo $REF_DIR_ROOT/*/$RELEASE_NAME.ref`
   if [ "$POSSIBLE_DIR" = "$REF_DIR_ROOT/*/$RELEASE_NAME.ref" ] ; then
      if [ "$EZ_VERBOSITY" != "quiet" ] ; then
         echo "Successfully deleted all $RELEASE_NAME references."
      fi
      exit 0
   fi
   POSSIBLE_DIR="${POSSIBLE_DIR%/*}"
   REF_DIR_NAME="${POSSIBLE_DIR##*/}"
fi

if [ "$REF_DIR_NAME" = "" ] ; then
   Usage "Must specify -d DirectoryName."
fi

if [ ! -d "$EZ_3RD_DIR/$REF_DIR_NAME" ] ; then
   echo "ERROR: Failed to find 3rd Party directory $EZ_3RD_DIR/$REF_DIR_NAME."
   exit 1
fi

REF_DIR_FULL_NAME="$REF_DIR_ROOT/$REF_DIR_NAME"
REF_FILE_NAME="$RELEASE_NAME.ref"
REF_FILE_FULL_NAME="$REF_DIR_FULL_NAME/$REF_FILE_NAME"
if [ "$REFERENCE_MODE" = STATUS ] ; then
   if [ "$RELEASE_NAME" != "" ] ; then
      if [ ! -f "$REF_FILE_FULL_NAME" ] ; then
         if [ "$EZ_VERBOSITY" != "quiet" -a \
              "$ONLY_REF_MESSAGES" != TRUE ] ; then
            echo "Successfully determined $REF_DIR_NAME is \c"
            echo "not referenced by $RELEASE_NAME."
         fi
         exit 0
      else
         if [ "$EZ_VERBOSITY" != "quiet" ] ; then
            echo "Successfully determined $REF_DIR_NAME is \c"
            echo "referenced by $RELEASE_NAME."
         fi
         exit 1
      fi
   else
      if [ ! -d "$REF_DIR_FULL_NAME" ] ; then
         if [ "$EZ_VERBOSITY" != "quiet" -a \
              "$ONLY_REF_MESSAGES" != TRUE ] ; then
            echo "Successfully determined $REF_DIR_NAME is not referenced."
         fi
         exit 0
      else
         REF_NAME_LIST=`echo $REF_DIR_FULL_NAME/*`
         if [ "$REF_NAME_LIST" = "$REF_DIR_FULL_NAME/*" ] ; then
            if [ "$EZ_VERBOSITY" != "quiet" -a \
                 "$ONLY_REF_MESSAGES" != TRUE ] ; then
               echo "Successfully determined $REF_DIR_NAME is not referenced."
            fi
            exit 0
         fi
         for SINGLE_REF_FILE_FULL_NAME in $REF_NAME_LIST
         do
            SINGLE_REF_FILE_NAME="${SINGLE_REF_FILE_FULL_NAME##*/}"
            SINGLE_REF_NAME="${SINGLE_REF_FILE_NAME%.ref}"
            if [ "$EZ_VERBOSITY" != "quiet" ] ; then
               echo "Successfully determined $REF_DIR_NAME is \c"
               echo "referenced by $SINGLE_REF_NAME."
            fi
         done
         exit 1
      fi
   fi
elif [ "$REFERENCE_MODE" = ADD -o "$REFERENCE_MODE" = UPDATE ] ; then
   if [ ! -d "$REF_DIR_FULL_NAME" ] ; then
      if [ "$EZ_DRYRUN" = TRUE ] ; then
         echo "Would: mkdir $REF_DIR_FULL_NAME"
      else
         mkdir "$REF_DIR_FULL_NAME"
      fi
      if [ "$EZ_VERBOSITY" = "verbose" ] ; then
         echo "Successfully created directory $REF_DIR_FULL_NAME."
      fi
   fi
   if [ -f "$REF_FILE_FULL_NAME" -a "$REFERENCE_MODE" != UPDATE ] ; then
      echo "ERROR: Detected $REF_DIR_NAME is already referenced \c"
      echo "by $RELEASE_NAME."
      exit 1
   else
      if [ "$EZ_DRYRUN" = TRUE ] ; then
         echo "Would: touch $REF_FILE_FULL_NAME"
      else
         REF_DATE=`date +'%b %d, %Y'`
         REF_DATE_SORT=`date +'%Y-%m-%d'`
         cat > $REF_FILE_FULL_NAME <<!
REFERENCE_DATE="$REF_DATE"
REFERENCE_DATE_SORT="$REF_DATE_SORT"
REFERENCE_MESSAGE="$REF_MESSAGE"
!
      fi
      if [ "$EZ_VERBOSITY" != "quiet" ] ; then
         echo "Successfully added $RELEASE_NAME reference to $REF_DIR_NAME."
      fi
      exit 0
   fi
elif [ "$REFERENCE_MODE" = DELETE ] ; then
   if [ -f "$REF_FILE_FULL_NAME" ] ; then
      if [ "$EZ_DRYRUN" = TRUE ] ; then
         echo "Would: rm -f $REF_FILE_FULL_NAME"
      else
         rm -f "$REF_FILE_FULL_NAME"
      fi
   fi
   REF_NAME_LIST=`echo $REF_DIR_FULL_NAME/*`
   if [ "$REF_NAME_LIST" = "$REF_DIR_FULL_NAME/*" ] ; then
      if [ "$EZ_DRYRUN" = TRUE ] ; then
         echo "Would: rmdir $REF_DIR_FULL_NAME"
      else
         rmdir "$REF_DIR_FULL_NAME"
      fi
   fi
   if [ "$EZ_VERBOSITY" != "quiet" ] ; then
      echo "Successfully deleted $RELEASE_NAME reference to \c"
      echo "$REF_DIR_NAME."
   fi
   exit 0
else
   echo "ERROR: Unsupported reference mode of $REFERENCE_MODE."
   exit 1
fi
