#===========================================================================
#  Filename: rel_build.ksh
#    Author: Craig Schneider
# Orig Date: 09/27/96
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Create, extract, make and deliver a new candidate release.
#===========================================================================

. $EZ_SHARED_ENV

function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $1
   fi
   cat <<!

Usage: build [-f ReleaseConfigFile] [-Pause] [-r ReleaseName] [-ShowErrors]
             [-SkipCreate] [-SkipDeliver] [-SkipErrors] [-SkipExtract]
             [-SkipMake] [-SkipPostBuild]

            -f: Release configuration file (.rel) to use.
        -Pause: Pause before each major step in the build process.
            -r: The Release name and number to be built.
   -ShowErrors: Show the build errors without asking for confirmation.
   -SkipCreate: Skip the creation of the release.
  -SkipDeliver: Skip the deliver of the release.
   -SkipErrors: Skip the display of the build errors.
  -SkipExtract: Skip the extraction of the release.
     -SkipMake: Skip the make of the release.
-SkipPostBuild: Skip the execution of the post build script.

!
   exit 1
}

BRANCH_TYPE=MAIN
AUTO_NUMBER=TRUE
NEW_RELEASE=
PAUSE=FALSE
SHOW_ERRORS=FALSE
typeset -u DO_CREATE
typeset -u DO_DELIVER
typeset -u DO_ERRORS
typeset -u DO_EXTRACT
typeset -u DO_MAKE
typeset -u DO_POST_BUILD

typeset -l LOWERCASE_ARG
while [ "$#" -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
              -f) RELEASE_FILE="$2"
                  shift
                  shift ;;
          -pause) PAUSE=TRUE
                  shift ;;
              -r) NEW_RELEASE="$2"
                  shift
                  shift ;;
     -showerrors) SHOW_ERRORS=TRUE
                  shift ;;
     -skipcreate) DO_CREATE=N
                  shift ;;
    -skipdeliver) DO_DELIVER=N
                  shift ;;
     -skiperrors) DO_ERRORS=N
                  shift ;;
    -skipextract) DO_EXTRACT=N
                  shift ;;
       -skipmake) DO_MAKE=N
                  shift ;;
  -skippostbuild) DO_POST_BUILD=N
                  shift ;;
              -?) Usage ;;
               *) Usage ;;
  esac
done

if [ "${EZ_FAMILYTREE_ROOT#$EZ_BUILD_ROOT}" != "$EZ_FAMILYTREE_ROOT" ] ; then
   echo
   echo -n "Detected attempt to $0 from $EZ_ACTIVE_RELEASE, continue <y,n>? "
   read ANSWER
   if [ "$ANSWER" != Y -a "$ANSWER" != y ] ; then
      exit 1
   fi
fi

top
if [ "$RELEASE_FILE" = "" ] ; then
   RELEASE_FILE_LIST=`echo */*.rel`
   if [ "$RELEASE_FILE_LIST" = "*/*.rel" ] ; then
      cd ..
      FROM_TIP_TOP=TRUE
      RELEASE_FILE_LIST=`ls */*/*.rel | egrep -v "_build|rel_label"`
      if [ "$RELEASE_FILE_LIST" = "*_dev/*/*.rel" ] ; then
         RELEASE_FILE_LIST="*/*.rel"
      fi
   fi
   echo
   PS3="Please specify the desired release: "
   select RELEASE_FILE in $RELEASE_FILE_LIST
   do
      if [ "$RELEASE_FILE" != "" ] ; then
         break
      fi
   done
   if [ "$RELEASE_FILE" = "" ] ; then
      echo
      echo "Warning: Exiting since user aborted."
      exit 1
   fi
fi

if [ ! -f $RELEASE_FILE ] ; then
   echo
   echo "ERROR: Failed to find release configuration file $RELEASE_FILE."
   exit 1
fi
. $RELEASE_FILE

PLAIN_REL_NAME=${RELEASE_FILE%.rel}
PLAIN_REL_NAME=${PLAIN_REL_NAME##*/}

if [ "$REL_FILE_VERSION" != "5.0" ] ; then
   echo "ERROR: The REL_FILE_VERSION of '$REL_FILE_VERSION' is not supported."
   exit 1
fi

#===========================================================================
# Determine previous built releases that we are dependent upon.
#===========================================================================
TEMP_BUILT_LIST=
for ROOT_DIR in $BUILT_LIST
do
   SIMPLE_REL_NAME=${ROOT_DIR%_LastRelease}
   if [ "$SIMPLE_REL_NAME" = "$ROOT_DIR" ] ; then
      DEPENDENT_REL_NAME="$ROOT_DIR" 
   else
      DEPENDENT_REL_NAME=`last_name.ksh -r $SIMPLE_REL_NAME`
   fi
   if [ ! -d "$EZ_BUILD_ROOT/$DEPENDENT_REL_NAME" ] ; then
      echo
      echo -n "Warning: Dependent release $EZ_BUILD_ROOT/$DEPENDENT_REL_NAME "
      echo              "not found"
   fi
   TEMP_BUILT_LIST="$TEMP_BUILT_LIST $DEPENDENT_REL_NAME"
done
BUILT_LIST="$TEMP_BUILT_LIST"

#===========================================================================
# Determine previous delivered releases that we are dependent upon.
#===========================================================================
TEMP_RELEASED_LIST=
for ROOT_DIR in $RELEASED_LIST
do
   SIMPLE_REL_NAME=${ROOT_DIR%_LastRelease}
   if [ "$SIMPLE_REL_NAME" = "$ROOT_DIR" ] ; then
      DEPENDENT_REL_NAME="$ROOT_DIR" 
   else
      DEPENDENT_REL_NAME=`last_name.ksh -r $SIMPLE_REL_NAME`
   fi
   if [ ! -d "$EZ_DELIVER_ROOT/$DEPENDENT_REL_NAME" ] ; then
      echo
      echo -n "Warning: Dependent release $EZ_DELIVER_ROOT/$DEPENDENT_REL_NAME "
      echo              "not found"
   fi
   TEMP_RELEASED_LIST="$TEMP_RELEASED_LIST $DEPENDENT_REL_NAME"
done
RELEASED_LIST="$TEMP_RELEASED_LIST"

#===========================================================================
# Determine the new release name and number.
#===========================================================================
if [ "$NEW_RELEASE" = "" ] ; then
   LAST_NAME_ARGS="-r $PLAIN_REL_NAME +1"
   if [ "$APPLICATION_VERSION" != "" ] ; then
      OUR_APPLICATION_VERSION="$APPLICATION_VERSION"
      if [ "$APPLICATION_PATCH_LEVEL" != "" ] ; then
         OUR_APPLICATION_VERSION="$APPLICATION_VERSION.$APPLICATION_PATCH_LEVEL"
      fi
      LAST_NAME_ARGS="$LAST_NAME_ARGS -v $OUR_APPLICATION_VERSION"
   fi
   if [ "$BUILD_NUMBER_SYNC_NAME" != "" ] ; then
      LAST_NAME_ARGS="$LAST_NAME_ARGS -sync_name $BUILD_NUMBER_SYNC_NAME"
   fi
   if [ "$BUILD_NUMBER_SYNC_ORDER" != "" ] ; then
      LAST_NAME_ARGS="$LAST_NAME_ARGS -sync_order $BUILD_NUMBER_SYNC_ORDER"
   fi

   DEF_REL_NAME=`last_name.ksh $LAST_NAME_ARGS`
   if [ "$AUTO_NUMBER" != TRUE ] ; then
      echo -n "Please specify the new release name ($DEF_REL_NAME): "
      read ANSWER
      if [ "$ANSWER" = y ] ; then
         echo "Warning: You said '$ANSWER', but what you meant is $DEF_REL_NAME!"
         echo "Warning: Don't worry, I have already automatically fixed it for you!"
         ANSWER=
      fi
   fi

   if [ "$ANSWER" = "" ] ; then
      NEW_RELEASE=$DEF_REL_NAME
   else
      NEW_RELEASE=$ANSWER
   fi
else
   EXT_NEW_REL_FILE="$EZ_FAMILYTREE_ROOT/Releases/Built/$NEW_RELEASE.ksh"
   src.ksh extract -quiet $EXT_NEW_REL_FILE
   if [ ! -f "$EXT_NEW_REL_FILE" ] ; then
      cd "$EZ_FAMILYTREE_ROOT/Releases/Built"
      svn export $EZ_SOURCE_CONTROL_ROOT/Releases/$NEW_RELEASE/FamilyTree/Releases/Built/$NEW_RELEASE.ksh | egrep -v 'Export complete.|A    '
      if [ ! -f "$EXT_NEW_REL_FILE" ] ; then
         echo "ERROR: Failed to extract $EXT_NEW_REL_FILE"
         exit 1
      else
         RM_THIS_FILE="$EXT_NEW_REL_FILE"
      fi
   fi
fi

if [ "$BRANCH_RELEASE" = "" ] ; then
   echo "ERROR: Must specify if the release is to be branched or not."
   exit 1
fi
export REL_MAKE_EXIT_CODE=0

function BuildRelease
{
#===========================================================================
# Perform the release creation.
#===========================================================================
if [ "$DO_CREATE" = "" ] ; then
   if [ "$PAUSE" = "TRUE" ] ; then
      echo -n "Would You Like To Create The Release? <y,n> "
      read DO_CREATE
   else
      DO_CREATE=Y
   fi
fi

if [ "$DO_CREATE" = Y ] ; then
   if [ "$RELEASED_LIST" != "" -o "$BUILT_LIST" != "" ] ; then
      echo "Successfully determined dependencies: $RELEASED_LIST $BUILT_LIST"
   fi

   #===========================================================================
   # Create the Releases/Built/<<ReleaseName>>.ksh file.
   #===========================================================================
   RELEASE_INFO_DIR=$EZ_FAMILYTREE_ROOT/Releases/Built
   RELEASE_INFO_FILE=$NEW_RELEASE.ksh
   cd $RELEASE_INFO_DIR

   REL_BUILD_DATE=`date '+%m/%d/%y %H:%M:%S'`

   cat > $RELEASE_INFO_FILE <<!
#===========================================================================
#  Filename: $RELEASE_INFO_FILE
#    Author: Craig Schneider
# Orig Date: $(date +%D)
#  synopsis: Environment file for release $NEW_RELEASE
#===========================================================================

REL_BUILD_DATE='$REL_BUILD_DATE'
REL_FILE_VERSION='$REL_FILE_VERSION'
BRANCH_TYPE='$BRANCH_TYPE'
BUILD_MODE='$BUILD_MODE'
SOURCED_LIST='$SOURCED_LIST'
BUILT_LIST='$BUILT_LIST '
RELEASED_LIST='$RELEASED_LIST '
DELIVER_LIST='$DELIVER_LIST'
DELIVER_TARGET_SYS_DIR='$DELIVER_TARGET_SYS_DIR'
DELIVER_TARGET_SYS_PREFIX='$DELIVER_TARGET_SYS_PREFIX'
TURNOVER_DELIVER_LIST='$TURNOVER_DELIVER_LIST'

export THE_RELEASE_NAME=$NEW_RELEASE

!
   if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
      cat >> $RELEASE_INFO_FILE <<!
export EZ_RELEASE_ROOT="Releases/$NEW_RELEASE"
!
   else # "$SOURCE_CONTROL_SYSTEM" = vss
      cat >> $RELEASE_INFO_FILE <<!
export EZ_RELEASE_ROOT="\\\$/Releases/$NEW_RELEASE"
!
   fi

   THE_BUILD_ROOT="$EZ_BUILD_ROOT"
   THE_DELIVER_ROOT="$EZ_DELIVER_ROOT"

   #===========================================================================
   # Grab any necessary release settings from another file.
   #===========================================================================
   if [ "$RELEASE_SETTINGS_FILE" != "" ] ; then
      if [ ! -f "$RELEASE_SETTINGS_FILE" ] ; then
         echo "ERROR: Release settings file $RELEASE_SETTINGS_FILE not found."
         exit 1
      fi
      src_section.ksh "# RELEASE SETTINGS BEGIN" "# RELEASE SETTINGS END" \
                      $RELEASE_SETTINGS_FILE >> $RELEASE_INFO_FILE
      echo "Successfully applied settings from ${RELEASE_SETTINGS_FILE##*/}."
   fi

   #===========================================================================
   # Setup the necessary active family roots.
   #===========================================================================
   if [ "$OVERRIDE_FAM_ROOTS_LIST" = "" ] ; then
      OVERRIDE_FAM_ROOTS_LIST="$SOURCED_LIST"
   fi
   echo "export EZ_FAM_ROOTS=\" \\" >> $RELEASE_INFO_FILE
   for ROOT_DIR in $OVERRIDE_FAM_ROOTS_LIST
   do
      echo "\$EZ_BUILD_ROOT/\$THE_RELEASE_NAME/$ROOT_DIR \\" \
           >> $RELEASE_INFO_FILE
   done
   echo "\"" >> $RELEASE_INFO_FILE
   echo ""   >> $RELEASE_INFO_FILE

   #===========================================================================
   # Setup the necessary built release family roots.
   #===========================================================================
   for ROOT_DIR in $BUILT_LIST
   do
      typeset -u UP_NAME
      ROOT_DIR_NAME=${ROOT_DIR%_*}
      ROOT_DIR_NAME=${ROOT_DIR_NAME%-*}
      UP_NAME=$ROOT_DIR_NAME
      cat >> $RELEASE_INFO_FILE <<!
EZSetRoot EZ_${UP_NAME}_ROOT $THE_BUILD_ROOT/$ROOT_DIR/$ROOT_DIR_NAME TRUE
!
   done

   #===========================================================================
   # Setup the necessary delivered release family roots.
   #===========================================================================
   for ROOT_DIR in $RELEASED_LIST
   do
      typeset -u UP_NAME
      ROOT_DIR_NAME=${ROOT_DIR%_*}
      ROOT_DIR_NAME=${ROOT_DIR_NAME%-*}
      UP_NAME=$ROOT_DIR_NAME
      cat >> $RELEASE_INFO_FILE <<!
EZSetRoot EZ_${UP_NAME}_ROOT $THE_DELIVER_ROOT/$ROOT_DIR TRUE
!
   done

   #===========================================================================
   # Log the logical reference if any exists
   #===========================================================================
   if [ "$LOGICAL_REFERENCE" != "" ] ; then
      echo "LOGICAL_REFERENCE='$LOGICAL_REFERENCE'" >> $RELEASE_INFO_FILE
      echo "" >> $RELEASE_INFO_FILE
      echo "Successfully logged logical reference: $LOGICAL_REFERENCE"
   fi
   if [ "$LAST_RELEASE_REFERENCE" != "" ] ; then
      echo "LAST_RELEASE_REFERENCE='$LAST_RELEASE_REFERENCE'" \
           >> $RELEASE_INFO_FILE
      echo "" >> $RELEASE_INFO_FILE
      echo "Successfully logged last release reference: $LAST_RELEASE_REFERENCE"
   fi

   #===========================================================================
   # Hold onto any destroy directories for the time of rel destroy.
   #===========================================================================
   if [ "$DESTROY_DIRECTORIES" != "" ] ; then
      DESTROY_DIRECTORIES=`echo "$DESTROY_DIRECTORIES" | \
           sed -e "s/NEW_RELEASE/$NEW_RELEASE/g"`
      echo "DESTROY_DIRECTORIES=\"$DESTROY_DIRECTORIES\"" >> $RELEASE_INFO_FILE
      echo "" >> $RELEASE_INFO_FILE
      echo "Successfully logged destroy directories: $DESTROY_DIRECTORIES"
   fi

   #===========================================================================
   # Give the caller a chance to fix up the release info file.
   #===========================================================================
   if [ "$RELEASE_INFO_FIXER" != "" ] ; then
      export BRANCH_TYPE
      eval $RELEASE_INFO_FIXER $RELEASE_INFO_DIR/$RELEASE_INFO_FILE
   fi

   #===========================================================================
   # strip Control M's from this file so it can be used under windows & solaris
   #===========================================================================
   if [ "$EZ_OS_TYPE" = windows ] ; then
      dos2unix.sh $RELEASE_INFO_FILE
   fi

   src.ksh create $RELEASE_INFO_FILE -m "Creating Release $NEW_RELEASE"

   ## Used to try and create in working stream.
   #EZ_REL_DIR="${EZ_ENV_DIR_ORIG%/*}/Releases/Built"
   #if [ ! -f $EZ_REL_DIR/$RELEASE_INFO_FILE ] ; then
   #   # copy into EZ_ENV_DIR_ORIG if not there already for later dotting
   #   cp $RELEASE_INFO_DIR/$RELEASE_INFO_FILE $EZ_REL_DIR/$RELEASE_INFO_FILE
   #   if [ "$BRANCH_TYPE" = PATCH ] ; then
   #      PREV_PWD=$PWD
   #      cd $EZ_REL_DIR
   #      chmod 777 $RELEASE_INFO_FILE
   #      src.ksh create $RELEASE_INFO_FILE -m "Creating Release $NEW_RELEASE"
   #      cd $PREV_PWD
   #   else
   #      chmod 555 $EZ_REL_DIR/$RELEASE_INFO_FILE    # for future refreshes
   #   fi
   #fi

   #===========================================================================
   # Update the version information in the .cs, .config & .ksh files specified.
   #===========================================================================

   ORIG_PWD="$PWD"
   BUILD_DATE=`date +'%b %d, %Y'`
   BUILD_DATE_CS=`date +'%Y, %_m, %_d, %_H, %_M, %_S'`
   BUILD_NUMBER=${NEW_RELEASE##*.}
   PARTIAL1=${NEW_RELEASE%.$BUILD_NUMBER}
   MINOR_NUMBER=${PARTIAL1##*.}
   PARTIAL2=${PARTIAL1%.$MINOR_NUMBER}
   MAJOR_NUMBER=${PARTIAL2##*_}
   BUILD_MAJOR_MINOR=$MAJOR_NUMBER.$MINOR_NUMBER
   BUILD_VERSION=$MAJOR_NUMBER.$MINOR_NUMBER.$BUILD_NUMBER
   COMMENTS_VERSION="VersionComments=\"$(date +'Build Date: %b %d, %Y')\""

   for VERSION_FILEENTRY in $VERSION_FILE_LIST
   do
      VERSION_FAMILY=${VERSION_FILEENTRY%:*}
      VERSION_FILESPEC=${VERSION_FILEENTRY#*:}
      typeset -u ROOT_VAR
      ROOT_VAR="\$EZ_${VERSION_FAMILY}_ROOT"
      eval cd $ROOT_VAR
      SAVE_PWD=$PWD
      for VERSION_FILEPATH in `find . -name $VERSION_FILESPEC -print`
      do
         cd $SAVE_PWD
         VERSION_FILE=${VERSION_FILEPATH##*/}
         VERSION_DIR=${VERSION_FILEPATH%/$VERSION_FILE}
         VERSION_EXTENSION=${VERSION_FILE##*.}

         cd $VERSION_DIR

         src.ksh test $VERSION_FILE
         if [ $? -ne 0 ] ; then     # no need to see this warning
            #echo "Warning: Skipping non source controlled file $VERSION_FILE"
            continue
         fi

         if [ "$VERSION_EXTENSION" = cs ] ; then
            BUILD_VERSION_LINE=`grep -n "string BuildVersionString" \
                                $VERSION_FILE | cut -f1 -d:`
            BUILD_DATE_LINE=`grep -n "DateTime BuildDateTime" $VERSION_FILE | \
                             cut -f1 -d:`
            sed -e "${BUILD_VERSION_LINE}s/\"[0-z., ]*\"/\"$BUILD_VERSION\"/" \
                -e "${BUILD_DATE_LINE}s/([0-z, ]*);/($BUILD_DATE_CS);/" \
                $VERSION_FILE > $VERSION_FILE.ver
         elif [ "$VERSION_EXTENSION" = "ksh" -o \
                "$VERSION_EXTENSION" = "config" ] ; then
            sed -e "s/BUILD_DATE=[ -~]*/BUILD_DATE='$BUILD_DATE'/" \
                -e "s/BUILD_NUMBER=[ -~]*/BUILD_NUMBER=$BUILD_NUMBER/" \
                -e "s/BUILD_VERSION=[ -~]*/BUILD_VERSION=$BUILD_VERSION/" \
                -e "s/BUILD_MAJOR_MINOR=[ -~]*/BUILD_MAJOR_MINOR=$BUILD_MAJOR_MINOR/" \
                -e "s/APPLICATION_NAME=[ -~]*/APPLICATION_NAME='$APPLICATION_NAME'/" \
                $VERSION_FILE > $VERSION_FILE.ver
            if [ "$EZ_OS_TYPE" = windows ] ; then
               dos2unix.sh $VERSION_FILE.ver
            fi
         elif [ "$VERSION_EXTENSION" = "settings" ] ; then
            sed -e "s/Key:\"ReleaseBuildVersion\",[ ]*Value:\"[ -~]*\"/Key:\"ReleaseBuildVersion\", Value:\"$BUILD_VERSION\"/" \
                -e "s/Key:\"ReleaseBuildDate\",[ ]*Value:\"[ -~]*\"/Key:\"ReleaseBuildDate\", Value:\"$BUILD_DATE\"/" \
                $VERSION_FILE > $VERSION_FILE.ver
            if [ "$EZ_OS_TYPE" = windows ] ; then
               dos2unix.sh $VERSION_FILE.ver
            fi
         else
            echo "Warning: Unable to set version info in $VERSION_FILEPATH."
         fi

         src.ksh stabalize -r -quiet -yes \
                           -m "Forced version to $BUILD_VERSION" \
             $VERSION_FILE.ver $VERSION_FILE
         echo "Successfully set the version to $BUILD_VERSION in $VERSION_FILE"
      done
   done
   cd "$ORIG_PWD"

   #===========================================================================
   # We made all of our source control changes, now branch the release.
   #===========================================================================
   typeset -u ROOT_VAR
   if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
      svn mkdir "$EZ_SOURCE_CONTROL_ROOT/Releases/$NEW_RELEASE" \
                -m "Branched release $NEW_RELEASE" | sed -e '/^Committ/d'
      for ROOT_DIR in $SOURCED_LIST
      do
         svn copy "$EZ_SOURCE_CONTROL_ROOT/$EZ_RELEASE_ROOT/$ROOT_DIR" \
                  "$EZ_SOURCE_CONTROL_ROOT/Releases/$NEW_RELEASE/$ROOT_DIR" \
                  -m "Branched release $NEW_RELEASE" | sed -e '/^Committ/d'
         echo "Successfully branched $NEW_RELEASE/$ROOT_DIR."
      done
   elif [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
      if [ "$BRANCH_RELEASE" = Y ] ; then
         ss Create $/Releases/$NEW_RELEASE -Csplit $EZ_VSS_OPTIONS | \
            grep -v $NEW_RELEASE
      fi
      for ROOT_DIR in $SOURCED_LIST
      do
         ss Label $EZ_RELEASE_ROOT/$ROOT_DIR -L$NEW_RELEASE $EZ_VSS_OPTIONS \
             "-CCreating Release $NEW_RELEASE" | grep -v "^$ROOT_DIR\$"
         echo "Successfully applied label $NEW_RELEASE to \c"
         echo "$EZ_RELEASE_ROOT/$ROOT_DIR"
         if [ "$BRANCH_RELEASE" = Y ] ; then
            ss CP $/Releases/$NEW_RELEASE $EZ_VSS_OPTIONS | grep -v $NEW_RELEASE
            ss Share $EZ_RELEASE_ROOT/$ROOT_DIR -R -E -Cbranched \
                 $EZ_VSS_OPTIONS | grep -v $NEW_RELEASE       # share but branch
            echo "Successfully shared $ROOT_DIR files in source safe."
         fi
      done
   else
      echo "ERROR: Unsupported source control system for branching."
      exit 1
   fi
fi

#===========================================================================
# Perform the release extraction.
#===========================================================================
if [ "$DO_EXTRACT" = "" ] ; then
   if [ "$PAUSE" = "TRUE" ] ; then
      echo -n "Would You Like To Extract The Release? <y,n> "
      read DO_EXTRACT
   else
      DO_EXTRACT=Y
   fi
fi

if [ $DO_EXTRACT = Y ] ; then
   mkdir -p $EZ_BUILD_ROOT/$NEW_RELEASE
   if [ $? -ne 0 ] ; then
      echo -n "ERROR: Failed creating the build directory "
      echo           "$EZ_BUILD_ROOT/$NEW_RELEASE."
      exit 1
   fi
   cd $EZ_BUILD_ROOT/$NEW_RELEASE
   rel.sh extract $NEW_RELEASE
   if [ $? -ne 0 ] ; then
      echo "ERROR: Encountered a failure during rel extract."
      exit 1
   fi
fi

#===========================================================================
# Set the environment to the new release.
#===========================================================================
rel set $NEW_RELEASE
top

if [ "$DO_MAKE" = "" ] ; then
   if [ "$PAUSE" = "TRUE" ] ; then
      echo -n "Would You Like To Make The Release? <y,n> "
      read DO_MAKE
   else
      DO_MAKE=Y
   fi
fi

if [ $DO_MAKE = Y ] ; then
   for ROOT_DIR in $RELEASED_LIST $BUILT_LIST
   do
      eval CONFIG_COMMAND=Config${ROOT_DIR%_*}
      whence $CONFIG_COMMAND >$EZ_DEV_NULL
      if [ $? -eq 0 ] ; then
         $CONFIG_COMMAND
      fi
   done
   rel.sh make $NEW_RELEASE $RELEASE_VERIFY_OPTIONS
   REL_MAKE_EXIT_CODE=$?
   if [ $REL_MAKE_EXIT_CODE -ne 0 ] ; then
      return $REL_MAKE_EXIT_CODE
   fi

   if [ "$MULTI_PLATFORM" = TRUE ] ; then
      OTHER_OS=windows
      if [ "$EZ_OS_TYPE" = windows ] ; then
         OTHER_OS=solaris
      fi
      if [ "$REMOTE_HOST" != "" ] ; then
         RemoteRun.ksh -h $REMOTE_HOST rel.sh make $NEW_RELEASE \
                                             $RELEASE_VERIFY_OPTIONS
         REMOTE_RUN_EXIT_CODE=$?
         if [ $REMOTE_RUN_EXIT_CODE -ne 0 ] ; then
            return $REMOTE_RUN_EXIT_CODE
         fi
      else
         echo -n "Successfully PAUSING for $OTHER_OS: "
         echo -n "rel.sh make $NEW_RELEASE "
         read WAIT_4_IT
      fi
   fi
else
   echo "Successfully skipping build of $NEW_RELEASE."
fi

DO_DELIVER_ORIG="$DO_DELIVER"
for ONE_DELIVER_FILE in $DELIVER_LIST
do
   DO_DELIVER="$DO_DELIVER_ORIG"
   if [ "$DO_DELIVER" = "" ] ; then
      if [ "$PAUSE" = "TRUE" ] ; then
         echo -n "Would you like to deliver $ONE_DELIVER_FILE? <y,n> "
         read DO_DELIVER
      else
         DO_DELIVER=Y
      fi
   fi

   if [ "$DO_DELIVER" = Y ] ; then
      top
      DELIVER_FILE_KSH=`echo $PWD/*/$ONE_DELIVER_FILE`
      DELIVER_FILE=`EZPath2Win $DELIVER_FILE_KSH`
      if [ ! -f $DELIVER_FILE ] ; then
         echo "ERROR: Failed to find deliver file $DELIVER_FILE."
         exit 1
      fi
      DELIVER_FILE_DIR=${DELIVER_FILE%/*}
      DELIVER_FILE_NAME=${DELIVER_FILE##*/}
      DELIVER_EXTENSION=${DELIVER_FILE##*.}
      DELIVER_FILE_PREFIX=${DELIVER_FILE_NAME%.deliver}
      DELIVER_TARGET_ROOT="$EZ_DELIVER_ROOT"
      if [ "$DELIVER_TARGET_SYS_DIR" != "" ] ; then
         DELIVER_TARGET_ROOT="$DELIVER_TARGET_ROOT/$DELIVER_TARGET_SYS_DIR"
      fi
      if [ "$DELIVER_TARGET_SYS_PREFIX" != "" ] ; then
         DELIVER_TARGET_ROOT="$DELIVER_TARGET_ROOT/$DELIVER_TARGET_SYS_PREFIX"
         DELIVER_TARGET_ROOT="$DELIVER_TARGET_ROOT$BUILD_VERSION"
      fi
      mkdir -p "$DELIVER_TARGET_ROOT"

      if [ "$DELIVER_EXTENSION" = "deliver" ] ; then
         C4.sh Deliver -f $DELIVER_FILE \
            -t $DELIVER_TARGET_ROOT/${DELIVER_FILE_PREFIX}_$BUILD_VERSION
      elif [ "$DELIVER_EXTENSION" = "ksh" ] ; then
         . $DELIVER_FILE
      else
         echo "Warning: Unrecognized deliver file extension $ONE_DELIVER_FILE."
      fi
   fi
done

#===========================================================================
# Perform the post build script.
#===========================================================================
if [ "$POST_BUILD_COMMAND" != "" ] ; then
   if [ "$DO_POST_BUILD" = "" ] ; then
      if [ "$PAUSE" = "TRUE" ] ; then
         echo -n "Would You Like To Execute The Post Build? <y,n> "
         read DO_POST_BUILD
      else
         DO_POST_BUILD=Y
      fi
   fi

   if [ "$DO_POST_BUILD" = Y ] ; then
      eval $POST_BUILD_COMMAND -r $NEW_RELEASE
   fi
fi

return 0
}

LOG_QUALIFIER=
if [ "$EZ_OS_TYPE" != windows ] ; then
   LOG_QUALIFIER=".$EZ_OS_TYPE"
fi
REL_BUILD_LOG="$TMPDIR/rel_${NEW_RELEASE}$LOG_QUALIFIER.log"
REL_BUILD_ERR="$TMPDIR/rel_${NEW_RELEASE}$LOG_QUALIFIER.err"

BuildRelease 2>&1 | tee "$REL_BUILD_LOG"

if [ "$DO_ERRORS" != N ] ; then
   echo
   # For some unknown reason something in the AppBase tests in C4BaseCore
   # outputs a list of like 16593 nulls.  The tr command deletes those.
   cat "$REL_BUILD_LOG" | tr -d '\0' | \
      egrep -v "^Successfully|\*\*\*\*\*\*\*\*\*\*\*\*\*|Encountered Failures on    0 of the" | sed '/^$/d' > "$REL_BUILD_ERR"

   TOTAL_LINES=`wc -l "$REL_BUILD_LOG" | awk '{ print $1 }'`
   TOTAL_SUCC=`grep "^Successfully" "$REL_BUILD_LOG" | wc -l | \
                                                 awk '{ print $1 }'`
   TOTAL_ERRORS=`wc -l "$REL_BUILD_ERR" | awk '{ print $1 }'`
   TOTAL_CHECKOUTS=`grep "has checkouts." "$REL_BUILD_ERR" | \
                    wc -l | awk '{ print $1 }'`

   if [ "$TOTAL_ERRORS" != 0 -a "$TOTAL_CHECKOUTS" = "$TOTAL_ERRORS" ] ; then
      echo "Successfully detected $TOTAL_CHECKOUTS activities having checkouts."
      TOTAL_ERRORS=0
   fi
   if [ "$TOTAL_ERRORS" = 0 ] ; then
      echo -n "Successfully completed ALL $TOTAL_SUCC tasks "
      echo    "creating release $NEW_RELEASE"
   else
      echo -n "Successfully completed $TOTAL_SUCC tasks "
      echo    "creating release $NEW_RELEASE"
      echo
      if [ "$SHOW_ERRORS" = TRUE ] ; then
         echo "Warning: detected the following $TOTAL_ERRORS unsuccessful lines."
         echo
         cat "$REL_BUILD_ERR" 
      else
         echo -n "Warning: Would you like to see the $TOTAL_ERRORS "
         echo -n "unsuccessful lines? "
         typeset -u ANSWER
         read ANSWER
         if [ "$ANSWER" = Y ] ; then
            echo
            echo
            cat "$REL_BUILD_ERR"
         fi
      fi
   fi
   if [ -d $EZ_BUILD_ROOT/$NEW_RELEASE ] ; then
      cp "$REL_BUILD_LOG" $EZ_BUILD_ROOT/$NEW_RELEASE/BuildLog.txt
      cp "$REL_BUILD_ERR" $EZ_BUILD_ROOT/$NEW_RELEASE/ErrorLog.txt
   else
      echo -n "ERROR: Unable to copy $REL_BUILD_LOG to "
      echo    "$EZ_BUILD_ROOT/$NEW_RELEASE/BuildLog.txt"
      echo -n "ERROR: Unable to copy $REL_BUILD_ERR to "
      echo    "$EZ_BUILD_ROOT/$NEW_RELEASE/ErrorLog.txt"
   fi
fi

if [ "$RM_THIS_FILE" != "" ] ; then
   rm -f "$RM_THIS_FILE"
fi
exit $REL_MAKE_EXIT_CODE
