#===========================================================================
#  Filename: rel_lock.ksh
#    Author: Craig Schneider
# Orig Date: 05/21/01
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Lock/unlock a specific release.
#===========================================================================

. $EZ_SHARED_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function usage
{
   if [ "$1" != "" ] ; then
      echo
      echo "$1"
   fi
   echo
   echo "Usage: lock {-lock|-unlock} [release_name]"
   echo 
   exit 1
}

#===========================================================================
# The Main.
#===========================================================================
if [ "$#" -gt 2 -o "$#" -lt 1 ]; then
    usage "Incorrect number of arguments."
fi

TYPE=""
while [ "$#" -ne 0 ]
do
   case $1 in
     -lock|-l)   TYPE=LOCK
                 shift ;;
     -unlock|-u) TYPE=UNLOCK
                 shift ;;
     -?)         usage ;;
     *)          if [ "$#" -gt 0 -a "$TYPE" = "" ]; then
                    usage
                 fi
                 LOCK_REL_NAME=$1
                 shift ;;
    esac
done        

RELEASE_INFO_DIR=$EZ_FAMILYTREE_ROOT/Releases/Built
if [ ! -d "$RELEASE_INFO_DIR" ] ; then
   echo "ERROR: Release info directory $RELEASE_INFO_DIR not found!"
   exit 1
fi

if [ "${EZ_FAMILYTREE_ROOT#$EZ_BUILD_ROOT}" != "$EZ_FAMILYTREE_ROOT" ] ; then
   echo
   echo -n "Detected attempt to $0 from $EZ_ACTIVE_RELEASE, continue <y,n>? "
   read ANSWER
   if [ "$ANSWER" != Y -a "$ANSWER" != y ] ; then
      exit 1
   fi
fi

#
# determine the release to lock/unlock
#
rel_pick -a $LOCK_REL_NAME
if [ "$THE_RELEASE" = "" ] ; then
   echo "ERROR: Failed To Select A Release, Quitting!"
   exit 1
fi

RELEASE_INFO_FILE=$THE_RELEASE.ksh
echo

cd $RELEASE_INFO_DIR
if [ ! -f $RELEASE_INFO_FILE ] ; then
   src.ksh extract $RELEASE_INFO_FILE
fi

# 
# determine if this is a wasted effort ie. locking if already locked
#
egrep "LOCKED=|DELIVER_LIST" $RELEASE_INFO_FILE > $TMPDIR/rel_lock.tmp
RETURN_CODE=$?
. $TMPDIR/rel_lock.tmp

if [ "$LOCKED" = "TRUE" -a "$TYPE" = "LOCK" ]; then
    echo 
    echo "ERROR: Release $THE_RELEASE is already locked."
    exit 1
elif [ "$RETURN_CODE" != 0 -a "$TYPE" = "UNLOCK" ]; then
    echo
    echo "ERROR: Release $THE_RELEASE is not currently locked."
    exit 1
fi

RUN_WHOAMI=`whence whoami`
CURRENT_USER=`$RUN_WHOAMI`

function LockDirectory
{
   DIR_TO_LOCK="$1"

   if [ -d $DIR_TO_LOCK ] ; then
      # check to see if we own all of the files and are on unix
      find $DIR_TO_LOCK ! -user $CURRENT_USER > $TMPDIR/rel_lock.tmp
      FILE_LENGTH=`wc -l $TMPDIR/rel_lock.tmp | awk '{print $1}'`
      if [ "${FILE_LENGTH}" -gt 0 ]; then
         # can only run take_ownership on unix
         if [ "$EZ_OS_TYPE" != "windows" ]; then
            /usr/local/bin/take_ownership.ksh $DIR_TO_LOCK $CURRENT_USER \
                                              rel_lock.ksh
         else
            echo -n "Warning: Running from $EZ_OS_TYPE, will not change "
            echo    "file ownerships."
            BYPASS_CHMOD_ERROR=TRUE
         fi
      fi

      # lock or unlock release directory
      if [ "$TYPE" = "LOCK" ]; then
          chmod -R 555 $DIR_TO_LOCK
      elif [ "$TYPE" = "UNLOCK" ]; then
          chmod -R 777 $DIR_TO_LOCK
      fi    
      if [ $? -ne 0 ] ; then
         # allow to bypass chmod fail if not on unix
         if [ "$BYPASS_CHMOD_ERROR" = "TRUE" ]; then
            echo -n "Warning: Failed to $TYPE $DIR_TO_LOCK possibly due "
            echo -n "to OS type, continue <y,n>? "
            read ANSWER
            if [ "$ANSWER" != Y -a "$ANSWER" != y ] ; then
               exit 1
            fi
         else
            echo "ERROR: Failed to $TYPE directory $DIR_TO_LOCK."
            exit 1
         fi
      fi
      echo "Successfully ${TYPE}ED release directory ${DIR_TO_LOCK}"
   fi
}

if [ "OLD_LOCKING" = "TRUE" ] ; then
   COMPACT_VERSION=${THE_RELEASE##*_}
   for DELIVER_FILE in $DELIVER_LIST
   do
      DELIVER_FILE_NAME=${DELIVER_FILE##*/}
      DELIVER_DIR_NAME=${DELIVER_FILE_NAME%.deliver}
      DELIVER_DIR_NAME=${DELIVER_DIR_NAME}_$COMPACT_VERSION
      ARCHIVE_DIR_NAME=$EZ_ARCHIVE_ROOT/$DELIVER_DIR_NAME
      RELEASE_DIR_NAME=$EZ_DELIVER_ROOT/$DELIVER_DIR_NAME
   
      LockDirectory "$RELEASE_DIR_NAME"
   done

   LockDirectory "$EZ_BUILD_ROOT/$THE_RELEASE"
fi

function UpdateReleaseInfoFile
{
   UPDATE_TEXT="$1"
   if [ ! -f $RELEASE_INFO_FILE ] ; then
      echo "ERROR: Failed to find file $RELEASE_INFO_FILE to update status."
      exit 1
   fi

   TMP_FILE="$TMPDIR/$RELEASE_INFO_FILE.fix"
   sed '/^LOCKED/d' $RELEASE_INFO_FILE > "$TMP_FILE"
   if [ "$TYPE" = "LOCK" ]; then
      sed '/RELEASED_LIST/a\
LOCKED=TRUE' "$TMP_FILE" > "$TMP_FILE.fix"
      mv "$TMP_FILE.fix" "$TMP_FILE"
   fi
   src.ksh stabalize -yes -m "Updated release lock status to $TYPE." \
          -t stabalized "$UPDATE_TEXT" -t unchanged "" \
          "$TMP_FILE" $RELEASE_INFO_FILE
}

# Set the release file to locked or unlocked
cd $RELEASE_INFO_DIR
UpdateReleaseInfoFile "Successfully updated lock status in $THE_RELEASE.ksh."

# Set the working release file to locked or unlocked
EZ_REL_DIR="${EZ_ENV_DIR_ORIG%/*}/Releases/Built"
if [ -f $EZ_REL_DIR/$THE_RELEASE.ksh ] ; then
   PREV_PWD=$PWD
   cd $EZ_REL_DIR
   UpdateReleaseInfoFile "Successfully updated working version of $THE_RELEASE.ksh."
   cd $PREV_PWD
fi

if [ "$TYPE" = "LOCK" ]; then
   echo "Successfully locked the release $THE_RELEASE"
else
   echo "Successfully unlocked the release $THE_RELEASE"
fi

exit 0
