#===========================================================================
#  Filename: rel_turnover.ksh
#    Author: Craig Schneider
# Orig Date: 08/27/13
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Turnover a development release for installation and testing.
#===========================================================================

. $EZ_RELMGR_ENV

function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo "$1"
   fi
   cat <<!

Usage: rel turnover [-Confirm] [-DryRun] [-LimitVersion Version]
                    [-r ReleaseName] [-Subset SubsetName] [-Verbose]
                    [-Version VersionNumber] [-Yes]

      -Confirm: Confirm each turnover command before it is executed.
       -DryRun: Display the commands to be executed, without executing them.
 -LimitVersion: Limit to the specified version (e.g. 2.9).
    -NoRefresh: Do not perform a src refresh of FamilyTree.
            -r: Release name to be turned over, possibilities include: 
                $(EZRelNameLongList ", ")

       -Subset: Turn over a subset of the release.
      -Verbose: Display verbose messages about operations being performed.
      -Version: Version number to to be used (e.g. 2.9.0.B.1).
          -Yes: All user questions will be implicitly answered with a yes.

!
   exit 1
}

CONFIRM=FALSE
EZ_DRYRUN=FALSE
EZ_VERBOSITY=
LIMIT_VERSION=
RELEASE_NAME=
SRC_REFRESH_REQUIRED=TRUE
SUBSET_NAME=
VERSION_OVERRIDE=
YES_MODE=FALSE

typeset -l LOWERCASE_ARG
while [ "$#" -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -confirm) CONFIRM=TRUE
                shift ;;
       -dryrun) EZ_DRYRUN=TRUE
                shift ;;
 -limitversion) LIMIT_VERSION="$2"
                shift
                shift ;;
    -norefresh) SRC_REFRESH_REQUIRED=FALSE
                shift ;;
            -r) RELEASE_NAME="$2"
                shift
                shift ;;
       -subset) SUBSET_NAME="$2"
                shift
                shift ;;
      -verbose) EZ_VERBOSITY="${LOWERCASE_ARG#-}"
                shift ;;
      -version) VERSION_OVERRIDE="$2"
                shift
                shift ;;
          -yes) YES_MODE=TRUE
                shift ;;
             *) if [ "$1" != "-?" ] ; then
                   Usage "Invalid argument: $1"
                else
                   Usage
                fi ;;
   esac
done

# update the list of built releases
if [ "$SRC_REFRESH_REQUIRED" = TRUE ] ; then
   cd $EZ_FAMILYTREE_ROOT/Releases/Built
   src.ksh refresh
fi

RELEASE_NAME=`EZRelNameSelect "$RELEASE_NAME"`
#echo "RELEASE_NAME=$RELEASE_NAME"
REL_FILE_NAME=`EZRelGetFileByName "$RELEASE_NAME"`
#echo "REL_FILE_NAME=$REL_FILE_NAME"

if [ "$REL_FILE_NAME" = "" ] ; then
   echo "ERROR: Failed to determine .rel file name for release $RELEASE_NAME."
   exit 1
fi

eval `egrep "^DELIVER_LIST=|^DELIVER_TARGET_SYS_DIR=|^DELIVER_TARGET_SYS_PREFIX=|^REL_NAME=|^TURNOVER_DELIVER_LIST=|^TURNOVER_SUBSET_LIST=|_DELIVER_" $REL_FILE_NAME `

if [ "$SUBSET_NAME" = "" -a "$TURNOVER_SUBSET_LIST" != "" ] ; then
   echo
   PS3="Select Release Subset: "
   select SUBSET_NAME in $TURNOVER_SUBSET_LIST
   do
      if [ "$SUBSET_NAME" != "" ] ; then
         break
      fi
   done
   if [ "$SUBSET_NAME" = "All" ] ; then
      SUBSET_NAME=""
   fi
fi

REL_ROOT="$EZ_DELIVER_ROOT"
if [ "$DELIVER_TARGET_SYS_DIR" != "" ] ; then
   REL_ROOT="$REL_ROOT/$DELIVER_TARGET_SYS_DIR"
fi
#echo "REL_ROOT=$REL_ROOT"

if [ ! -d "$REL_ROOT" ] ; then
   echo "ERROR: Failed to find release directory $REL_ROOT."
   exit 1
else
   cd "$REL_ROOT"

   REL_NAME_PREFIX="$RELEASE_NAME"
   if [ "$DELIVER_TARGET_SYS_PREFIX" != "" ] ; then
      REL_NAME_PREFIX="${DELIVER_TARGET_SYS_PREFIX%_}"
   fi
   #echo "REL_NAME_PREFIX=$REL_NAME_PREFIX"
   REL_LAST_NAME_ARG="${REL_NAME_PREFIX}_*" 
   if [ "$LIMIT_VERSION" != "" ] ; then
      REL_LAST_NAME_ARG="${REL_NAME_PREFIX}_${LIMIT_VERSION}*"
   fi
   if [ "$EZ_VERBOSITY" = "verbose" ] ; then
      echo "Running last_name command in $(EZPath2Win $PWD)."
      echo "last_name.ksh $REL_LAST_NAME_ARG"
   fi
   REL_NAME_AND_VERSION=`last_name.ksh $REL_LAST_NAME_ARG`
   if [ "$REL_NAME_AND_VERSION" != "${REL_NAME_AND_VERSION%\*}" -a \
        "$DELIVER_TARGET_SYS_PREFIX" = "" ] ; then
      REL_LAST_NAME_ARG="${REL_NAME}_*" 
      if [ "$LIMIT_VERSION" != "" ] ; then
         REL_LAST_NAME_ARG="${REL_NAME}_${LIMIT_VERSION}*"
      fi
      if [ "$EZ_VERBOSITY" = "verbose" ] ; then
         echo "Running last_name command in $(EZPath2Win $PWD)."
         echo "last_name.ksh $REL_LAST_NAME_ARG"
      fi
      REL_NAME_AND_VERSION=`last_name.ksh $REL_LAST_NAME_ARG`
   fi
fi
#echo "REL_NAME_AND_VERSION=$REL_NAME_AND_VERSION"

REL_BUILD_VERSION="${REL_NAME_AND_VERSION##*_}"
REL_NAME_MAJOR_MINOR_PATCH="${REL_NAME_AND_VERSION%.*}"
REL_MAJOR_MINOR_PATCH="${REL_NAME_MAJOR_MINOR_PATCH##*_}"

#echo "REL_BUILD_VERSION=$REL_BUILD_VERSION"
#echo "REL_NAME_MAJOR_MINOR_PATCH=$REL_NAME_MAJOR_MINOR_PATCH"
#echo "REL_MAJOR_MINOR_PATCH=$REL_MAJOR_MINOR_PATCH"

QA_REL_ROOT="$EZ_QA_RELEASE_ROOT"
if [ "$DELIVER_TARGET_SYS_DIR" != "" ] ; then
   QA_REL_ROOT="$QA_REL_ROOT/$DELIVER_TARGET_SYS_DIR"
fi
#echo "QA_REL_ROOT=$QA_REL_ROOT"

if [ "$SUBSET_NAME" != "" ] ; then
   eval export SUBSET_TARGET_SYS_PREFIX=\"\$${SUBSET_NAME}_DELIVER_TARGET_SYS_PREFIX\"
   #echo "SUBSET_DELIVER_TARGET_SYS_PREFIX=$SUBSET_DELIVER_TARGET_SYS_PREFIX"
   eval export SUBSET_DELIVER_LIST=\"\$${SUBSET_NAME}_DELIVER_LIST\"
   #echo "SUBSET_DELIVER_LIST=$SUBSET_DELIVER_LIST"
fi

QA_REL_NAME_PREFIX="$REL_NAME_MAJOR_MINOR_PATCH"
if [ "$SUBSET_TARGET_SYS_PREFIX" != "" ] ; then
   QA_REL_NAME_PREFIX="${SUBSET_TARGET_SYS_PREFIX}${REL_MAJOR_MINOR_PATCH}"
elif [ "$DELIVER_TARGET_SYS_PREFIX" != "" ] ; then
   QA_REL_NAME_PREFIX="$DELIVER_TARGET_SYS_PREFIX${REL_MAJOR_MINOR_PATCH}"
fi
#echo "QA_REL_NAME_PREFIX=$QA_REL_NAME_PREFIX"

if [ "$VERSION_OVERRIDE" != "" ] ; then
   QA_REL_NAME_AND_VERSION="${QA_REL_NAME_PREFIX%_*}_$VERSION_OVERRIDE"
elif [ ! -d "$QA_REL_ROOT" ] ; then
   QA_REL_NAME_AND_VERSION="${QA_REL_NAME_PREFIX}.A.1"
else
   cd "$QA_REL_ROOT"
   FILE_LIST=`echo $QA_REL_NAME_PREFIX*` 
   if [ "$FILE_LIST" = "$QA_REL_NAME_PREFIX*" ] ; then
      QA_REL_NAME_AND_VERSION="${QA_REL_NAME_PREFIX}.A.1"
   else
      if [ "$EZ_VERBOSITY" = "verbose" ] ; then
         echo "Running last_name command in $(EZPath2Win $PWD)."
         echo "last_name.ksh +1 ${QA_REL_NAME_PREFIX}*"
      fi
      QA_REL_NAME_AND_VERSION=`last_name.ksh +1 ${QA_REL_NAME_PREFIX}*`
   fi
fi

#echo "QA_REL_NAME_AND_VERSION=$QA_REL_NAME_AND_VERSION"
QA_REL_ROOT="$QA_REL_ROOT/$QA_REL_NAME_AND_VERSION"
#echo "QA_REL_ROOT=$QA_REL_ROOT"
if [ "$YES_MODE" = FALSE ] ; then
   echo
   echo "Turnover Release To $QA_REL_ROOT  <y,n>? \c"
   read ANSWER
else
   ANSWER="y"
fi
if [ "$ANSWER" != y -a "$ANSWER" != Y ] ; then
  exit 1
fi

QA_FULL_DELIVER_LIST="$DELIVER_LIST $TURNOVER_DELIVER_LIST"
if [ "$SUBSET_DELIVER_LIST" != "" ] ; then
   QA_FULL_DELIVER_LIST="$SUBSET_DELIVER_LIST"
fi

for DELIVER_FILE in $QA_FULL_DELIVER_LIST
do
   DELIVER_FILE_NAME=${DELIVER_FILE##*/}
   DELIVER_EXTENSION=${DELIVER_FILE##*.}
   DELIVER_DIR_NAME=${DELIVER_FILE_NAME%.deliver}
   DELIVER_DIR_NAME=${DELIVER_DIR_NAME}_$REL_BUILD_VERSION
   DELIVER_TARGET_ROOT="$EZ_DELIVER_ROOT"
   if [ "$DELIVER_TARGET_SYS_DIR" != "" ] ; then
      DELIVER_TARGET_ROOT="$DELIVER_TARGET_ROOT/$DELIVER_TARGET_SYS_DIR"
   fi
   if [ "$DELIVER_TARGET_SYS_PREFIX" != "" ] ; then
      DELIVER_TARGET_ROOT="$DELIVER_TARGET_ROOT/$DELIVER_TARGET_SYS_PREFIX"
      DELIVER_TARGET_ROOT="$DELIVER_TARGET_ROOT$REL_BUILD_VERSION"
   fi
   REL_ELEMENT_NAME="$DELIVER_DIR_NAME"
   REL_ELEMENT_FULL_NAME="$DELIVER_TARGET_ROOT/$REL_ELEMENT_NAME"

   QA_REL_ELEMENT_FULL_NAME="$QA_REL_ROOT/$REL_ELEMENT_NAME"
   if [ ! -d "$REL_ELEMENT_FULL_NAME" ] ; then
      echo
      echo "ERROR: Failed to find directory $REL_ELEMENT_FULL_NAME to turnover."
      echo "ERROR: Press enter to continue..."
      read CONTINUE_ANSWER
   else
      if [ "$CONFIRM" = TRUE ] ; then
         echo
         if [ "$EZ_VERBOSITY" = "verbose" ] ; then
            echo "Copy $REL_ELEMENT_FULL_NAME"
         else
            echo "Copy $REL_ELEMENT_NAME"
         fi
         echo "To   $QA_REL_ROOT  <y,n>? \c"
         read ANSWER
      else
         ANSWER="y"
      fi
      if [ "$ANSWER" = y -o "$ANSWER" = Y ] ; then
         if [ -d "$QA_REL_ELEMENT_FULL_NAME" ] ; then
            echo "$QA_REL_ELEMENT_FULL_NAME already exists, \c"
            echo "replace it <y,n>? \c"
            read ANSWER
            if [ "$ANSWER" = y -o "$ANSWER" = Y ] ; then
               rm -rf "$QA_REL_ELEMENT_FULL_NAME"
            fi
         fi
         if [ "$EZ_DRYRUN" = TRUE ] ; then
            echo "Would: mkdir -p $QA_REL_ROOT"
            echo "Would: cp -R $REL_ELEMENT_FULL_NAME $QA_REL_ELEMENT_FULL_NAME"
         else
            mkdir -p "$QA_REL_ROOT"
            cp -R "$REL_ELEMENT_FULL_NAME" "$QA_REL_ELEMENT_FULL_NAME"
         fi
#      if [ "$UPDATE_QA_TURNOVER_VERSION" = TRUE ] ; then
#         for VERSION_FILE in `find "$QA_DIR_FULL_NAME" \
#                                   -name SystemDBComponent.4db`
#         do
#            sed -e "s/QA_TURNOVER_VERSION/$QA_RELEASE_NUMBER/" \
#               $VERSION_FILE >   $VERSION_FILE.fix
#            mv $VERSION_FILE.fix $VERSION_FILE 
#            VER_FILE_NAME="${VERSION_FILE##*/}"
#            echo -n "Successfully set the QA Turnover Version "
#            echo    "to $QA_RELEASE_NUMBER in $VER_FILE_NAME."
#         done
#      fi
      echo "Successfully turned over $REL_ELEMENT_NAME."
      fi
   fi
done

if [ -d "$QA_REL_ROOT" ] ; then
   cat <<!

Remember to send an e-mail to ReleaseTurnover with the following content:

SUBJECT:
Release $QA_REL_NAME_AND_VERSION has been turned over to QA.

The release content can be found here:

$QA_REL_ROOT


!
fi

# Please install this release into Dev_NP_A when it is convenient.
# Thanks!!

exit 0
