#===========================================================================
#  Filename: src_create.ksh
#    Author: Craig Schneider
# Orig Date: 08/04/04
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Create new files and/or directories in source control.
#===========================================================================

. $EZ_SOURCE_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: src create [-m Message] FileName(s)...
                  [-dryrun] [-quiet | -verbose]

     -dryrun: Display the commands to be executed, without executing them.
 FileName(s): One or more file names to create in source control.
          -m: The message to be used as the reason for creation.
      -quiet: Suppress non critical messages about operations being performed.
    -verbose: Display verbose messages about operations being performed.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi
if [ $# -eq 0 ] ; then
   Usage
fi

integer SRC_FILE_LIST_COUNT=0
EXIT_CODE=0
MESSAGE_TEXT="None"

EZ_DRYRUN=FALSE
EZ_VERBOSITY=
EZ_PASS_ALONG_OPTS=

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -c) shift ;; # simply ignore this if called from src put -c
      -dryrun)
          EZ_DRYRUN=TRUE
          EZ_PASS_ALONG_OPTS="$EZ_PASS_ALONG_OPTS $1"
          shift ;;
      -m) MESSAGE_TEXT="$2"
          shift
          shift ;;
      -quiet|-verbose)
          EZ_VERBOSITY="${LOWERCASE_ARG#-}"
          EZ_PASS_ALONG_OPTS="$EZ_PASS_ALONG_OPTS $1"
          shift ;;
      -*) if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
       *) SRC_FILE_NAME_LIST[SRC_FILE_LIST_COUNT]="$1"
          SRC_FILE_LIST_COUNT=SRC_FILE_LIST_COUNT+1
          shift ;;
    esac
done

EZSrcSetCheckoutActivity

#===========================================================================
# Perform the requested source control operations.
#===========================================================================
integer COUNT=0
while [ $COUNT -lt $SRC_FILE_LIST_COUNT ]
do
   SRC_FILE_NAME="${SRC_FILE_NAME_LIST[COUNT]}"
   COUNT=COUNT+1
   if [ ! -f "$SRC_FILE_NAME" -a ! -d "$SRC_FILE_NAME" ] ; then
      echo "ERROR: File $SRC_FILE_NAME was not found, creation aborted."
      continue
   fi

   EZSrcMappedFileNameSetup "$SRC_FILE_NAME"
   if [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
      SRC_PROJECT_DIR=`EZSrcGetProjectDirectory`
      EZInvokeCommand -out_err_file $SRC_OUTPUT \
              $SRC_CHANGE_PROJECT_COMMAND "$SRC_PROJECT_DIR"
      if [ $? -ne 0 ] ; then  # if fail to change to project, must create it.
         EZInvokeCommand -out_err_file $SRC_OUTPUT \
              $SRC_CREATE_PROJECT_COMMAND "$SRC_PROJECT_DIR" -CNewProject
         if [ $? -eq 0 ] ; then
            if [ "$EZ_VERBOSITY" != quiet ] ; then
               echo "Successfully created project directory $SRC_PROJECT_DIR."
            fi
            EZInvokeCommand -out_err_file $SRC_OUTPUT \
                    $SRC_CHANGE_PROJECT_COMMAND "$SRC_PROJECT_DIR"
         else
            cat $SRC_OUTPUT
            echo "ERROR: Failed creating project directory $SRC_PROJECT_DIR."
            EXIT_CODE=1
         fi
      fi

      EZInvokeCommand -out_err_file $SRC_OUTPUT \
              $SRC_CREATE_COMMAND -c"$MESSAGE_TEXT" "$SRC_BASE_FILE_NAME"
   elif [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
      EZInvokeCommand -out_err_file $SRC_OUTPUT \
              $SRC_CREATE_COMMAND "$SRC_BASE_FILE_NAME"

      EZInvokeCommand -out_err_file $SRC_OUTPUT \
              $SRC_COMMIT_COMMAND $SRC_MESSAGE_FLAG"$MESSAGE_TEXT" \
                                 "$SRC_BASE_FILE_NAME"
   else
      echo "ERROR: Unsupported source control system $SOURCE_CONTROL_SYSTEM."
      EXIT_CODE=1
   fi
   if [ $? -eq 0 ] ; then
      if [ "$EZ_VERBOSITY" != quiet ] ; then
         echo "Successfully created $(EZSrcObjectText $SRC_FILE_NAME)."
      fi
   else
      if [ -d "$SRC_FILE_NAME.save" ] ; then
         mv "$SRC_FILE_NAME.save" "$SRC_FILE_NAME"
      fi
      cat $SRC_OUTPUT
      echo "ERROR: Failed creating $(EZSrcObjectText $SRC_FILE_NAME)."
      EXIT_CODE=1
   fi

   EZSrcMappedDirNameRestore
done

EZSrcCleanup
exit $EXIT_CODE
