#===========================================================================
#  Filename: src_destroy.ksh
#    Author: Craig Schneider
# Orig Date: 08/04/04
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Destroy files and/or directories from source control.
#===========================================================================

. $EZ_SOURCE_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: src destroy [-m Message] FileName(s)...
                   [-dryrun] [-force] [-quiet | -verbose]

     -dryrun: Display the commands to be executed, without executing them.
      -force: Forcibly destroy the file, even if not in source control.
 FileName(s): One or more file names to destroy from source control.
          -m: The message to be used as the reason for destroying.
      -quiet: Suppress non critical messages about operations being performed.
    -verbose: Display verbose messages about operations being performed.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi
if [ $# -eq 0 ] ; then
   Usage
fi

integer SRC_FILE_LIST_COUNT=0
EXIT_CODE=0
MESSAGE_TEXT="None"
FORCE_MODE=FALSE
SVN_OPTIONS=

EZ_DRYRUN=FALSE
EZ_VERBOSITY=

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -dryrun)
          EZ_DRYRUN=TRUE
          shift ;;
      -force)
          FORCE_MODE=TRUE
          SVN_OPTIONS="$SVN_OPTIONS --force"
          shift ;;
      -m) MESSAGE_TEXT="$2"
          shift
          shift ;;
      -quiet|-verbose)
          EZ_VERBOSITY="${LOWERCASE_ARG#-}"
          shift ;;
      -*) if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
       *) SRC_FILE_NAME_LIST[SRC_FILE_LIST_COUNT]="$1"
          SRC_FILE_LIST_COUNT=SRC_FILE_LIST_COUNT+1
          shift ;;
    esac
done

EZSrcSetCheckoutActivity

#===========================================================================
# Perform the requested source control operations.
#===========================================================================
integer COUNT=0
while [ $COUNT -lt $SRC_FILE_LIST_COUNT ]
do
   SRC_FILE_NAME="${SRC_FILE_NAME_LIST[COUNT]}"
   COUNT=COUNT+1

   # snag this now while the object exists.
   SRC_OBJECT_TEXT="$(EZSrcObjectText $SRC_FILE_NAME)"
   EZSrcMappedFileNameSetup "$SRC_FILE_NAME" LOCAL

   if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
      EZInvokeCommand -egrep_v "^D    " \
           $SRC_DELETE_COMMAND $SVN_OPTIONS "$SRC_MAPPED_FILE_NAME"
      DELETE_EXIT_CODE=$?
      if [ $DELETE_EXIT_CODE -eq 0 ] ; then
         if [ "$FORCE_MODE" = TRUE ] ; then
            EZInvokeCommand -egrep_v "^\$|^Deleting |Committing transaction|Committed revision|Commit failed|is not under version control" \
                $SRC_COMMIT_COMMAND $SRC_MESSAGE_FLAG"$MESSAGE_TEXT" \
                "$SRC_MAPPED_FILE_NAME"
         else
            EZInvokeCommand -egrep_v "^\$|^Deleting |Committing transaction|Committed revision" \
                $SRC_COMMIT_COMMAND $SRC_MESSAGE_FLAG"$MESSAGE_TEXT" \
                "$SRC_MAPPED_FILE_NAME"
            DELETE_EXIT_CODE=$?
         fi
      fi
      if [ $DELETE_EXIT_CODE -eq 0 ] ; then
         if [ "$EZ_VERBOSITY" != quiet ] ; then
            echo "Successfully destroyed $SRC_OBJECT_TEXT."
         fi
      else
         echo "ERROR: Failed destroying $SRC_OBJECT_TEXT."
         EXIT_CODE=1
      fi
   elif [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
      EZInvokeCommand -out_err_file $SRC_OUTPUT \
              $SRC_DESTROY_COMMAND "$SRC_MAPPED_FILE_NAME"
      if [ $? -eq 0 ] ; then
         if [ "$EZ_VERBOSITY" != quiet ] ; then
            echo "Successfully destroyed $SRC_OBJECT_TEXT."
            if [ -d "$SRC_FILE_NAME" ] ; then
               rmdir "$SRC_FILE_NAME"
            fi
         fi
      else
         cat $SRC_OUTPUT
         echo "ERROR: Failed destroying $SRC_OBJECT_TEXT."
         if [ -d "$SRC_FILE_NAME" ] ; then
            rmdir "$SRC_FILE_NAME"
         else 
            rm -f "$SRC_FILE_NAME"
         fi
         EXIT_CODE=1
      fi
   else
      echo "ERROR: Unsupported source control system $SOURCE_CONTROL_SYSTEM."
      EXIT_CODE=1
   fi
   EZSrcMappedDirNameRestore
done

EZSrcCleanup
exit $EXIT_CODE
