#===========================================================================
#  Filename: src_grep.ksh
#    Author: Craig Schneider
# Orig Date: 01/26/06
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Perform source code greps across multiple files.
#===========================================================================

# . $EZ_SOURCE_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi

   cat <<!

Usage: src grep [-e Extension(s)] [-E | -F] [-f FileListFile(s)]
                [-family FamilyName(s)] [-i] [-I] [-l] [-not NotMatchText] [-r] 
                [-verbose] MatchText [FilePattern(s)]

   -no_skips: Do not auto skip files by extension (.doc, .exe, .gif, ...).
          -e: Comma separated list of file endings that must be matched.
          -E: Behave like egrep, match using full regular expressions.
          -F: Behave like fgrep, match using fixed strings.
          -f: Comma separated list of file names where each file contains a
              list of filenames to grep through (one file per line).
     -family: Comma separated list of family names whose source files to grep.
 FilePattern: File name list or match patterns to grep through (ex \*.cs).
          -i: Ignore case while performing the MatchText comparisions.
          -I: Ignore case while performing the NotMatchText comparisions.
          -l: List matching file names only, suppress grep output.
   MatchText: Match string to be searched for in the files.
        -not: Only display matches that do not match the NotMatchText. 
          -r: Display results only, suppress the names of the matching files.
    -verbose: Display verbose messages about operations being performed.

     example: src grep -i "Find This" -e .cs,.js,mk.ksh
     example: src grep "Find This" *.cs

        NOTE: All FilePattern arguments must be last on the command line.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi

EXIT_CODE=0

EZ_VERBOSITY=

AUTO_SKIP=TRUE
DISPLAY_ADDL_INFO=TRUE
DISPLAY_FILE_NAMES=TRUE
DISPLAY_GREP_RESULTS=TRUE
DISPLAY_SKIPS=FALSE
EXTENSION_LIST=
GREP_COMMAND=grep
GREP_TO_TXT_FILE="$TMPDIR/src_grep_txt.$$"
GREP_RESULTS="$TMPDIR/src_grep_res.$$"
FILE_LIST_FILE="$TMPDIR/src_grep_list.$$"
FILE_LIST_EMPTY=TRUE
IGNORE_CASE=
IGNORE_CASE_NOT=
MATCH_TEXT=
NOT_MATCH_TEXT=
FIND_BINARY_FILES=FALSE

rm -f "$FILE_LIST_FILE"

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -e|-E) if [ "$1" = "-e" ] ; then
                EXTENSION_LIST="$EXTENSION_LIST $(echo $2 | tr ',' ' ')"
                shift
                shift
             else
                GREP_COMMAND=egrep # can not use 'grep -E' under solaris
                shift
             fi ;;
      -f|-F) if [ "$1" = "-f" ] ; then
                EXPLICIT_FILE="$2"
                if [ "${EXPLICIT_FILE#*,}" = "$EXPLICIT_FILE" ] ; then
                   # This way supports a file name with a space
                   cat "$EXPLICIT_FILE" >> "$FILE_LIST_FILE"
                else
                   for SINGLE_FILE in `echo $EXPLICIT_FILE | tr ',' ' '`
                   do
                      cat "$SINGLE_FILE" >> "$FILE_LIST_FILE"
                   done
                fi
                FILE_LIST_EMPTY=FALSE
                shift
                shift
             else
                GREP_COMMAND=fgrep # can not use 'grep -F' under solaris
                shift
             fi ;;
    -family) typeset -u FAMILY_NAME
             for FAMILY_NAME in `echo $2 | tr ',' ' '`
             do
                eval FAMILY_SRC_LIST="\$EZ_${FAMILY_NAME}_ROOT/Env/src_list.dat"
                if [ -f "$FAMILY_SRC_LIST" ] ; then
                   cat "$FAMILY_SRC_LIST" >> "$FILE_LIST_FILE"
                   FILE_LIST_EMPTY=FALSE
                else
                   echo "ERROR: Failed to find src file list $FAMILY_SRC_LIST."
                   exit 1
                fi
             done
             shift
             shift ;;
      -i|-I) if [ "$1" = "-i" ] ; then
                IGNORE_CASE="-i"
                shift
             else
                IGNORE_CASE_NOT="-i"
                shift
             fi ;;
         -l) DISPLAY_ADDL_INFO=FALSE
             DISPLAY_GREP_RESULTS=FALSE
             DISPLAY_FILE_NAMES=NAMES_ONLY
             shift ;;
       -not) NOT_MATCH_TEXT="$2"
             shift
             shift ;;
  -no_skips) AUTO_SKIP=FALSE
             shift ;;
         -r) DISPLAY_ADDL_INFO=FALSE
             DISPLAY_FILE_NAMES=FALSE
             shift ;;
   -verbose) EZ_VERBOSITY=verbose
             shift ;;
          *) if [ "$1" != "-?" ] ; then
                if [ "$MATCH_TEXT" = "" ] ; then
                   MATCH_TEXT="$1"
                else
                   dlook.sh -FindOptions "-a ! -path '*.svn*' -a ! -path '*.git*'" \
                             "$@" >> "$FILE_LIST_FILE"
                   FILE_LIST_EMPTY=FALSE
                   break
                fi
                shift
             else
                Usage
             fi ;;
    esac
done

if [ "$MATCH_TEXT" = "" ] ; then
   Usage "Must specify the MatchText to grep for."
fi

if [ "$FILE_LIST_EMPTY" = TRUE ] ; then
   typeset -R7 FILE_LIST_LINES
   if [ "$DISPLAY_ADDL_INFO" = TRUE ] ; then
      echo
      echo "Using the following source file lists: "
   fi
   for ROOT_DIR in $EZ_FAM_ROOTS
   do
      FAMILY_FILE_LIST="$ROOT_DIR/Env/src_list.dat"
      if [ -f "$FAMILY_FILE_LIST" ] ; then
         cat "$FAMILY_FILE_LIST" >> "$FILE_LIST_FILE"
         if [ "$DISPLAY_ADDL_INFO" = TRUE ] ; then
            FILE_LIST_LINES=`wc -l "$FAMILY_FILE_LIST" | awk '{print $1}'`
            echo "$FILE_LIST_LINES files in $FAMILY_FILE_LIST"
         fi
      fi
   done
   if [ "$DISPLAY_ADDL_INFO" = TRUE ] ; then
      FILE_LIST_LINES=`wc -l "$FILE_LIST_FILE" | awk '{print $1}'`
      echo "-----------------------------------------------"
      echo "$FILE_LIST_LINES files in ALL source file lists"
      echo
   fi
fi

#===========================================================================
# Perform the requested source control operations.
#===========================================================================
SAVE_PWD=$PWD
MATCHING_LINES=0
MATCHING_FILES=0
TOTAL_FILES=0

if [ "$EZ_KSH_BASE" = CYGWIN ] ; then
   dos2unix.sh "$FILE_LIST_FILE"
fi
# Open 6
exec 6<< !
`cat "$FILE_LIST_FILE"`
!

while read -u6 FILE_NAME
do
   eval GREP_FILE_NAME=\"$FILE_NAME\"
   if [ ! -f "$GREP_FILE_NAME" ] ; then
      if [ "$DISPLAY_FILE_NAMES" = FALSE -a ! -d "$GREP_FILE_NAME" ] ; then
         echo
         echo "ERROR: Failed to find file $GREP_FILE_NAME."
      fi
      continue
   fi
   EXTENSION="${GREP_FILE_NAME##*.}"
   if [ "$EXTENSION_LIST" != "" ] ; then
      VALID_EXTENSION=FALSE
      for ONE_EXTENSION in $EXTENSION_LIST
      do
         if [ "${GREP_FILE_NAME%$ONE_EXTENSION}" != "$GREP_FILE_NAME" ] ; then
            VALID_EXTENSION=TRUE
         fi
      done
      if [ "$VALID_EXTENSION" = "FALSE" ] ; then
         if [ "$DISPLAY_SKIPS" = TRUE ] ; then
            echo "Skipping: $GREP_FILE_NAME filtered by extension."
         fi
         continue
      fi
   fi
   if [ "$AUTO_SKIP" = TRUE ] ; then
      if [ "$EXTENSION" = png -o "$EXTENSION" = gif -o \
           "$EXTENSION" = bmp -o "$EXTENSION" = ico -o \
           "$EXTENSION" = jpg -o "$EXTENSION" = mdb -o \
           "$EXTENSION" = suo -o "$EXTENSION" = asp -o \
           "$EXTENSION" = rep -o "$EXTENSION" = unv -o \
           "$EXTENSION" = dll -o "$EXTENSION" = exe -o \
           "$EXTENSION" = doc -o "$EXTENSION" = xls -o \
           "$EXTENSION" = bcp -o "$EXTENSION" = psp -o \
           "$EXTENSION" = tar -o "$EXTENSION" = jar -o \
           "$EXTENSION" = zip -o "$EXTENSION" = gtz -o \
           "$EXTENSION" = 7z  -o "$EXTENSION" = Z   -o \
           "$EXTENSION" = ldf ] ; then
         if [ "$DISPLAY_SKIPS" = TRUE ] ; then
            echo "Skipping: Unsupported extension ($EXTENSION) $FILE_NAME"
         fi
         continue
      fi
   fi

   # perform any file conversions for better grep results
   if [ "$EXTENSION" = dll -o "$EXTENSION" = mdf ] ; then
      strings "$GREP_FILE_NAME" > "$GREP_TO_TXT_FILE"
      GREP_FILE_NAME="$GREP_TO_TXT_FILE"
   elif [ "$EXTENSION" = lib -o "$EXTENSION" = a -o \
          "$EXTENSION" = so  -o "$EXTENSION" = o ] ; then
      nm "$GREP_FILE_NAME" > "$GREP_TO_TXT_FILE"
      GREP_FILE_NAME="$GREP_TO_TXT_FILE"
   fi

   let TOTAL_FILES=TOTAL_FILES+1
   if [ "$EZ_VERBOSITY" = verbose ] ; then
     echo "Performing: $GREP_COMMAND $IGNORE_CASE '$MATCH_TEXT' $GREP_FILE_NAME"
   fi
   $GREP_COMMAND $IGNORE_CASE "$MATCH_TEXT" "$GREP_FILE_NAME" \
                 > "$GREP_RESULTS" 2>&1
   MATCH_STATUS=$?
   if [ "$FIND_BINARY_FILES" = TRUE ] ; then
      grep "input lines truncated" "$GREP_RESULTS"
      if [ $? -eq 0 ] ; then
         echo "ERROR: Lines truncated for file $GREP_FILE_NAME."
      fi
   fi
   if [ $MATCH_STATUS -eq 0 -a "$NOT_MATCH_TEXT" != "" ] ; then
      $GREP_COMMAND -v $IGNORE_CASE_NOT "$NOT_MATCH_TEXT" "$GREP_RESULTS" > \
                   "$GREP_RESULTS".fix
      MATCH_STATUS=$?
      mv "$GREP_RESULTS".fix "$GREP_RESULTS"
   fi
   if [ $MATCH_STATUS -eq 0 ] ; then
      let MATCHING_FILES=MATCHING_FILES+1
      MATCHING_LINE_COUNT=`wc -l "$GREP_RESULTS" | awk '{print $1}'`
      let MATCHING_LINES=MATCHING_LINES+MATCHING_LINE_COUNT
      if [ "$DISPLAY_FILE_NAMES" = TRUE -a \
           "$DISPLAY_GREP_RESULTS" = TRUE ] ; then
         echo
      fi
      if [ "$DISPLAY_FILE_NAMES" = TRUE ] ; then
         echo "FOUND: $FILE_NAME"
      elif [ "$DISPLAY_FILE_NAMES" = NAMES_ONLY ] ; then
         echo "$FILE_NAME"
      fi
      if [ "$DISPLAY_GREP_RESULTS" = TRUE ] ; then
         cat "$GREP_RESULTS"
      fi
   fi
done

#   Close 6
exec 6<&-

if [ "$DISPLAY_ADDL_INFO" = TRUE ] ; then
   cat <<!

===============================================================================
=== $MATCHING_LINES lines matched in $MATCHING_FILES of the $TOTAL_FILES files for: $MATCH_TEXT
===============================================================================
!
fi

rm -f "$FILE_LIST_FILE" "$GREP_RESULTS" "$GREP_TO_TXT_FILE"
exit $EXIT_CODE
