#===========================================================================
#  Filename: src_modify.ksh
#    Author: Craig Schneider
# Orig Date: 01/23/06
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Perform source code modifications across multiple files.
#===========================================================================

. $EZ_SOURCE_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo "$*"
   fi

   cat <<!

Usage: src modify [-brace] [-diffs] [-dos2unix] [-end_project]
                  [-f FileListFile] [FileNames ...] [-force]
                  [-delete MatchText] [-no_put]
                  [-replace MatchText ReplaceText] [-unix2dos] [-yes]
                  [-quiet | -verbose]

      -brace: Perform the sed using { instead of / to delimit patterns.
      -diffs: Display differences of files before the user is prompted.
   -dos2unix: The dos2unix versions of files are diffed and stabalized.
-end_project: Special CTRL-M processing of the </Project> tag in .csproj files.
          -f: File containing list of filenames perform modifications on.
      -force: All user questions will be implicitly answered with a yes.
     -delete: Delete all lines containing the MatchText.
     -no_put: Updated files are not put back into source control.
    -replace: Replace all occurrances of MatchText with ReplaceText.
      -quiet: Suppress non critical messages about operations being performed.
   -unix2dos: The unix2dos versions of files are diffed and stabalized.
    -verbose: Display verbose messages about operations being performed.
        -yes: All user questions will be implicitly answered with a yes.

     example: src modify -delete "A B C String" \`glook -l "A B C" *.cs\`
     example: src modify -replace "A B C" "abc" \`glook -l "A B C" *.cs\`
     example: src modify -replace "A B C" "abc" \`src grep -l "A B C" -e cs\`
     example: src modify -replace 123 456 -replace abc def \
                         -delete DelMe -delete RmMe \`dlook *.cs\`

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi

EZ_DRYRUN=FALSE
EZ_VERBOSITY=

BRACE_OPT=
END_PROJECT=FALSE
FILE_LIST_FILE="$TMPDIR/src_mod_list.$$"
FILE_LIST_EMPTY=TRUE
integer DELETE_COUNT=0
integer REPLACE_COUNT=0
SRC_STABALIZE_OPTS=

rm -f "$FILE_LIST_FILE"

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
        -brace) BRACE_OPT="-brace"
                shift ;;
       -delete) DELETE_MATCH_TEXT[DELETE_COUNT]="$2"
                let DELETE_COUNT=DELETE_COUNT+1
                shift
                shift ;;
        -diffs) SRC_STABALIZE_OPTS="$SRC_STABALIZE_OPTS -diffs"
                shift ;;
     -dos2unix) SRC_STABALIZE_OPTS="$SRC_STABALIZE_OPTS -dos2unix"
                shift ;;
  -end_project) END_PROJECT=TRUE
                shift ;;
            -f) cat "$2" >> "$FILE_LIST_FILE"
                FILE_LIST_EMPTY=FALSE
                shift
                shift ;;
   -force|-yes) SRC_STABALIZE_OPTS="$SRC_STABALIZE_OPTS -force"
                shift ;;
       -no_put) SRC_STABALIZE_OPTS="$SRC_STABALIZE_OPTS -no_put"
                shift ;;
      -replace) REPLACE_MATCH_TEXT[REPLACE_COUNT]="$2"
                REPLACE_TEXT[REPLACE_COUNT]="$3"
                let REPLACE_COUNT=REPLACE_COUNT+1
                shift
                shift
                shift ;;
     -unix2dos) SRC_STABALIZE_OPTS="$SRC_STABALIZE_OPTS -unix2dos"
                shift ;;
      -quiet|-verbose)
                EZ_VERBOSITY="${LOWERCASE_ARG#-}"
                SRC_STABALIZE_OPTS="$SRC_STABALIZE_OPTS $1"
                shift ;;
      *)        if [ "$1" != "-?" ] ; then
                   echo "$1" >> "$FILE_LIST_FILE"
                      FILE_LIST_EMPTY=FALSE
                   shift
                else
                   Usage
                fi ;;
    esac
done

if [ "$FILE_LIST_EMPTY" = TRUE ] ; then
   Usage "ERROR: src modify must be run with -f or a list of file names."
fi

if [ "$REPLACE_COUNT" = "0" -a "$DELETE_COUNT" = "0" ] ; then
   Usage "ERROR: A transformation function must be specified:\n       Use either -replace or -delete"
fi

#===========================================================================
# Perform the requested source control operations.
#===========================================================================
SAVE_PWD=$PWD

# Open 6
exec 6<< !
`cat "$FILE_LIST_FILE"`
!

while read -u6 FILE_NAME
do
   cp "$FILE_NAME" "$FILE_NAME.fix"
   chmod 777 "$FILE_NAME.fix"
   if [ "$DELETE_COUNT" != "0" ] ; then
      DELETE_INDEX=0
      while [ $DELETE_INDEX -lt $DELETE_COUNT ]
      do
         delete_lines.ksh -m "${DELETE_MATCH_TEXT[DELETE_INDEX]}" \
                          -input "$FILE_NAME.fix" -output "$FILE_NAME.fix1"
         mv "$FILE_NAME.fix1" "$FILE_NAME.fix"
         let DELETE_INDEX=DELETE_INDEX+1
      done
   fi
   if [ "$REPLACE_COUNT" != "0" ] ; then
      REPLACE_INDEX=0
      while [ $REPLACE_INDEX -lt $REPLACE_COUNT ]
      do
         replace_text.ksh $BRACE_OPT \
                      -m "${REPLACE_MATCH_TEXT[REPLACE_INDEX]}" \
                      -r "${REPLACE_TEXT[REPLACE_INDEX]}" \
                      -input "$FILE_NAME.fix" -output "$FILE_NAME.fix1"
         mv "$FILE_NAME.fix1" "$FILE_NAME.fix"
         let REPLACE_INDEX=REPLACE_INDEX+1
      done
   fi
   if [ "$END_PROJECT" = TRUE ] ; then
      # remove last line with CTRL-M and add back without CTRL-M
      sed -e '/<\/Project>/d' "$FILE_NAME.fix" > "$FILE_NAME.fix1"
      echo '</Project>\c' >> "$FILE_NAME.fix1"
      mv "$FILE_NAME.fix1" "$FILE_NAME.fix"
   fi
   src.ksh stabalize -r $SRC_STABALIZE_OPTS "$FILE_NAME.fix" "$FILE_NAME"
done

#   Close 6
exec 6<&-

rm -f "$FILE_LIST_FILE"
exit 0
