#===========================================================================
# FileName: <<FileName>>
#   Author: Craig Schneider
#  Created: 2004-08-16
# Synopsis: Stabalize a files contents possibly updating source control.
# <<Copyright>>
#===========================================================================

. $EZ_SHARED_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: src stabalize [-bdiff] [-c CreationCommand] [-certify FileName]
                     [-d DestructionCommand] [-diff] [-diffs] [-dll_diffs] 
                     [-dos2unix | -dos2unixcmp] [-e ExcludeFile] [-filever] 
                     [-file_fixer Fixer] [-force | -no | -no_prompt | -yes]
                     [-groups] [-i] [-m Message] [-no_diffs] [-no_destroy]
                     [-no_put] [-owners] [-perms] [-r] 
                     [-replace MatchText ReplaceText] [-src_options SrcOptions]
                     [-sorted] [-t MessageName Text]
                     [-transform TransformFunction] [-unix2dos | -unix2doscmp]
                     SourceFile TargetFile
                     [-dryrun] [-quiet | -verbose]

      -bdiff: Use bdiff instead of cmp to perform the file comparision.
          -c: Creation command used when stabalization creates a new TargetFile.
                 chmod:  After creating the TargetFile, perform a chmod 777.
                 deduce: Deduce if TargetFile should be source controled.
                 none:   Do nothing after creating the TargetFile (default). 
                 put:    After creating the TargetFile, perform a src put -c.
    -certify: File containing difference certification information.
          -d: Destruction command used when stabalization destroys a TargetFile.
                 deduce:      Deduce if TargetFile should be src destroyed.
                 rm:          Use rm to destroy the TargetFile (default). 
                 src_delete:  Use src delete to destroy the TargetFile.
                 src_destroy: Use src destroy to destroy the TargetFile.
       -diff: Use diff instead of cmp to perform the file comparision.
      -diffs: Display differences of files before the user is prompted.
  -dll_diffs: Display differences of .dll files before the user is prompted.
   -dos2unix: dos2unix versions of files are diffed and stabalized.
-dos2unixcmp: dos2unix versions of files are diffed, originals are stabalized.
     -dryrun: Display the commands to be executed, without executing them.
          -e: Name of the exclusion list file to use.
    -filever: Allow the use of the filever when displaying file differences.
 -file_fixer: Function to invoke to perform advanced file stabalizing.
      -force: All user questions will be implicitly answered with a yes.
     -groups: Ensure that the files group owners are identical.
          -i: Ignore the case of the file content being compared.
          -m: The message to be used as the reason for source control changes.
         -no: All user questions will be implicitly answered with a no.
   -no_diffs: Do not display differences of files before the user is prompted.
 -no_destroy: No destruction of TargetFile if SourceFile does not exist.
  -no_prompt: All user questions will be implicitly answered with a no.
     -no_put: Updated files are not put back into source control.
     -owners: Ensure that the files owners are identical.
      -perms: Ensure that file permissions are identical.
          -r: Remove the SourceFile when done.
    -replace: Quick interface to the replace transform function.
-src_options: Specify options for the 'src create/get/put' commands.
  SourceFile: The file to be used to replace the contents of the TargetFile.
  TargetFile: The file to be updated from the contents of the SourceFile.
     -sorted: Sorted versions of the files are diffed, originals are stabalized.
          -t: Specify the text to be displayed for the following messages:
                 Created:    Output after the target file is created.
                 Certified:  Output if a files differences have been certified.
                 Destroyed:  Output after the target file is destroyed.
                 Difference: Output after displaying file differences.
                 Excluded:   Output if a file is excluded from consideration.
                 Recompiled: Output if file is has been recompiled.
                 Retained:   Output if file is not updated.
                 Stabalized: Output after the target file has been stabalized.
                 Unchanged:  Output if the source and target files match.
  -transform: Specify the name of a transform function to be used.
   -unix2dos: unix2dos versions of files are diffed and stabalized.
-unix2doscmp: unix2dos versions of files are diffed, originals are stabalized.
      -quiet: Suppress non critical messages about operations being performed.
    -verbose: Display verbose messages about operations being performed.
        -yes: All user questions will be implicitly answered with a yes.

!
   exit 5
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = "-?" ] ; then
   Usage
fi
if [ "$#" -eq 0 ] ; then
   Usage
fi

EXIT_CODE=0  # 0) TargetFile already matched.
             # 1) TargetFile was created.
             # 2) TargetFile was updated.
             # 3) TargetFile was destroyed.
             # 4) TargetFile was different or missing and excluded.
             # 5) TargetFile was not updated and/or certified.
             # 6) TargetFile was recompiled.
             # 7) TargetFile was certified.

EZ_DRYRUN=FALSE
EZ_VERBOSITY=

typeset -l CREATION_COMMAND
typeset -l DESTRUCTION_COMMAND
ALLOW_FILEVER=FALSE
ALLOW_DESTROY=TRUE
ALLOW_PUT=TRUE
CERTIFY_FILE_NAME=
CHECK_RECOMPILED=FALSE 
if [ "$EZ_OS_TYPE" = windows ] ; then
   CHECK_RECOMPILED=TRUE
fi
CREATION_COMMAND=none
DESTRUCTION_COMMAND=rm
DIFF_EXE=diff
DIFF_EXE_ARGS=
EXCLUSION_FILE_NAME=
EXCL_OPT=
FILE_FIXER=
MESSAGE_TEXT="Updated by $LOGNAME using src stabalize on $(date '+%m/%d/%y')."
REMOVE_SOURCE_FILE=FALSE
DISPLAY_DIFFS=FALSE
DISPLAY_DLL_DIFFS=FALSE
DOS2UNIX_FILES=FALSE
DOS2UNIXCMP_FILES=FALSE
SORTED_FILES=FALSE
STABALIZE_AUTO_ANSWER=
STABALIZE_GROUPS=FALSE
STABALIZE_OWNERS=FALSE
STABALIZE_PERMS=FALSE
SYNCRONIZATION_MODE=SYNC
SOURCE_FILE_NAME=
SOURCE_FILE_NAME_FIX=
SOURCE_FILE_NAME_ORIGINAL=
TARGET_FILE_NAME=
TRANSFORM_FUNCTION=
UNIX2DOS_FILES=FALSE
UNIX2DOSCMP_FILES=FALSE
USE_BDIFF_EXE=FALSE
USE_DIFF_EXE=FALSE
SRC_OPTS=

FAILED_CREATING_TEXT="Failed creating file \$TARGET_FILE_NAME."
FAILED_DESTROY_TEXT="Failed destroying file \$TARGET_FILE_NAME."
FAILED_STABALIZING_TEXT="Failed stabalizing file \$TARGET_FILE_NAME."
FAILED_CHANGING_GROUP_TEXT="Failed changing group of file \$TARGET_FILE_NAME."
FAILED_CHANGING_OWNER_TEXT="Failed changing owner of file \$TARGET_FILE_NAME."
FAILED_CHANGING_PERMS_TEXT="Failed changing permissions of file \$TARGET_FILE_NAME."
CERTIFIED_BY_NAME_TEXT="Certified: \$CERTIFY_REASON"
CERTIFIED_BY_CONTENT_TEXT="Certified: \$CERTIFY_REASON"
CERTIFIED_TEXT="Successfully detected certified file \$TARGET_FILE_NAME."
CREATED_TEXT="Successfully created file \$TARGET_FILE_NAME."
DESTROYED_TEXT="Successfully destroyed file \$TARGET_FILE_NAME."
CHANGED_GROUP_TEXT="Successfully changed group of file \$TARGET_FILE_NAME."
CHANGED_OWNER_TEXT="Successfully changed owner of file \$TARGET_FILE_NAME."
CHANGED_PERMS_TEXT="Successfully changed permissions of file \$TARGET_FILE_NAME."
DIFFERENCE_TEXT=""
EXCLUDED_TEXT="Successfully EXCLUDED from stabalizing \$TARGET_FILE_NAME."
GROUP_DIFFERENCE_TEXT=""
OWNER_DIFFERENCE_TEXT=""
PERMS_DIFFERENCE_TEXT=""
RECOMPILED_TEXT="Successfully detected recompiled file \$TARGET_FILE_NAME."
RETAINED_TEXT=""
STABALIZED_TEXT="Successfully stabalized file \$TARGET_FILE_NAME."
UNCHANGED_TEXT="Successfully detected unchanged file \$TARGET_FILE_NAME."

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -bdiff)
          USE_DIFF_EXE=TRUE
          DIFF_EXE=BdiffWithStatus
          shift ;;
      -certify)
          CERTIFY_FILE_NAME="$2"
          shift
          shift ;;
      -c) CREATION_COMMAND="$2"
          if [ "$CREATION_COMMAND" != chmod  -a \
               "$CREATION_COMMAND" != deduce -a \
               "$CREATION_COMMAND" != none   -a \
               "$CREATION_COMMAND" != put ] ; then
             Usage "Invalid CreationCommand: $CREATION_COMMAND"
          fi
          shift
          shift ;;
      -d) DESTRUCTION_COMMAND="$2"
          if [ "$DESTRUCTION_COMMAND" != deduce     -a \
               "$DESTRUCTION_COMMAND" != rm         -a \
               "$DESTRUCTION_COMMAND" != src_delete -a \
               "$DESTRUCTION_COMMAND" != src_destroy ] ; then
             Usage "Invalid DestructionCommand: $DESTRUCTION_COMMAND"
          fi
          shift
          shift ;;
      -diff)
          USE_DIFF_EXE=TRUE
          DIFF_EXE=diff
          shift ;;
      -diffs)
          DISPLAY_DIFFS=TRUE
          shift ;;
      -dll_diffs)
          DISPLAY_DLL_DIFFS=TRUE
          shift ;;
      -dos2unix)
          DOS2UNIX_FILES=TRUE
          shift ;;
      -dos2unixcmp)
          DOS2UNIXCMP_FILES=TRUE
          shift ;;
      -dryrun)
          EZ_DRYRUN=TRUE
          shift ;;
      -e) EXCLUSION_FILE_NAME="$2"
          EXCL_OPT=",x"
          shift
          shift ;;
      -ee) EXCLUSION_FILE_NAME="$2" # special for diff_dirs only
          shift
          shift ;;
      -filever)
          ALLOW_FILEVER=TRUE
          shift ;;
      -file_fixer) FILE_FIXER="$2"
          shift
          shift ;;
      -force|-yes)
          STABALIZE_AUTO_ANSWER="y"
          shift ;;
      -groups)
          STABALIZE_GROUPS=TRUE
          shift ;;
      -i)
          USE_DIFF_EXE=TRUE
          DIFF_EXE_ARGS="$DIFF_EXE_ARGS -i"
          shift ;;
      -m) MESSAGE_TEXT="$2"
          shift
          shift ;;
      -no_diffs)
          DISPLAY_DIFFS=FALSE
          shift ;;
      -no_destroy)
          ALLOW_DESTROY="FALSE"
          shift ;;
      -no_prompt|-no_prompts|-no)
          STABALIZE_AUTO_ANSWER="n"
          shift ;;
      -no_put)
          ALLOW_PUT="FALSE"
          shift ;;
      -owners)
          STABALIZE_OWNERS=TRUE
          shift ;;
      -perms)
          STABALIZE_PERMS=TRUE
          shift ;;
      -r) REMOVE_SOURCE_FILE=TRUE
          shift ;;
      -readonly)
          SYNCRONIZATION_MODE=READONLY
          shift ;;
      -replace)
          TRANSFORM_FUNCTION="replace_text -m '$2' -r '$3'"
          shift
          shift
          shift ;;
      -src_options) 
          SRC_OPTS="$2"
          shift
          shift ;;
      -sorted)
          SORTED_FILES=TRUE
          shift ;;
      -sync)
          SYNCRONIZATION_MODE=SYNC
          shift ;;
      -t) typeset -l TEXT_MESSAGE_NAME
          TEXT_MESSAGE_NAME="$2"
          TEXT_MESSAGE="$3"
          if [ "$TEXT_MESSAGE" = "[[CANDIDATE_BUILD_WARNING]]" ] ; then
             TEXT_MESSAGE="Warning: Candidate builds never stabalize files."
          fi
          if [ "$TEXT_MESSAGE_NAME" = certified ] ; then
             CERTIFIED_TEXT="$TEXT_MESSAGE"
          elif [ "$TEXT_MESSAGE_NAME" = created ] ; then
             CREATED_TEXT="$TEXT_MESSAGE"
          elif [ "$TEXT_MESSAGE_NAME" = destroyed ] ; then
             DESTROYED_TEXT="$TEXT_MESSAGE"
          elif [ "$TEXT_MESSAGE_NAME" = difference ] ; then
             DIFFERENCE_TEXT="$TEXT_MESSAGE"
          elif [ "$TEXT_MESSAGE_NAME" = excluded ] ; then
             EXCLUDED_TEXT="$TEXT_MESSAGE"
          elif [ "$TEXT_MESSAGE_NAME" = recompiled ] ; then
             RECOMPILED_TEXT="$TEXT_MESSAGE"
          elif [ "$TEXT_MESSAGE_NAME" = retained ] ; then
             RETAINED_TEXT="$TEXT_MESSAGE"
          elif [ "$TEXT_MESSAGE_NAME" = stabalized ] ; then
             STABALIZED_TEXT="$TEXT_MESSAGE"
          elif [ "$TEXT_MESSAGE_NAME" = unchanged ] ; then
             UNCHANGED_TEXT="$TEXT_MESSAGE"
          fi
          shift
          shift
          shift ;;
      -transform)
          TRANSFORM_FUNCTION="$2"
          shift
          shift ;;
      -unix2dos)
          UNIX2DOS_FILES=TRUE
          shift ;;
      -unix2doscmp)
          UNIX2DOSCMP_FILES=TRUE
          shift ;;
      -quiet|-verbose)
          EZ_VERBOSITY="${LOWERCASE_ARG#-}"
          SRC_OPTS="$SRC_OPTS $1"
          shift ;;
      -*) if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
       *) if [ "$SOURCE_FILE_NAME" = "" ] ; then
             SOURCE_FILE_NAME="$1"
          elif [ "$TARGET_FILE_NAME" = "" ] ; then
             TARGET_FILE_NAME="$1"
          else
             Usage "Too many file names specified."
          fi
          shift ;;
    esac
done

if [ "$SOURCE_FILE_NAME" = "" -o "$TARGET_FILE_NAME" = "" ] ; then
   Usage "Must specify both a SourceFile and TargetFile." 
fi

if [ "$EXCLUSION_FILE_NAME" != "" -a ! -f "$EXCLUSION_FILE_NAME" ] ; then
   touch "$EXCLUSION_FILE_NAME"
fi
if [ "$CERTIFY_FILE_NAME" != "" -a ! -f "$CERTIFY_FILE_NAME" ] ; then
   touch "$CERTIFY_FILE_NAME"
fi

#===========================================================================
# Cleanup any temporary files.
#===========================================================================
function EZSrcStabalizeCleanup
{
   if [ "$DOS2UNIXCMP_FILES" = TRUE -o "$SORTED_FILES"     = TRUE -o \
        "$DOS2UNIX_FILES"    = TRUE -o "$CHECK_RECOMPILED" = TRUE -o \
        "$UNIX2DOSCMP_FILES" = TRUE -o \
        "$UNIX2DOS_FILES"    = TRUE ] ; then
      RM_SOURCE=`EZGetCompareFileName "$SOURCE_FILE_NAME" "REMOVE"`
      RM_TARGET=`EZGetCompareFileName "$TARGET_FILE_NAME" "REMOVE"`
   fi
   if [ "$REMOVE_SOURCE_FILE" = TRUE ] ; then
      rm -f "$SOURCE_FILE_NAME_ORIGINAL"
   fi 
   rm -f "$SOURCE_FILE_NAME_ORIGINAL.fix"
}

trap "EZSrcStabalizeCleanup ; exit 5" 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15

#===========================================================================
# Display the specified message text if allowed and not empty.
#===========================================================================
function EZDisplayMessage
{
   DISPLAY_MESSAGE_TEXT="$*"

   if [ "$EZ_VERBOSITY" != quiet -a "$DISPLAY_MESSAGE_TEXT" != "" ] ; then
      # only eval if has $XXX env vars as eval has issues if message has ()
      if [ "${DISPLAY_MESSAGE_TEXT#*\$}" != "$DISPLAY_MESSAGE_TEXT" ] ; then
         eval echo "$DISPLAY_MESSAGE_TEXT" # has $XXX env vars so eval
      else
         echo "$DISPLAY_MESSAGE_TEXT"      # no $XXX env vars so just echo
      fi
   fi
}

#===========================================================================
# View the contents of the specified file.
#===========================================================================
function EZPerformCat
{
   CAT_OBJECT_NAME="$1"

   echo
   echo "Performing: cat $CAT_OBJECT_NAME"
   echo
   cat "$CAT_OBJECT_NAME"
   echo
}

#===========================================================================
# Deduce the appropriate creation command for the current target file.
#===========================================================================
function EZSrcDeduceCreationCommand
{
   PARENT_MK_SH="../mk.sh"
   if [ ! -f "$PARENT_MK_SH" ] ; then
      PARENT_MK_SH="../mk.ksh"
   fi
   if [ ! -f "$PARENT_MK_SH" ] ; then
      PARENT_MK_SH="../../mk.sh"
   fi
   if [ ! -f "$PARENT_MK_SH" ] ; then
      PARENT_MK_SH="../../mk.ksh"
   fi
   EZRun src.sh test "$PARENT_MK_SH"
   if [ $? -eq 0 ] ; then
      echo "put"
   else
      echo "chmod"
   fi
}

#===========================================================================
# Deduce the appropriate destruction command for the current target file.
#===========================================================================
function EZSrcDeduceDestructionCommand
{
   EZRun src.sh test "$TARGET_FILE_NAME"
   if [ $? -eq 0 ] ; then
      echo "src_destroy"
   else
      echo "rm"
   fi
}

#===========================================================================
# Add the TargetFile to the exclusion list.
#===========================================================================
function EZCertifyFile
{
   FILE_TO_CERTIFY="$1"

   if [ "$CERTIFY_FILE_NAME" = "" ] ; then
      return 1
   fi

   CHECKSUM_INFO="ALL:ALL"
   while true
   do
      echo
      echo -n "Certify based upon difference information <y,n,d>? "
      read CERTIFY_ANSWER
      if [ "$CERTIFY_ANSWER" = y -o "$CERTIFY_ANSWER" = Y ] ; then
         DIFF_OUTPUT="$TMPDIR/ss.diff.$$"
         EZDiffStableFiles JUST_DIFFS > "$DIFF_OUTPUT"
         CHECKSUM=`cksum "$DIFF_OUTPUT" | awk '{print $1}'`
         BYTECOUNT=`cksum "$DIFF_OUTPUT" | awk '{print $2}'`
         CHECKSUM_INFO="$CHECKSUM:$BYTECOUNT"
         break
      fi
      if [ "$CERTIFY_ANSWER" = n -o "$CERTIFY_ANSWER" = N ] ; then
         break
      fi
      if [ "$CERTIFY_ANSWER" = d -o "$CERTIFY_ANSWER" = D ] ; then
         EZDiffStableFiles FALSE
      fi
   done
   
   echo
   echo "Please enter the reason for certification:"
   read CERTIFICATION_REASON
   CERTIFY_BASE_FILE="${FILE_TO_CERTIFY##*/}"
   sed -e "/\/$CERTIFY_BASE_FILE:/d" \
       -e "/^$CERTIFY_BASE_FILE:/d" \
       "$CERTIFY_FILE_NAME" > "$CERTIFY_FILE_NAME.fix"
   echo "$CERTIFY_BASE_FILE:$CHECKSUM_INFO:$CERTIFICATION_REASON" >> \
        "$CERTIFY_FILE_NAME.fix"
   src.sh stabalize -yes -r "$CERTIFY_FILE_NAME.fix" "$CERTIFY_FILE_NAME"
}

#===========================================================================
# Add the TargetFile to the exclusion list.
#===========================================================================
function EZAddExcludedFile
{
   if [ "$EXCLUSION_FILE_NAME" = "" ] ; then
      return 1
   fi
   BASE_TARGET_FILE_NAME="${TARGET_FILE_NAME##*/}"

   cp "$EXCLUSION_FILE_NAME" "$EXCLUSION_FILE_NAME.fix"
   chmod 777 "$EXCLUSION_FILE_NAME.fix"
   echo "$BASE_TARGET_FILE_NAME" >> "$EXCLUSION_FILE_NAME.fix"
   sort "$EXCLUSION_FILE_NAME.fix" > "$EXCLUSION_FILE_NAME.fix1"
   mv "$EXCLUSION_FILE_NAME.fix1" "$EXCLUSION_FILE_NAME.fix"
   dos2unix.sh "$EXCLUSION_FILE_NAME.fix"
   src.sh stabalize -yes -r "$EXCLUSION_FILE_NAME.fix" "$EXCLUSION_FILE_NAME"
   return 0
}

#===========================================================================
# Determine if the file differences have been certified.
#===========================================================================
function EZIsFileCertified
{
   FILE_TO_CERTIFY="$1"

   if [ "$CERTIFY_FILE_NAME" = "" ] ; then
      return 1
   fi
   # if the FileToCertify was C:/aaa/bbb/ccc/ddd/eee.dll
   # first look for ddd/eee.dll then look for eee.dll
   BASE_FILE_NAME="${FILE_TO_CERTIFY##*/}"
   DIRECTORY_PATH="${FILE_TO_CERTIFY%/*}"
   BASE_FILE_AND_DIR="${DIRECTORY_PATH##*/}/$BASE_FILE_NAME"
   CERTIFY_CONTENT=`egrep "^$BASE_FILE_AND_DIR:" "$CERTIFY_FILE_NAME"`
   if [ $? -ne 0 ] ; then
      CERTIFY_CONTENT=`egrep "^$BASE_FILE_NAME:" "$CERTIFY_FILE_NAME"`
      if [ $? -ne 0 ] ; then
         return 1
      fi
   fi
   CERTIFY_TEMP="${CERTIFY_CONTENT#*:}"
   SAVED_CHECKSUM="${CERTIFY_TEMP%%:*}"
   CERTIFY_TEMP="${CERTIFY_TEMP#*:}"
   SAVED_BYTECOUNT="${CERTIFY_TEMP%%:*}"
   CERTIFY_REASON="${CERTIFY_TEMP#*:}"
   if [ "$SAVED_CHECKSUM" == ALL ] ; then
      EZDisplayMessage "$CERTIFIED_BY_NAME_TEXT"
      return 0
   fi
   DIFF_OUTPUT="$TMPDIR/ss.diff.$$"
   EZDiffStableFiles JUST_DIFFS > "$DIFF_OUTPUT"
   NEW_CHECKSUM=`cksum "$DIFF_OUTPUT" | awk '{print $1}'`
   NEW_BYTECOUNT=`cksum "$DIFF_OUTPUT" | awk '{print $2}'`
   if [ "$SAVED_CHECKSUM" = "$NEW_CHECKSUM" -a \
        "$SAVED_BYTECOUNT" = "$NEW_BYTECOUNT" ] ; then
      EZDisplayMessage "$CERTIFIED_BY_CONTENT_TEXT"
      return 0
   fi

   return 1
}

#===========================================================================
# Determine if the file is excluded from stabalizing.
#===========================================================================
function EZIsFileExcluded
{
   if [ "$EXCLUSION_FILE_NAME" = "" ] ; then
      return 1
   fi
   BASE_TARGET_FILE_NAME=`echo "${TARGET_FILE_NAME##*/}" | tr '+' '.'`
   egrep $EZ_EGREP_QUIET "^$BASE_TARGET_FILE_NAME\$" "$EXCLUSION_FILE_NAME"
   if [ $? -eq 0 ] ; then
      return 0
   fi
   return 1
}

#===========================================================================
# Create the TargetFile from the SourceFile.
#===========================================================================
function EZCreateStableFile
{
   EZIsFileCertified "$SOURCE_FILE_NAME"
   if [ $? -eq 0 ] ; then
      EZDisplayMessage "$CERTIFIED_TEXT"
      EXIT_CODE=7
      return
   fi
   EZIsFileExcluded
   if [ $? -eq 0 ] ; then
      EZDisplayMessage "$EXCLUDED_TEXT"
      EXIT_CODE=4
      return
   fi
   if [ "$DISPLAY_DIFFS" = TRUE ] ; then
      EZPerformCat "$SOURCE_FILE_NAME"
   fi
   while true
   do
      EZDisplayMessage "$DIFFERENCE_TEXT"
      if [ "$STABALIZE_AUTO_ANSWER" != "" ] ; then
         STABLE_ANSWER="$STABALIZE_AUTO_ANSWER"
      else
         CERT=
         if [ "$CERTIFY_FILE_NAME" != "" ] ; then
            CERT=",c"
         fi
         if [ "$SYNCRONIZATION_MODE" = SYNC ] ; then
            echo -n "Create file $TARGET_FILE_NAME with the contents "
            echo -n "of $SOURCE_FILE_NAME <y,n,v$EXCL_OPT>? "
         else # READONLY
            echo -n "File missing: $TARGET_FILE_NAME <i$CERT,v$EXCL_OPT>? "
         fi
         read STABLE_ANSWER
      fi
      if [ "$STABLE_ANSWER" = y ] ; then
         if [ "$SYNCRONIZATION_MODE" = SYNC ] ; then
            EZRun cp "$SOURCE_FILE_NAME" "$TARGET_FILE_NAME"
            if [ $? -ne 0 ] ; then
               EZDisplayMessage "$FAILED_CREATING_TEXT"
               EXIT_CODE=5
               break
            fi
            if [ "$CREATION_COMMAND" = deduce ] ; then
               CREATION_COMMAND=`EZSrcDeduceCreationCommand`
            fi
            if [ "$CREATION_COMMAND" = chmod ] ; then
               EZRun chmod 777 "$TARGET_FILE_NAME"
            elif [ "$CREATION_COMMAND" = put ] ; then
               EZRun src.sh create $SRC_OPTS -m "$MESSAGE_TEXT" \
                                                       "$TARGET_FILE_NAME"
            fi
            EZDisplayMessage "$CREATED_TEXT"
            EXIT_CODE=1
            break
         fi
      elif [ "$STABLE_ANSWER" = n -o "$STABLE_ANSWER" = i ] ; then
         EZDisplayMessage "$RETAINED_TEXT"
         EXIT_CODE=5
         break
      elif [ "$STABLE_ANSWER" = c ] ; then
         if [ "$CERTIFY_FILE_NAME" != "" ] ; then
            EZCertifyFile "$SOURCE_FILE_NAME"
            EXIT_CODE=7
            break
         fi
      elif [ "$STABLE_ANSWER" = v -o "$STABLE_ANSWER" =  d ] ; then
         EZPerformCat "$SOURCE_FILE_NAME"
      elif [ "$STABLE_ANSWER" = x ] ; then
         EZAddExcludedFile
         EXIT_CODE=4
         break
      fi
   done
}

#===========================================================================
# Compare stable files.
#===========================================================================
function EZGetCompareFileName
{
   CMP_FILE="$1"
   CREATE_USE_REMOVE="$2"

   CMP_EXTENSION="${CMP_FILE##*.}"
   if [ "$CMP_EXTENSION" = dll -o "$CMP_EXTENSION" = exe ] ; then
      if [ "$CHECK_RECOMPILED" = TRUE ] ; then
         ORIG_CMP_FILE="$CMP_FILE"
         CMP_FILE="$CMP_FILE.2il"
         touch "$CMP_FILE" 2>$EZ_DEV_NULL
         if [ $? -ne 0 ] ; then   # directory is not writeable
            BASE_CMP_FILE="${CMP_FILE##*/}"
            if [ "$ORIG_CMP_FILE" = "$SOURCE_FILE_NAME" ] ; then
               CMP_FILE="$TMPDIR/$BASE_CMP_FILE.source"
            else
               CMP_FILE="$TMPDIR/$BASE_CMP_FILE.target"
            fi
         fi

         if [ "$CREATE_USE_REMOVE" = CREATE ] ; then
            ildasm /TEXT "$ORIG_CMP_FILE" 2>$EZ_DEV_NULL > "$CMP_FILE.a"
            if [ $? -eq 0 ] ; then
               # .ver and MVID info change on every compile
               # why do managed resources have to be in a different order?
               sed -e '/^  .ver /d' -e '/^\/\/ MVID: /d' \
                   -e '/^.mresource public/d' "$CMP_FILE.a" > "$CMP_FILE"
            fi
            rm -f "$CMP_FILE.a"
         fi
         if [ -f "$CMP_FILE" ] ; then
            if [ "$CREATE_USE_REMOVE" = REMOVE ] ; then
               rm -f "$CMP_FILE"
            fi
         else
            CMP_FILE="$ORIG_CMP_FILE"
         fi
      fi
   fi

   if [ "$SORTED_FILES" = TRUE ] ; then
      if [ "$CREATE_USE_REMOVE" = CREATE ] ; then
         sort "$CMP_FILE" > "${CMP_FILE}.sorted"
      fi
      CMP_FILE="$CMP_FILE.sorted"  
      if [ "$CREATE_USE_REMOVE" = REMOVE ] ; then
         rm -f "$CMP_FILE"
      fi
   fi 
   if [ "$DOS2UNIXCMP_FILES" = TRUE ] ; then
      if [ "$CREATE_USE_REMOVE" = CREATE ] ; then
         dos2unix.sh "$CMP_FILE" "${CMP_FILE}.2unix"
         if [ "$SORTED_FILES" = TRUE ] ; then
            rm -f "$1.sorted"
         fi
      fi
      CMP_FILE="$CMP_FILE.2unix"  
      if [ "$CREATE_USE_REMOVE" = REMOVE ] ; then
         rm -f "$CMP_FILE"
      fi
   fi 
   if [ "$UNIX2DOSCMP_FILES" = TRUE ] ; then
      if [ "$CREATE_USE_REMOVE" = CREATE ] ; then
         dos2unix.sh -reverse "$CMP_FILE" "${CMP_FILE}.2dos"
         if [ "$SORTED_FILES" = TRUE ] ; then
            rm -f "$1.sorted"
         fi
      fi
      CMP_FILE="$CMP_FILE.2dos"  
      if [ "$CREATE_USE_REMOVE" = REMOVE ] ; then
         rm -f "$CMP_FILE"
      fi
   fi 

   echo "$CMP_FILE"
}

#===========================================================================
# Deal with the fact that bdiff always exits with a 0 on solaris.
#===========================================================================
function BdiffWithStatus
{
   CMP_SOURCE="$1"
   CMP_TARGET="$2"

   EZRun cmp -s "$CMP_SOURCE" "$CMP_TARGET"
   BDIFF_CMP_EXIT=$?
   if [ $BDIFF_CMP_EXIT -ne 0 ] ; then
      EZRun bdiff "$CMP_SOURCE" "$CMP_TARGET"      
   fi
   return $BDIFF_CMP_EXIT
}

#===========================================================================
# Compare stable files.
#===========================================================================
function EZCompareStableFiles
{
   CMP_SOURCE=`EZGetCompareFileName "$SOURCE_FILE_NAME" "CREATE"`
   CMP_TARGET=`EZGetCompareFileName "$TARGET_FILE_NAME" "CREATE"`

   if [ "$USE_DIFF_EXE" = TRUE ] ; then
      TMP_OUTPUT="$TMPDIR/sstbl.$$.output"
      EZRun -output_file "$TMP_OUTPUT" \
                      $DIFF_EXE $DIFF_EXE_ARGS "$CMP_SOURCE" "$CMP_TARGET"
      DIFF_CMP_EXIT=$?
      if [ $DIFF_CMP_EXIT -ne 0 ] ; then
         egrep $EZ_EGREP_QUIET "is binary" "$TMP_OUTPUT"
         if [ $? -eq 0 ] ; then # even if want diff, use cmp if binary
            EZRun cmp -s "$CMP_SOURCE" "$CMP_TARGET"
            DIFF_CMP_EXIT=$?
         fi
      fi
      rm -f "TMP_OUTPUT"
   else
      EZRun cmp -s "$CMP_SOURCE" "$CMP_TARGET"
      DIFF_CMP_EXIT=$?
   fi
   return $DIFF_CMP_EXIT
}

#===========================================================================
# Compare stable files.
#===========================================================================
function EZDiffStableFiles
{
   PRE_DIFF="$1"

   DIFF_SOURCE=`EZGetCompareFileName "$SOURCE_FILE_NAME" "USE"`
   DIFF_TARGET=`EZGetCompareFileName "$TARGET_FILE_NAME" "USE"`

   if [ "$PRE_DIFF" = TRUE -a "$DISPLAY_DLL_DIFFS" = FALSE ] ; then
      SOURCE_FILE_EXT="${SOURCE_FILE_NAME##*.}"
      if [ "$SOURCE_FILE_EXT" = dll -o "$SOURCE_FILE_EXT" = exe ] ; then
         echo
         echo "Not Performing: $DIFF_EXE $DIFF_EXE_ARGS $DIFF_SOURCE $DIFF_TARGET"
         echo
         echo "Use -dll_diffs to see diffs of .dll or .exe files."
         echo
         return
      fi
   fi

   if [ ! -f "$DIFF_TARGET" ] ; then
      echo
      if [ "$PRE_DIFF" != JUST_DIFFS ] ; then
         echo "Performing: cat $DIFF_SOURCE"
      fi
      echo
      cat "$DIFF_SOURCE"
      echo
   elif [ ! -f "$DIFF_SOURCE" ] ; then
      echo
      if [ "$PRE_DIFF" != JUST_DIFFS ] ; then
         echo "Performing: cat $DIFF_TARGET"
      fi
      echo
      cat "$DIFF_TARGET"
      echo
   else
      echo
      if [ "$PRE_DIFF" != JUST_DIFFS ] ; then
         if [ "$DIFF_EXE" = BdiffWithStatus ] ; then
            echo "Performing: bdiff $DIFF_SOURCE $DIFF_TARGET"
         else
            echo "Performing: $DIFF_EXE $DIFF_EXE_ARGS $DIFF_SOURCE $DIFF_TARGET"
         fi
      fi
      echo
      $DIFF_EXE $DIFF_EXE_ARGS "$DIFF_SOURCE" "$DIFF_TARGET"
      echo
   fi
}

#===========================================================================
# Update the TargetFile from the SourceFile.
#===========================================================================
function EZUpdateStableFile
{
   SOURCE_FILE_EXT="${SOURCE_FILE_NAME##*.}"
   EZCompareStableFiles
   if [ $? -ne 0 ] ; then
      EZIsFileCertified "$SOURCE_FILE_NAME"
      if [ $? -eq 0 ] ; then
         EZDisplayMessage "$CERTIFIED_TEXT"
         EXIT_CODE=7
         return
      fi
      EZIsFileExcluded
      if [ $? -eq 0 ] ; then
         EZDisplayMessage "$EXCLUDED_TEXT"
         EXIT_CODE=4
         return
      fi

      if [ "$DISPLAY_DIFFS" = TRUE ] ; then
         EZDiffStableFiles TRUE
      elif [ "$DISPLAY_DLL_DIFFS" = TRUE ] ; then
         if [ "$SOURCE_FILE_EXT" = dll -o "$SOURCE_FILE_EXT" = exe ] ; then
            EZDiffStableFiles TRUE
         fi
      fi
      while true
      do
         EZDisplayMessage "$DIFFERENCE_TEXT"
         STABLE_ANSWER=""
         if [ "$STABALIZE_AUTO_ANSWER" != "" ] ; then
            STABLE_ANSWER="$STABALIZE_AUTO_ANSWER"
         fi
         if [ "$STABLE_ANSWER" = "" ] ; then
            AUTO_STABALIZE=`which AutoStabalize.sh 2>&1`
            if [ -f "$AUTO_STABALIZE" ] ; then
               DIFF_FILE_NAME="$TMPDIR/auto_stabalize.diffs.$$"
               EZDiffStableFiles JUST_DIFFS > "$DIFF_FILE_NAME"
               "$AUTO_STABALIZE" "$DIFF_FILE_NAME"
               if [ $? -eq 0 ] ; then
                  STABLE_ANSWER=y
                  sleep 2
               fi
               rm -f "$DIFF_FILE_NAME"
            fi
         fi
         if [ "$STABLE_ANSWER" = "" ] ; then
            CERT=
            if [ "$CERTIFY_FILE_NAME" != "" ] ; then
               CERT=",c"
            fi
            if [ "$SYNCRONIZATION_MODE" = SYNC ] ; then
               echo -n "Replace $TARGET_FILE_NAME with the contents "
               FVER=
               if [ "$EZ_OS_TYPE" = windows -a "$ALLOW_FILEVER" = TRUE ] ; then
                  FVER=",f"
               fi
               echo -n "of $SOURCE_FILE_NAME <y,n$CERT,d,v$FVER$EXCL_OPT>? "
            else
               echo -n "Files differ: $SOURCE_FILE_NAME  "
               echo -n "$TARGET_FILE_NAME <i$CERT,d,v$EXCL_OPT>? "
            fi
            read STABLE_ANSWER
         fi
         if [ "$STABLE_ANSWER" = y ] ; then
            if [ "$SYNCRONIZATION_MODE" = SYNC ] ; then
               CHECKED_OUT_FILE=FALSE
               EZRun src.sh test "$TARGET_FILE_NAME"
               if [ $? -eq 0 ] ; then
                  CHECKED_OUT_FILE=TRUE
                  # No need to check out files under git.
                  if [ "$EZ_SOURCE_CONTROL" != git ] ; then
                     EZRun src.sh get $SRC_OPTS   "$TARGET_FILE_NAME"
                  fi
               else
                  EZRun chmod 777           "$TARGET_FILE_NAME"
                  if [ $? -ne 0 ] ; then
                     EZDisplayMessage "$FAILED_STABALIZING_TEXT"
                     EXIT_CODE=5
                     break
                  fi
               fi
               EZRun    cp "$SOURCE_FILE_NAME" "$TARGET_FILE_NAME"
               if [ "$CHECKED_OUT_FILE" = TRUE -a "$ALLOW_PUT" = TRUE ] ; then
                  EZRun src.sh put $SRC_OPTS -m "$MESSAGE_TEXT" "$TARGET_FILE_NAME"
               fi
               EZDisplayMessage "$STABALIZED_TEXT"
               EXIT_CODE=2
               break
            fi
         elif [ "$STABLE_ANSWER" = n -o "$STABLE_ANSWER" = i ] ; then
            EZDisplayMessage "$RETAINED_TEXT"
            EXIT_CODE=5
            break
         elif [ "$STABLE_ANSWER" = f ] ; then
            echo
            echo
            filever "$SOURCE_FILE_NAME"
            echo
            filever "$TARGET_FILE_NAME"
            echo
         elif [ "$STABLE_ANSWER" = c ] ; then
            if [ "$CERTIFY_FILE_NAME" != "" ] ; then
               EZCertifyFile "$SOURCE_FILE_NAME"
               EXIT_CODE=7
               break
            fi
         elif [ "$STABLE_ANSWER" = d ] ; then
            EZDiffStableFiles FALSE
         elif [ "$STABLE_ANSWER" = v ] ; then
            EZPerformCat "$SOURCE_FILE_NAME"
         elif [ "$STABLE_ANSWER" = x ] ; then
            EZAddExcludedFile
            EXIT_CODE=4
            break
         fi
      done
   else
      if [ "$CHECK_RECOMPILED" = TRUE ] ; then
         if [ "$SOURCE_FILE_EXT" = dll -o "$SOURCE_FILE_EXT" = exe ] ; then
            cmp -s "$SOURCE_FILE_NAME" "$TARGET_FILE_NAME"
            if [ $? -eq 0 ] ; then
               EZDisplayMessage "$UNCHANGED_TEXT"
            else
               EZDisplayMessage "$RECOMPILED_TEXT"
               EXIT_CODE=6
            fi
         else
            EZDisplayMessage "$UNCHANGED_TEXT"
         fi
      else
         EZDisplayMessage "$UNCHANGED_TEXT"
      fi
   fi
}

#===========================================================================
# Destroy the TargetFile since no SourceFile.
#===========================================================================
function EZDestroyStableFile
{
   EZIsFileCertified "$TARGET_FILE_NAME"
   if [ $? -eq 0 ] ; then
      EZDisplayMessage "$CERTIFIED_TEXT"
      EXIT_CODE=7
      return
   fi
   EZIsFileExcluded
   if [ $? -eq 0 ] ; then
      EZDisplayMessage "$EXCLUDED_TEXT"
      EXIT_CODE=4
      return
   fi
   if [ "$DISPLAY_DIFFS" = TRUE ] ; then
      EZPerformCat "$TARGET_FILE_NAME"
   fi
   while true
   do
      EZDisplayMessage "$DIFFERENCE_TEXT"
      if [ "$STABALIZE_AUTO_ANSWER" != "" ] ; then
         STABLE_ANSWER="$STABALIZE_AUTO_ANSWER"
      else
         CERT=
         if [ "$CERTIFY_FILE_NAME" != "" ] ; then
            CERT=",c"
         fi
         if [ "$SYNCRONIZATION_MODE" = SYNC ] ; then
            echo -n "Destroy $TARGET_FILE_NAME <y,n,v$EXCL_OPT>? "
         else # READONLY
            echo -n "File missing: $SOURCE_FILE_NAME <i$CERT,v$EXCL_OPT>? "
         fi
         read STABLE_ANSWER
      fi
      if [ "$STABLE_ANSWER" = y ] ; then
         if [ "$SYNCRONIZATION_MODE" = SYNC ] ; then
            if [ "$DESTRUCTION_COMMAND" = deduce ] ; then
               DESTRUCTION_COMMAND=`EZSrcDeduceDestructionCommand`
            fi
            if [ "$DESTRUCTION_COMMAND" = rm ] ; then
               EZRun rm -f "$TARGET_FILE_NAME"
            elif [ "$DESTRUCTION_COMMAND" = src_delete ] ; then
               EZRun src.sh delete  $SRC_OPTS -quiet -m "$MESSAGE_TEXT" \
                                                       "$TARGET_FILE_NAME"
            elif [ "$DESTRUCTION_COMMAND" = src_destroy ] ; then
               EZRun src.sh destroy $SRC_OPTS -quiet -m "$MESSAGE_TEXT" \
                                                       "$TARGET_FILE_NAME"
            fi
            if [ $? -eq 0 ] ; then
               EZDisplayMessage "$DESTROYED_TEXT"
               EXIT_CODE=3
            else
               EZDisplayMessage "$FAILED_DESTROY_TEXT"
               EXIT_CODE=5
            fi
            break
         fi
      elif [ "$STABLE_ANSWER" = n -o "$STABLE_ANSWER" = i ] ; then
         EZDisplayMessage "$RETAINED_TEXT"
         EXIT_CODE=5
         break
      elif [ "$STABLE_ANSWER" = c ] ; then
         EZCertifyFile "$SOURCE_FILE_NAME"
         EXIT_CODE=7
         break
      elif [ "$STABLE_ANSWER" = v ] ; then
         EZPerformCat "$TARGET_FILE_NAME"
      elif [ "$STABLE_ANSWER" = x ] ; then
         EZAddExcludedFile
         EXIT_CODE=4
         break
      fi
   done
}

#===========================================================================
# Ensure the source and target file groups match.
#===========================================================================
function EZStabalizeGroups
{
   SOURCE_GROUP=`ls -l "$SOURCE_FILE_NAME" | awk '{print $4}'`
   TARGET_GROUP=`ls -l "$TARGET_FILE_NAME" | awk '{print $4}'`

   if [ "$SOURCE_GROUP" != "$TARGET_GROUP" ] ; then
      while true
      do
         EZDisplayMessage "$GROUP_DIFFERENCE_TEXT"
         if [ "$STABALIZE_AUTO_ANSWER" != "" ] ; then
            GROUP_STABLE_ANSWER="$STABALIZE_AUTO_ANSWER"
            if [ "$GROUP_STABLE_ANSWER" = n ] ; then
               EZDisplayMessage "Not changing the group of $TARGET_FILE_NAME" \
                                "from $TARGET_GROUP to $SOURCE_GROUP."
            fi
         elif [ "$STABLE_ANSWER" = y ] ; then
            GROUP_STABLE_ANSWER="$STABLE_ANSWER"
         else
            echo -n "Change the group of $TARGET_FILE_NAME from $TARGET_GROUP "
            echo -n "to $SOURCE_GROUP <y,n$EXCL_OPT>? "
            read GROUP_STABLE_ANSWER
         fi
         if [ "$GROUP_STABLE_ANSWER" = y ] ; then
            chgrp $SOURCE_GROUP $TARGET_FILE_NAME
            if [ $? -eq 0 ] ; then
               EZDisplayMessage "$CHANGED_GROUP_TEXT"
            else
               EZDisplayMessage "$FAILED_CHANGING_GROUP_TEXT"
               EXIT_CODE=5
            fi
            break
         elif [ "$GROUP_STABLE_ANSWER" = n ] ; then
            EZDisplayMessage "$RETAINED_GROUP_TEXT"
            EXIT_CODE=5
            break
         elif [ "$GROUP_STABLE_ANSWER" = x ] ; then
            EZAddExcludedFile
            EXIT_CODE=4
            break
         fi
      done
   fi
}

#===========================================================================
# Ensure the source and target file owners match.
#===========================================================================
function EZStabalizeOwners
{
   SOURCE_OWNER=`ls -l "$SOURCE_FILE_NAME" | awk '{print $3}'`
   TARGET_OWNER=`ls -l "$TARGET_FILE_NAME" | awk '{print $3}'`

   if [ "$SOURCE_OWNER" != "$TARGET_OWNER" ] ; then
      while true
      do
         EZDisplayMessage "$OWNER_DIFFERENCE_TEXT"
         if [ "$STABALIZE_AUTO_ANSWER" != "" ] ; then
            OWNER_STABLE_ANSWER="$STABALIZE_AUTO_ANSWER"
            if [ "$OWNER_STABLE_ANSWER" = n ] ; then
               EZDisplayMessage "Not changing the owner of $TARGET_FILE_NAME" \
                                "from $TARGET_OWNER to $SOURCE_OWNER."
            fi
         elif [ "$STABLE_ANSWER" = y ] ; then
            OWNER_STABLE_ANSWER="$STABLE_ANSWER"
         else
            echo -n "Change the owner of $TARGET_FILE_NAME from $TARGET_OWNER "
            echo -n "to $SOURCE_OWNER <y,n$EXCL_OPT>? "
            read OWNER_STABLE_ANSWER
         fi
         if [ "$OWNER_STABLE_ANSWER" = y ] ; then
            chown $SOURCE_OWNER $TARGET_FILE_NAME
            if [ $? -eq 0 ] ; then
               EZDisplayMessage "$CHANGED_OWNER_TEXT"
            else
               EZDisplayMessage "$FAILED_CHANGING_OWNER_TEXT"
               EXIT_CODE=5
            fi
            break
         elif [ "$OWNER_STABLE_ANSWER" = n ] ; then
            EZDisplayMessage "$RETAINED_OWNER_TEXT"
            EXIT_CODE=5
            break
         elif [ "$OWNER_STABLE_ANSWER" = x ] ; then
            EZAddExcludedFile
            EXIT_CODE=4
            break
         fi
      done
   fi
}
#===========================================================================
# Convert ls type permissions into the appropriate chmod permissions.
#===========================================================================
function EZConvertToChmodPerms
{
  LS_PERMS="$1"
  TEMP_USER_PERMS="${LS_PERMS#?}"
  TEMP_GROUP_PERMS="${LS_PERMS#????}"
  USER_PERMS=`echo "${TEMP_USER_PERMS%??????}" | sed 's/-//g' | sed 's/s/sx/'`
  GROUP_PERMS=`echo "${TEMP_GROUP_PERMS%???}" | sed 's/-//g'`
  OTHER_PERMS=`echo "${LS_PERMS#???????}" | sed 's/-//g'`

  echo "u=$USER_PERMS,g=$GROUP_PERMS,o=$OTHER_PERMS"
}

#===========================================================================
# Ensure the source and target file perms match.
#===========================================================================
function EZStabalizePerms
{
   SOURCE_PERMS=`ls -l "$SOURCE_FILE_NAME" | awk '{print $1}'`
   TARGET_PERMS=`ls -l "$TARGET_FILE_NAME" | awk '{print $1}'`

   SOURCE_FILE_PERMS="${SOURCE_PERMS#?}"
   TARGET_FILE_PERMS="${TARGET_PERMS#?}"
   if [ "$SOURCE_FILE_PERMS" != "$TARGET_FILE_PERMS" ] ; then
      while true
      do
         EZDisplayMessage "$PERMS_DIFFERENCE_TEXT"
         if [ "$STABALIZE_AUTO_ANSWER" != "" ] ; then
            PERMS_STABLE_ANSWER="$STABALIZE_AUTO_ANSWER"
            if [ "$PERMS_STABLE_ANSWER" = n ] ; then
               EZDisplayMessage "Not changing the permissions of" \
                  "$TARGET_FILE_NAME from $TARGET_PERMS to $SOURCE_PERMS."
            fi
         elif [ "$STABLE_ANSWER" = y ] ; then
            PERMS_STABLE_ANSWER="$STABLE_ANSWER"
         else
            echo -n "Change the permissions of $TARGET_FILE_NAME from $TARGET_PERMS "
            echo -n "to $SOURCE_PERMS <y,n$EXCL_OPT>? "
            read PERMS_STABLE_ANSWER
         fi
         if [ "$PERMS_STABLE_ANSWER" = y ] ; then
            CHMOD_PERMS=`EZConvertToChmodPerms $SOURCE_PERMS`
            chmod $CHMOD_PERMS $TARGET_FILE_NAME
            if [ $? -eq 0 ] ; then
               EZDisplayMessage "$CHANGED_PERMS_TEXT"
            else
               EZDisplayMessage "$FAILED_CHANGING_PERMS_TEXT"
               EXIT_CODE=5
            fi
            break
         elif [ "$PERMS_STABLE_ANSWER" = n ] ; then
            EZDisplayMessage "$RETAINED_PERMS_TEXT"
            EXIT_CODE=5
            break
         elif [ "$PERMS_STABLE_ANSWER" = x ] ; then
            EZAddExcludedFile
            EXIT_CODE=4
            break
         fi
      done
   fi
}

#===========================================================================
# The Main.
#===========================================================================

if [ "$FILE_FIXER" != "" ] ; then
   WIN_TMPDIR="$(Path2Win.ksh $TMPDIR)"
   SOURCE_FILE_FIXED="$WIN_TMPDIR/${SOURCE_FILE_NAME##*/}"
   TARGET_FILE_FIXED="$WIN_TMPDIR/target_${TARGET_FILE_NAME##*/}"
   rm -f "$SOURCE_FILE_FIXED" "$TARGET_FILE_FIXED"
   eval $FILE_FIXER -SourceName  "$SOURCE_FILE_NAME" \
                    -TargetName  "$TARGET_FILE_NAME" \
                    -SourceFixed "$SOURCE_FILE_FIXED" \
                    -TargetFixed "$TARGET_FILE_FIXED"
   if [ $? -eq 0 ] ; then
      if [ -f "$SOURCE_FILE_FIXED" ] ; then
         SOURCE_FILE_NAME="$SOURCE_FILE_FIXED"
      fi
      if [ -f "$TARGET_FILE_FIXED" ] ; then
         TARGET_FILE_NAME="$TARGET_FILE_FIXED"
      fi
   fi
fi

# Perform any necessary file fixing.
SOURCE_FILE_NAME_ORIGINAL="$SOURCE_FILE_NAME"
SOURCE_FILE_NAME_FIX="${SOURCE_FILE_NAME}.fix"
if [ "$TRANSFORM_FUNCTION" != "" ] ; then
   if [ ! -f "$SOURCE_FILE_NAME" ] ; then
      echo "ERROR: Failed to find SourceFile $SOURCE_FILE_NAME."
      exit 5
   fi
   eval $TRANSFORM_FUNCTION -input  "$SOURCE_FILE_NAME" \
                            -output "$SOURCE_FILE_NAME_FIX"
   if [ ! -f "$SOURCE_FILE_NAME_FIX" ] ; then
      echo -n "ERROR: Tranformation function failed to create file "
      echo "$SOURCE_FILE_NAME_FIX."
      exit 5
   fi
   SOURCE_FILE_NAME="$SOURCE_FILE_NAME_FIX"
fi
SOURCE_FILE_NAME_FIX="${SOURCE_FILE_NAME}.fix"
if [ "$DOS2UNIX_FILES" = TRUE ] ; then
   if [ ! -f "$SOURCE_FILE_NAME" ] ; then
      echo "ERROR: Failed to find SourceFile $SOURCE_FILE_NAME."
      exit 5
   fi
   dos2unix.sh "$SOURCE_FILE_NAME" "$SOURCE_FILE_NAME_FIX"
   if [ "$SOURCE_FILE_NAME_ORIGINAL.fix.fix" = "$SOURCE_FILE_NAME_FIX" ] ; then
      mv "$SOURCE_FILE_NAME_FIX" "$SOURCE_FILE_NAME_ORIGINAL.fix"
      SOURCE_FILE_NAME_FIX="$SOURCE_FILE_NAME_ORIGINAL.fix"
   fi
   SOURCE_FILE_NAME="$SOURCE_FILE_NAME_FIX"
fi
if [ "$UNIX2DOS_FILES" = TRUE ] ; then
   if [ ! -f "$SOURCE_FILE_NAME" ] ; then
      echo "ERROR: Failed to find SourceFile $SOURCE_FILE_NAME."
      exit 5
   fi
   dos2unix.sh -reverse "$SOURCE_FILE_NAME" "$SOURCE_FILE_NAME_FIX"
   if [ "$SOURCE_FILE_NAME_ORIGINAL.fix.fix" = "$SOURCE_FILE_NAME_FIX" ] ; then
      mv "$SOURCE_FILE_NAME_FIX" "$SOURCE_FILE_NAME_ORIGINAL.fix"
      SOURCE_FILE_NAME_FIX="$SOURCE_FILE_NAME_ORIGINAL.fix"
   fi
   SOURCE_FILE_NAME="$SOURCE_FILE_NAME_FIX"
fi

typeset -l STABLE_ANSWER
if [ ! -f "$TARGET_FILE_NAME" ] ; then
   if [ ! -f "$SOURCE_FILE_NAME" ] ; then
      echo "ERROR: Failed to find SourceFile $SOURCE_FILE_NAME."
      exit 5
   fi
   EZCreateStableFile
else
   if [ -f "$SOURCE_FILE_NAME" ] ; then
      EZUpdateStableFile
   else
      if [ "$ALLOW_DESTROY" = TRUE ] ; then
         EZDestroyStableFile
      else
         echo "ERROR: Failed to find SourceFile $SOURCE_FILE_NAME."
         exit 5
      fi
   fi
fi

EZSrcStabalizeCleanup

FILE_STABALIZE_EXIT_CODE=$EXIT_CODE
if [ $FILE_STABALIZE_EXIT_CODE -eq 0 -o \
     $FILE_STABALIZE_EXIT_CODE -eq 1 -o \
     $FILE_STABALIZE_EXIT_CODE -eq 2 ] ; then
   if [ "$STABALIZE_GROUPS" = TRUE ] ; then
     EZStabalizeGroups
   fi
   if [ "$STABALIZE_OWNERS" = TRUE ] ; then
     EZStabalizeOwners
   fi
   if [ "$STABALIZE_PERMS" = TRUE ] ; then
     EZStabalizePerms
   fi
fi
exit $EXIT_CODE
