#===========================================================================
#  Filename: src_status.ksh
#    Author: Craig Schneider
# Orig Date: 03/29/11
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: List the status of source control files.
#===========================================================================

. $EZ_SOURCE_ENV

USER_NAME="$LOGNAME"
SHOW_USER=FALSE

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: src status FileName(s)... [-dryrun] [-quiet | -verbose]
              [-show_user] [-u UserName] [-v]

     -dryrun: Display the commands to be executed, without executing them.
 FileName(s): One or more file names to get the source control status of.
      -quiet: Suppress non critical messages about operations being performed.
  -show_user: Show the user name next to each checked out file name.
          -u: User name of the owner of the file locks (default $USER_NAME).
    -verbose: Display verbose messages about operations being performed.
          -v: Display verbose status output.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi

EXIT_CODE=0

EZ_DRYRUN=FALSE
EZ_VERBOSITY=
SRC_STATUS_OPTIONS=
VERBOSE_STATUS=FALSE

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -dryrun)
          EZ_DRYRUN=TRUE
          shift ;;
      -u) USER_NAME="$2"
          shift
          shift ;;
      -quiet|-verbose)
          EZ_VERBOSITY="${LOWERCASE_ARG#-}"
          shift ;;
      -show_user)
          SHOW_USER=TRUE
          shift ;;
      -v) VERBOSE_STATUS=TRUE
          shift ;;
      -*) if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
       *) SRC_STATUS_OPTIONS="$SRC_STATUS_OPTIONS $1"
          shift ;;
    esac
done

#===========================================================================
# Perform the requested source control operations.
#===========================================================================
if [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
   if [ "$SHOW_USER" = TRUE ] ; then
      SRC_STATUS_RESULTS_FILE="$TMPDIR/src_status.$$"
      EZInvokeCommand -out_err_file "$SRC_STATUS_RESULTS_FILE" \
                      $SRC_STATUS_COMMAND -u $SRC_STATUS_OPTIONS
      EXIT_CODE=$?
      typeset -L12 LOCK_OWNER_FMT

      # Open 6
      exec 6<< !
`cat "$SRC_STATUS_RESULTS_FILE" | tr '\\' '/' 2>&1 | cut -c 6,21- | \
                     egrep "^O|^K|tr: warning: an unescaped backslash"`
!
      while read -u6 LOCK_TYPE FILE_NAME
      do
         SRC_RELATIVE_ROOT="${EZ_CURRENT_ROOT#$EZ_DEV_ROOT}"
         SRC_FILE_URL="$EZ_SOURCE_CONTROL_ROOT/$SRC_RELATIVE_ROOT/$FILE_NAME"
         LOCK_OWNER=`svn info "$SRC_FILE_URL" | grep "Lock Owner: " | \
                                                cut -c13-`
         if [ "$USER_NAME" = anyone -o "$USER_NAME" = "$LOCK_OWNER" ] ; then
            LOCK_OWNER_FMT="$LOCK_OWNER"
            echo "$LOCK_OWNER_FMT $FILE_NAME"
         fi
      done

      #   Close 6
      exec 6<&-

      rm -f "$SRC_STATUS_RESULTS_FILE"
   else
      if [ "$VERBOSE_STATUS" = FALSE ] ; then
         EGREP_FLAGS="^\?       "
      else
         EGREP_FLAGS="^NO_SUCH_MATCH$"

         SRC_SCRIPTS_DIR=`EZGetSrcScriptsDir`
         if [ -f "$SRC_SCRIPTS_DIR/status_exclude.dat" ] ; then
            for EXCLUDE_PATTERN in `cat "$SRC_SCRIPTS_DIR/status_exclude.dat" \
                                        | tr -d '\r'`
            do
               EGREP_FLAGS="$EGREP_FLAGS|^\?       [ -~]*${EXCLUDE_PATTERN}\$"
            done
         fi
         FAMILY_STATUS_EXCLUDE="$EZ_CURRENT_ROOT/Env/src_status_exclude.dat"
         if [ -f "$FAMILY_STATUS_EXCLUDE" ] ; then
            for EXCLUDE_PATTERN in `cat "$FAMILY_STATUS_EXCLUDE" | tr -d '\r'`
            do
               EGREP_FLAGS="$EGREP_FLAGS|^\?       [ -~]*${EXCLUDE_PATTERN}\$"
            done
         fi
      fi
      if [ "$EZ_DRYRUN" = TRUE ] ; then
         echo "$SRC_STATUS_COMMAND $SRC_STATUS_OPTIONS | \
               tr -d '\r' | tr '\\' '/' | egrep -v \"$EGREP_FLAGS\""
      else
            EZInvokeCommand $SRC_STATUS_COMMAND $SRC_STATUS_OPTIONS | \
                            tr -d '\r' | tr '\\' '/' | egrep -v "$EGREP_FLAGS" 
            EXIT_CODE=$?
      fi
   fi
else
   echo "ERROR: Unsupported source control system $SOURCE_CONTROL_SYSTEM."
   EXIT_CODE=1
fi

exit $EXIT_CODE
